/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.bittorrent.common.util;

import java.util.Random;

/**
 * @author Manuel GOMEZ-ROSADO
 *
 */
public class BittorrentUtils {
	
    /**
     * A magic constant used to identify the Snark library in the clientid.
     * 
     * <pre>Taking Three as the subject to reason about--
     * A convenient number to state--
     * We add Seven, and Ten, and then multiply out
     * By One Thousand diminished by Eight.
     *
     * The result we proceed to divide, as you see,
     * By Nine Hundred and Ninety Two:
     * Then subtract Seventeen, and the answer must be
     * Exactly and perfectly true.</pre>
     */
    protected static final byte snark = (((3 + 7 + 10) * (1000 - 8)) / 992) - 17;

	public static byte[] generateNewSnarkPeerID(){
	    /** Our BitTorrent client id number, randomly assigned */
	    byte[] id = new byte[20];
        // Create a new ID and fill it with something random. First nine
        // zeros bytes, then three bytes filled with snark and then
        // sixteen random bytes.
        Random random = new Random();
        int i;
        for (i = 0; i < 9; i++) {
            id[i] = 0;
        }
        id[i++] = snark;
        id[i++] = snark;
        id[i++] = snark;
        while (i < 20) {
            id[i++] = (byte)random.nextInt(256);
        }
        
        return id;
	}
	
}
