/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.bittorrent.torrent.maker;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.klomp.snark.MetaInfo;
import org.klomp.snark.Storage;
import org.klomp.snark.Tracker;
import org.klomp.snark.cmd.ConsoleStorageReporter;
import org.openmis.bittorrent.common.exception.BittorrentException;

public class BittorrentMaker {

    /** The Java logger used to process our log events. */
    protected static final Logger log = Logger.getLogger("org.openmis.bittorrent.torrent.maker");
	
	public static BittorrentMaker maker = null;
	
	private Tracker tracker = null;
	
	public BittorrentMaker(Tracker tracker){
		this.tracker = tracker;
	}
	
	public static BittorrentMaker getBittorrentMaker(){
		return maker;
	}
	
	public static void registerBittorrentMaker(BittorrentMaker newmaker){
		maker = newmaker;
	}
	
	public void generateTorrentFromFile(MetaInfo meta, String torrentDestFile) throws BittorrentException{
		if(tracker != null){
	        // Code for writing out .torrent to disk
	        try {
	        	MetaInfo info = tracker.getMetaInfo(meta.getHexInfoHash());
	        	if(info == null)
	        		throw new BittorrentException("The MetaInfo is not register on the Tracker");
	        	
	        	byte[] torrentData = info.getTorrentData();
	        	
	        	File torrent = new File(torrentDestFile);
	        	torrent.createNewFile();
	  
	            FileOutputStream fos = new FileOutputStream(torrent);
	            fos.write(torrentData);
	            fos.close();
	            
	            log.log(Level.INFO, "Writing torrent to file " + torrent);
	            
            } catch (Exception ioe2) {
               throw new BittorrentException("Could not create torrent for '" + torrentDestFile + "'", ioe2);
            }
		}
	}
	
}
