/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.bittorrent.tracker;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.util.HashSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.klomp.snark.ConnectionAcceptor;
import org.klomp.snark.HttpAcceptor;
import org.klomp.snark.MetaInfo;
import org.klomp.snark.PeerAcceptor;
import org.klomp.snark.PeerCoordinator;
import org.klomp.snark.PeerID;
import org.klomp.snark.Storage;
import org.klomp.snark.StorageListener;
import org.klomp.snark.Tracker;
import org.klomp.snark.TrackerClient;
import org.klomp.snark.cmd.ConsoleStorageReporter;
import org.openmis.bittorrent.common.exception.BittorrentException;
import org.openmis.bittorrent.common.util.BittorrentUtils;
import org.openmis.bittorrent.torrent.maker.BittorrentMaker;

/**
 * @author Manuel GOMEZ-ROSADO
 *
 */
public class OpenMISTrackerBittorrent implements TrackerBittorrent {

    /** The Java logger used to process our log events. */
    protected static final Logger log = Logger.getLogger("org.openmis.bittorrent.tracker");
	
    public final static int DEFAULT_PORT = 6969;
    
	private ServerSocket serversocket = null;
	private String ip = null;
	private int port = -1;
	
	private StorageListener slistener = null;
	private HttpAcceptor httpacceptor = null;
	private Tracker tracker = null;
	
	public OpenMISTrackerBittorrent(String ip, int port) throws IOException{
		this.serversocket = new ServerSocket(port);
		this.port = port;
		this.ip = ip;
		
		this.slistener = new ConsoleStorageReporter();
		this.tracker = new Tracker(new HashSet());
		this.httpacceptor = new HttpAcceptor(tracker);
	}
	
	public Tracker getTracker(){
		return tracker;
	}
	
	public StorageListener getStorageListener(){
		return slistener;
	}
	
	public HttpAcceptor getHttpAcceptor(){
		return httpacceptor;
	}
	
	public String getIP(){
		return ip;
	}
	
	public int getPort(){
		return port;
	}
	
	public void addFileToTracker(File shareFile, String torrentDestFile) throws BittorrentException{
		try{
			Storage storage = new Storage(shareFile, "http://" + ip + ":" + port + "/announce", slistener);
	        storage.create();
	        MetaInfo meta = storage.getMetaInfo();
	            
	        if(meta != null){
	        	
	        	byte[] id = BittorrentUtils.generateNewSnarkPeerID();
	        	
	        	PeerCoordinator coordinator = new PeerCoordinator(id, meta, storage, null);
	            MetaInfo info = meta.reannounce("http://" + ip + ":" + port+ "/announce");

	            tracker.addMetaInfo(info);
	            tracker.addPeer(meta.getHexInfoHash(), new PeerID(id, InetAddress.getByName(ip), port));
	            
	            
	            if(BittorrentMaker.getBittorrentMaker() == null){
	            	BittorrentMaker.registerBittorrentMaker(new BittorrentMaker(tracker));
	            } 
	            BittorrentMaker.getBittorrentMaker().generateTorrentFromFile(meta, torrentDestFile);
	            
	            PeerAcceptor peeracceptor = new PeerAcceptor(coordinator);
	            ConnectionAcceptor acceptor = new ConnectionAcceptor(serversocket, httpacceptor, peeracceptor);
	            acceptor.start();

	            log.log(Level.INFO, "Torrent available on " + "http://" + ip + ":" + port + "/" + meta.getHexInfoHash() + ".torrent");
	            

	            TrackerClient trackerclient = new TrackerClient(meta, coordinator, port);
	            trackerclient.start();
	            coordinator.setTracker(trackerclient);
	            
	        }
	            
		}catch(Exception ex){
			throw new BittorrentException(ex);
		}
	}
	
	public void removeFileToTracker(File shareFile)throws BittorrentException{
		//------- TODO
	}
}
