/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.init;

import java.util.List;

import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.project.DistantProjectFactory;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.project.impl.DistantProjectFactoryFODB;
import org.openmis.tellus.project.impl.DistantProjectFactoryMemory;
import org.openmis.tellus.service.distant.OpenMISLoadFileDistantService;
import org.openmis.tellus.services.impl.AliveDistantService;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.servlet.OpenMISInit;

public abstract class SIGOpenMISInit implements OpenMISInit {


	public void preLoadingInit() throws ServiceException {
		try{
		    //------------------- register manager  ---------------------
			DistantServiceManager.registerManager(getDistantServiceManager());
			//------------------- load distant services -----------------
			//this.loadCommonDistantServices();
			
			if(getListDistantServiceToLoad() != null && getListDistantServiceToLoad().size()>0){
				List list = getListDistantServiceToLoad();
				for(int i=0;i<list.size();i++){
					DistantService service = (DistantService)list.get(i);
					DistantServiceManager.getManager().registerDistantService(
							service);
				}
			}
			
			//----------------- load DistantProject DB --------------------
			if(getDistantProjectFactory() != null){
				DistantProjectManager.registerDistantProjectManager(
					new DistantProjectManager(getDistantProjectFactory()));
			}else{
				DistantProjectManager.registerDistantProjectManager(
						new DistantProjectManager(new DistantProjectFactoryMemory()));
			}
			
		}catch(Exception ex){
			throw new ServiceException(ex);
		}
	}
	
	public void postLoadingInit() throws ServiceException {
	}
	
	
	public abstract DistantServiceManager getDistantServiceManager() throws CommException;
	public abstract List getListDistantServiceToLoad();
	public abstract DistantProjectFactory getDistantProjectFactory();

}
