/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.layers.impl;

import java.io.File;
import java.io.Serializable;

import org.openmis.tellus.project.DistantGPSManager;

public class DefaultDistantGPSManager implements DistantGPSManager , Serializable {
	public static final long serialVersionUID = 5521257935120563452L;

	private boolean viewGPSPosition = false;
	
	private File simulationFile = null;
	private File trackerFile = null;
	private File autoGenerationFile = null;
	
	public DefaultDistantGPSManager(){
		super();
	}
	
	public void setAutoGenerationFile(File autoGenerationFile) {
		this.autoGenerationFile = autoGenerationFile;
	}
	
	public void setSimulationFile(File simulationFile) {
		this.simulationFile = simulationFile;
	}
	
	public void setTrackerFile(File trackerFile) {
		this.trackerFile = trackerFile;
	}
	
	public void setViewGPSPosition(boolean viewGPSPosition) {
		this.viewGPSPosition = viewGPSPosition;
	}
	
	public boolean isViewGPSPosition() {
		return viewGPSPosition;
	}
	
	public boolean isSimulationFile(){
		if(simulationFile != null && simulationFile.exists())
			return true;
		return false;
	}
	
	public boolean isAutoGenerationFile(){
		if(autoGenerationFile != null)
			return true;
		return false;
	}
	
	public boolean isTrackerFile(){
		if(trackerFile != null)
			return true;
		return false;
	}
	
	public File getSimulationFile() {
		return simulationFile;
	}
	public String getSimulationFilePath() {
		if(simulationFile != null)
			return simulationFile.getPath();
		return null;
	}
	
	public File getAutoGenerationFile() {
		return autoGenerationFile;
	}
	public String getAutoGenerationFilePath() {
		if(autoGenerationFile != null)
			return autoGenerationFile.getPath();
		return null;
	}
	
	public File getTrackerFile() {
		return trackerFile;
	}
	public String getTrackerFilePath() {
		if(trackerFile != null)
			return trackerFile.getPath();
		return null;
	}
	
}
