/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.layers.impl;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;

import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.symbo.DistantLayerSymbology;

public class DefaultDistantLayer implements DistantLayer, Serializable {
	public static final long serialVersionUID = 5521257935120563452L;
	private File file = null;
	private String geoFileName = null;
	private String path = null;
	private String ext = null;
	private int position = -1;	
	
	private DistantLayerSymbology symbology = null;
	
	public DefaultDistantLayer(File file) throws IOException{
		
		if(file != null && file.exists()){
			this.file = file;
			this.path = file.getPath();
			this.geoFileName = file.getName();
			this.ext = (file.getName().substring(file.getName().indexOf(".")+1, file.getName().length())).toLowerCase();
		}else{
			throw new IOException("File not FOUND : "+file.getAbsolutePath());
		}
	}
	
	/**
	 * Without the extention
	 */
	public String getLayerName() {
		return geoFileName.substring(0, geoFileName.indexOf("."));
	}
	
	public String getDBFFileName() {
		return getLayerName()+".dbf";
	}
	
	public String getSHXFileName() {
		return getLayerName()+".shx";
	}
	
	/**
	 * All the fileName (with extension)
	 */
	public String getGeoFileName() {
		return geoFileName;
	}

	public String getPathGeoFile() {
		return path;
	}
	
	public String getExtension() {
		return ext;
	}
	
	public File getFile() {
		return file;
	}
	
	public int getPosition() {
		return position;
	}
	
	public void setPosition(int position) {
		this.position = position;
	}
	
	public void setSymbology(DistantLayerSymbology symbology) {
		this.symbology = symbology;
	}
	
	public DistantLayerSymbology getSymbology() {
		return symbology;
	}

}
