/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.project.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;

public class DefaultDistantProject implements DistantProject, Serializable {
	public static final long serialVersionUID = 5521257935120563452L;
	
	private Hashtable layersList = null;
	private String uniqueProjectName = null;
	private List normals = null;
	private List infoables = null;
	private List editables = null;
	private List classifiedLayersList = null;
	
	private DistantGPSManager GPSManager = null;
	
	public DefaultDistantProject(String uniqueProjectName){
		this.layersList = new Hashtable();
		this.uniqueProjectName = uniqueProjectName;
		this.normals = new ArrayList();
		this.infoables = new ArrayList();
		this.editables = new ArrayList();
		this.classifiedLayersList = new ArrayList();
	}
	
	public String getUniqueProjectName() {
		return uniqueProjectName;
	}
	
	public DistantLayer getDistantLayer(Class classLink) {
		List list = getInfoableLayers();
		for(int i=0;i<list.size();i++){
			DistantInfoableLayer layer = (DistantInfoableLayer)list.get(i);
			if(layer.getDistantInteraction() != null && layer.getDistantInteraction().getLinkedClasse().equals(classLink)){
				return layer;
			}
		}
		return null;
	}
	
	public DistantLayer getDistantLayer(String geoFileName) {
		return (DistantLayer)this.layersList.get(geoFileName);
	}
	
	public void addDistantLayer(DistantLayer layer){
		this.layersList.put(layer.getGeoFileName(), layer);
	}
	
	public void removeDistantLayer(DistantLayer layer){
		this.layersList.remove(layer.getGeoFileName());
	}
	
	public Hashtable getDistantLayersTable() {
		return layersList;
	}
	
	public List getClassifiedDistantLayersList(){
		classifiedLayersList.clear();
		Enumeration enume = getDistantLayersTable().elements();
		while(enume.hasMoreElements()){
			DistantLayer layer = (DistantLayer)enume.nextElement();
			classifiedLayersList.add(layer);	//to put all the object in the list but whithout any classement
		}
		
		int size = getDistantLayersTable().size();
		enume = getDistantLayersTable().elements();
		while(enume.hasMoreElements()){
			DistantLayer layer = (DistantLayer)enume.nextElement();
			if(layer.getPosition() >= 0 && layer.getPosition() < size){
				//classifiedLayersList.remove(layer.getPosition());
				classifiedLayersList.set(layer.getPosition(), layer);	//to put the layer at the right place
			}
		}
		
	
		return classifiedLayersList;
	}
	
	public List getEditableLayers() {
		editables.clear();
		Enumeration enume = getDistantLayersTable().elements();
		while(enume.hasMoreElements()){
			DistantLayer layer = (DistantLayer)enume.nextElement();
			if(layer instanceof DistantEditableLayer)
				editables.add(layer);
		}
		return editables;
	}
	
	public List getInfoableLayers() {
		infoables.clear();
		Enumeration enume = getDistantLayersTable().elements();
		while(enume.hasMoreElements()){
			DistantLayer layer = (DistantLayer)enume.nextElement();
			if(layer instanceof DistantInfoableLayer)
				infoables.add(layer);
		}
		return infoables;
	}
	
	public List getNormalLayers() {
		normals.clear();
		Enumeration enume = getDistantLayersTable().elements();
		while(enume.hasMoreElements()){
			DistantLayer layer = (DistantLayer)enume.nextElement();
			normals.add(layer);
		}
		return normals;
	}
	
	public void setGPSManager(DistantGPSManager manager) {
		GPSManager = manager;
	}
	
	public DistantGPSManager getGPSManager() {
		return GPSManager;
	}

}
