/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.project.impl;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectFactory;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FODBCollectionDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.query.soda.FODBSodaObjectSet;

public class DistantProjectFactoryFODB implements DistantProjectFactory{

	public DistantProjectFactoryFODB(){
		try {
			FastObjectDB db = FastObjectDBManager.getManager().getCurrentFODB();
			
			//JournalManager.getManager().registerJournalLogRenderer(new MyLogRenderer(this.getDBName(),"Interventions","NbrInterRecu"));
			if (!db.isCollectionExist(this.getDBName()))	{
				FODBCollectionDescriptor desc  = new FODBCollectionDescriptor(this.getDBName(), DefaultDistantProject.class);
				desc.setSynchronize(false);
				db.createCollection(desc);
				
				FODBStringIndexDescriptor id = new FODBStringIndexDescriptor("projectName", FODBStringIndexDescriptor.UNIQUE, "getUniqueProjectName()", 12, 50);
				db.addIndex(this.getDBName(), id);
				
			}
		      // always register notification listener because it's not persistent.
		     // ((SynchroFastObjectDB) db).registerSynchroFODBReturnListener(this.getDBName(), this.getSynchroListener(this.getDBName()));
		      
		      this.init();
		} catch (Throwable ex)  {
			LogManager.traceError(0,ex);
		}
	}
	
	private String getDBName(){
		return "distantPrj";
	}
	
	private String currentProjectName = null;
	public DistantProject getCurrentDistantProject() throws DatabaseException {
		if(currentProjectName != null)
			return getDistantProject(currentProjectName);
		return null;
	}
	
	public void setProjectAsCurrent(String nameprj) {
		if(nameprj != null){
			currentProjectName = nameprj;
		}
	}
	
	public void store(DistantProject i) throws DatabaseException{
		FastObjectDB db = FastObjectDBManager.getManager().getCurrentFODB();
		boolean test = this.exists(i.getUniqueProjectName());
		if(test == false){
		  db.add(this.getDBName(), i);
		}else{
		  db.replace(this.getDBName(), i);
		}

	}
  
	private boolean exists(String namePrj)  throws DatabaseException{
			Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
			q.constrain(this.getDBName());
			Query subq = q.descend("getUniqueProjectName()");
			subq.constrain(namePrj).equal();
			ObjectSet set = q.execute();
			if (set.size() > 0) {
				return true;
			}
			return false;
	}
  
  
	public DistantProject getDistantProject(String namePrj) throws DatabaseException	{
		Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
		q.constrain(this.getDBName());
		Query subq = q.descend("getUniqueProjectName()");
		subq.constrain(namePrj).equal();
		ObjectSet set = q.execute();  
		if (set.hasNext())	{
			return (DistantProject) set.next();
		}
		return null;  
	}
  
	public Array getDistantProjects() throws DatabaseException {
		Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
		q.constrain(this.getDBName());
		q.descend("getUniqueProjectName()");
		ObjectSet set = q.execute();   
		return ((FODBSodaObjectSet)set).getArrayFromSet();  
	}
	
	public void delete(String namePrj) throws DatabaseException {
		DistantProject prj = this.getDistantProject(namePrj);
		if (prj != null)	{
			FastObjectDBManager.getManager().getCurrentFODB().delete(this.getDBName(),prj);
		}
	}
	
	private void init(){
	}
	

}
