/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.project.parser;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.openmis.com.inter.agents.sax.data.CommMessage;
import org.openmis.com.inter.agents.sax.data.CommMessageSOAP;
import org.openmis.tellus.layers.DistantInteraction;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.impl.DefaultDistantEditableLayer;
import org.openmis.tellus.layers.impl.DefaultDistantInfoableLayer;
import org.openmis.tellus.layers.impl.DefaultDistantInteraction;
import org.openmis.tellus.layers.impl.DefaultDistantLayer;
import org.openmis.tellus.layers.symbo.DefaultDistantLayerSymbology;
import org.openmis.tellus.layers.symbo.DistantLayerSymbology;
import org.openmis.tellus.layers.symbo.Dynamisme;
import org.openmis.tellus.layers.symbo.IconDynamique;
import org.openmis.tellus.layers.symbo.IconSymbology;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.project.impl.DefaultDistantGPSManager;
import org.openmis.tellus.project.impl.DefaultDistantProject;
import org.openmis.tellus.utils.StringUtilities;
import org.openmobileis.common.util.log.LogManager;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class DistantProjectHandler extends DefaultHandler {

	private static final int TYPE_LAYER_NORMAL = 0;
	private static final int TYPE_LAYER_INFOABLE = 1;
	private static final int TYPE_LAYER_EDITABLE = 2;
	
	
	private ArrayList result;

	private boolean listprojectEntrance = false, projectEntrance = false, layerEntrance = false, nameEntrance = false, positionEntrance = false, interactionEntrance = false, geofieldlink = false, classlink = false;
	private boolean gpsmanagerEntrance = false, simulationFileEntrance = false, trackerEntrance = false, autogenerationEntrance = false;
	private boolean symboEntrance = false, colorEntrance = false, colorFillEntrance = false, sizeEntrance = false, iconSymEntrance = false, defaultEntrance = false, DynmismeEntrance = false, iconDynamicEntrance = false;
	private boolean buttonEditEntrance = false;
	
	private DistantProject project = null;
	private String directory = null;
	
	private File fileLayer = null;
	private int position = -1;
	
	private String geoFieldLink = null;
	private Class classLink = null;
	
	private DistantGPSManager GPSManager = null;
	
	private DistantLayerSymbology symbology = null;
	private IconDynamique iconDynamique = null;
	
	private int typeLayer = TYPE_LAYER_NORMAL;
	
	private String directoryFile = null;
	
	public DistantProjectHandler(String directoryFile) {
		result = new ArrayList();
		this.directoryFile = directoryFile;
	}

    public void startElement (String uri, String localName, String qName, Attributes atts) throws SAXException {
    	try{
    		if (qName.compareToIgnoreCase("listproject") == 0) {
    			listprojectEntrance = true;
    		}
    		
    		if(listprojectEntrance){
		    	if (qName.compareToIgnoreCase("project") == 0) {
		    		projectEntrance = true;
		    		
		    		for(int i=0;i<atts.getLength();i++){
		    			if(atts.getQName(i).compareToIgnoreCase("name") == 0){
		    				String name = atts.getValue(i);
		    				project = new DefaultDistantProject(name);
		    			}else if(atts.getQName(i).compareToIgnoreCase("directorypath") == 0){
		    				directory = atts.getValue(i);
		    				directory = directoryFile+File.separator+directory;
		    			}else if(atts.getQName(i).compareToIgnoreCase("current") == 0){
		    				String current = atts.getValue(i);
		    				if("true".equals(current)){
		    					DistantProjectManager.getManager().setProjectAsCurrent(project.getUniqueProjectName());
		    				}
		    			}
		    		}
		    	}
    		}
    		
	    	if(projectEntrance){
	    		if (qName.compareToIgnoreCase("distantlayer") == 0){
	    			layerEntrance = true;
	    			
	    			for(int i=0;i<atts.getLength();i++){
	        			if(atts.getQName(i).compareToIgnoreCase("type") == 0){
	        				String type = atts.getValue(i);
	        				if("infoable".equals(type)){
	        					typeLayer = TYPE_LAYER_INFOABLE;
	        				}else if("editable".equals(type)){
	        					typeLayer = TYPE_LAYER_EDITABLE;
	        				}else if("normal".equals(type)){
	        					typeLayer = TYPE_LAYER_NORMAL;
	        				}else{
	        					typeLayer = TYPE_LAYER_NORMAL;
	        				}
	        			}
	    			}  			
	    			
	        	}else if (layerEntrance){
	        		if (qName.compareToIgnoreCase("name") == 0){			
	        			nameEntrance = true;        			
	        		}else if (qName.compareToIgnoreCase("position") == 0){
	        			positionEntrance = true;
	        		}else if (qName.compareToIgnoreCase("interaction") == 0){
	        			interactionEntrance = true;
	        		}else if (qName.compareToIgnoreCase("symbology") == 0){
	        			symboEntrance = true;
	        			symbology = new DefaultDistantLayerSymbology();
	        		}
	        		
	        		if(interactionEntrance){
	        			if (qName.compareToIgnoreCase("geofieldlink") == 0){			
	        				geofieldlink = true;		        			
		        		}else if (qName.compareToIgnoreCase("classlink") == 0){
		        			classlink = true;      			
		        		}
	        		}
	        		
	        		if(symboEntrance){
		        		if (qName.compareToIgnoreCase("color") == 0){			
		        			colorEntrance = true;        			
		        		}else if (qName.compareToIgnoreCase("colorFill") == 0){
		        			colorFillEntrance = true;
		        			for(int i=0;i<atts.getLength();i++){
		            			if(atts.getQName(i).compareToIgnoreCase("filled") == 0){
		            				boolean filled = false;
		            				if("true".equals(atts.getValue(i)))
		            					 filled = true;
		            				symbology.setFilled(filled);
		            			}
		        			}
		        			
		        		}else if (qName.compareToIgnoreCase("size") == 0){
		        			sizeEntrance = true;
		        		}else if (qName.compareToIgnoreCase("iconsymbology") == 0){
		        			iconSymEntrance = true;
		        			symbology.setIconSymbology(new IconSymbology());
		        		}
		        		
		        		
		        		if(iconSymEntrance){
			        		if (qName.compareToIgnoreCase("default") == 0){			
			        			defaultEntrance = true;        			
			        		}else if (qName.compareToIgnoreCase("dynamisme") == 0){
			        			DynmismeEntrance = true;        
			        			for(int i=0;i<atts.getLength();i++){
			            			if(atts.getQName(i).compareToIgnoreCase("fieldreference") == 0){	
			            				symbology.getIconSymbology().setDynamisme(new Dynamisme(atts.getValue(i)));
			            			}
			        			}
			        		}
			        		
			        		if(DynmismeEntrance){
			        			
			        			
				        		if (qName.compareToIgnoreCase("icondynamic") == 0){			
				        			iconDynamicEntrance = true;
				        			
				        			String value = null, iconName = null;
				        			for(int i=0;i<atts.getLength();i++){
				            			if(atts.getQName(i).compareToIgnoreCase("value") == 0){	
				            				value = atts.getValue(i);
				            			}
				            			if(atts.getQName(i).compareToIgnoreCase("iconName") == 0){	
				            				iconName = atts.getValue(i);
				            			}
				        			}
				        			if(value != null && iconName != null)
				        				iconDynamique = new IconDynamique(value, iconName);
				        				//symbology.getIconSymbology().getDynamisme().addIconDynamic(value, iconName);
				        		}
				        		
				        		if(iconDynamicEntrance){
					        		if (qName.compareToIgnoreCase("buttonedit") == 0){			
					        			buttonEditEntrance = true;
					        			
					        			if(iconDynamique!= null){
					        				iconDynamique.setButtonEdit(true);
					        				
						        			for(int i=0;i<atts.getLength();i++){
						            			if(atts.getQName(i).compareToIgnoreCase("selectable") == 0){
						            				if("true".equals(atts.getValue(i))){
						            					iconDynamique.setSelectableButton(true);
						            				}else{
						            					iconDynamique.setSelectableButton(false);
						            				}	
						            			}
						            			if(atts.getQName(i).compareToIgnoreCase("actionable") == 0){
						            				if("true".equals(atts.getValue(i))){
						            					iconDynamique.setActionableButton(true);
						            				}else{
						            					iconDynamique.setActionableButton(false);
						            				}
						            				
						            			}
						            			if(atts.getQName(i).compareToIgnoreCase("iconButtonName") == 0){	
						            				iconDynamique.setIconButtonName(atts.getValue(i));
						            			}
						            			if(atts.getQName(i).compareToIgnoreCase("iconButtonSelectName") == 0){	
						            				iconDynamique.setIconButtonSelectName(atts.getValue(i));
						            			}
						        			}
					        			}	
					        		}
					        		
					        		if(iconDynamique != null)
					        			symbology.getIconSymbology().getDynamisme().addIconDynamic(iconDynamique.getValue(), iconDynamique);
				        		}
			        		}
		        		}
	        		}
	    		}
	    	}
	    	
	    	if(qName.compareToIgnoreCase("gpsmanager") == 0){
	    		gpsmanagerEntrance = true;
    			
    			for(int i=0;i<atts.getLength();i++){
        			if(atts.getQName(i).compareToIgnoreCase("viewgpsPosition") == 0){
        				boolean view = false;
        				if("true".equals(atts.getValue(i))){
        					view = true;
        				}
        				
        				GPSManager = new DefaultDistantGPSManager();
        				GPSManager.setViewGPSPosition(view);
        			}
    			} 
	    	}else if(gpsmanagerEntrance){
	    		if (qName.compareToIgnoreCase("simulationfile") == 0){			
        			simulationFileEntrance = true;
	    		}else if (qName.compareToIgnoreCase("tracker") == 0){
        			trackerEntrance = true;  
        			for(int i=0;i<atts.getLength();i++){
            			if(atts.getQName(i).compareToIgnoreCase("file") == 0){
            				File file = new File(directory+File.separator+atts.getValue(i));
            				GPSManager.setTrackerFile(file);
            			}
        			}
        			
        		}
	    		
	    		if(trackerEntrance){
	    			if (qName.compareToIgnoreCase("autogenerationshp") == 0){			
        				autogenerationEntrance = true;
	    			}
	    		}
	    	}
    	}catch(Exception ex){
    		throw new SAXException(ex);
    	}
    	
    }
    
    public void characters (char ch[], int start, int length)throws SAXException {
    	if(listprojectEntrance)
    		if(projectEntrance){
		    	if(layerEntrance){
		    		if(nameEntrance){
		    			try{
		    				fileLayer = new File(directory+File.separator+String.copyValueOf(ch,start,length));
		    			}catch(Exception ex){
		    				throw new SAXException(ex);
		    			}
		    		}
		    		
		    		if(positionEntrance){
		    			position = Integer.parseInt(String.copyValueOf(ch,start,length));
		    		}
		    		if(interactionEntrance){
		    			if(geofieldlink){
		    				geoFieldLink = String.copyValueOf(ch,start,length);
		    			}
		    			if(classlink){
		    				try {
		    					//System.out.println(String.copyValueOf(ch,start,length));
								classLink = Class.forName(String.copyValueOf(ch,start,length));
							} catch (ClassNotFoundException e) {
								LogManager.traceInfo(0, "Class not found for OWSIGProject parsing: "+e.getMessage());
							}
		    			}
		    		}
		    		
		    		
		    		if(symboEntrance){
		    			if(colorEntrance || colorFillEntrance){
		    				String coTemp = String.copyValueOf(ch,start,length);
		    				String args[] = StringUtilities.split(coTemp, ",");
		    				if(args != null && args.length == 3){
		    					Color color = new Color(Integer.parseInt(args[0]), Integer.parseInt(args[1]), Integer.parseInt(args[2]));
		    					if(colorEntrance)
		    						symbology.setColor(color);
		    					else if(colorFillEntrance)
		    						symbology.setColorFilled(color);
		    				}
		    			}
		    			
		    			if(sizeEntrance){
		    				symbology.setSize(Integer.parseInt(String.copyValueOf(ch,start,length)));
		    			}
		    			
		    			if(iconSymEntrance){
		    				if(defaultEntrance){
		    					symbology.getIconSymbology().setDefaultIconName(String.copyValueOf(ch,start,length));
		    				}
		    			}
		    		}
		    	}
		    	
		    	if(gpsmanagerEntrance){
		    		if(simulationFileEntrance){
		    			File file = new File(directory+File.separator+String.copyValueOf(ch,start,length));
		    			GPSManager.setSimulationFile(file);
		    		}
		    		
		    		if(trackerEntrance){
		    			if(autogenerationEntrance){
		    				File file = new File(directory+File.separator+String.copyValueOf(ch,start,length));
		    				GPSManager.setAutoGenerationFile(file);
		    			}
		    		}
		    	}
    		}
    }

    public void endElement (String uri, String localName, String qName)throws SAXException{
    	try{
    		if (qName.compareToIgnoreCase("listproject") == 0) {
    			listprojectEntrance = false;
    		}
    		
	    	if (qName.compareToIgnoreCase("project") == 0) {
	    		result.add(project);
	    		DistantProjectManager.getManager().storeDistantProject(project);
	    		projectEntrance = false;
	    	}
	    	if (qName.compareToIgnoreCase("distantlayer") == 0) {
	    		
	    		DistantLayer layer = null;
	    		if(typeLayer == TYPE_LAYER_NORMAL){
	    			layer = new DefaultDistantLayer(fileLayer);
	    			
	    		}else if(typeLayer == TYPE_LAYER_INFOABLE){
	    			if(geoFieldLink != null && classLink != null)
	    				layer = new DefaultDistantInfoableLayer(fileLayer, new DefaultDistantInteraction(geoFieldLink, classLink));
	    			else
	    				layer = new DefaultDistantInfoableLayer(fileLayer);
	    			
	    		}else if(typeLayer == TYPE_LAYER_EDITABLE ){
	    			if(geoFieldLink != null && classLink != null)
	    				layer = new DefaultDistantEditableLayer(fileLayer, new DefaultDistantInteraction(geoFieldLink, classLink));
	    			else
	    				layer = new DefaultDistantEditableLayer(fileLayer);
	    			
	    		}else{
	    			layer = new DefaultDistantLayer(fileLayer);
	    		}
	    		
	    		layer.setPosition(position);
	    		if(symbology != null)
	    			layer.setSymbology(symbology);
	    		project.addDistantLayer(layer);
	    		
	    		layerEntrance = false;
	    		classLink = null;
	    		geoFieldLink = null;
	    		position = -1;
	    		symbology = null;
	    	}
	    	if(qName.compareToIgnoreCase("name") == 0) {
	    		nameEntrance = false;
	    	}
	    	if(qName.compareToIgnoreCase("position") == 0) {
	    		positionEntrance = false;
	    	}
	    	if(qName.compareToIgnoreCase("interaction") == 0) {
	    		interactionEntrance = false;
	    	}
			if (qName.compareToIgnoreCase("geofieldlink") == 0){			
				geofieldlink = false;		        			
    		}
			if (qName.compareToIgnoreCase("classlink") == 0){
    			classlink = false;
    		}
	    	if(qName.compareToIgnoreCase("symbology") == 0) {	
	    		symboEntrance = false;
	    	}
			if (qName.compareToIgnoreCase("color") == 0){			
				colorEntrance = false;		        			
    		}
			if (qName.compareToIgnoreCase("colorfill") == 0){
    			colorFillEntrance = false;
    		}
			if (qName.compareToIgnoreCase("size") == 0){
    			sizeEntrance = false;
    		}
			if (qName.compareToIgnoreCase("iconsymbology") == 0){
    			iconSymEntrance = false;
    		}
			if (qName.compareToIgnoreCase("default") == 0){
    			defaultEntrance = false;
    		}
			if (qName.compareToIgnoreCase("dynamisme") == 0){
    			DynmismeEntrance = false;
    		}
			if (qName.compareToIgnoreCase("icondynamic") == 0){
    			iconDynamicEntrance = false;
    			iconDynamique = null;
    		}
			if (qName.compareToIgnoreCase("buttonEdit") == 0){
    			buttonEditEntrance = false;
    		}
			
			
			if (qName.compareToIgnoreCase("gpsmanager") == 0){
				project.setGPSManager(GPSManager);
    			gpsmanagerEntrance = false;
    			GPSManager = null;
    		}
			if (qName.compareToIgnoreCase("simulationfile") == 0){
    			simulationFileEntrance = false;
			}	
			if (qName.compareToIgnoreCase("tracker") == 0){
    			trackerEntrance = false;
    		}
			if (qName.compareToIgnoreCase("autogenerationshp") == 0){
    			autogenerationEntrance = false;
    		}
	    	
    	}catch(Exception ex){
    		throw new SAXException(ex);
    	}
    }

	public ArrayList getArray() {
		return result;
	}	
	
}
