/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.project.parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import org.openmis.com.inter.agents.sax.data.CommMessage;
import org.openmis.com.inter.agents.sax.data.CommParameter;
import org.openmis.com.inter.agents.sax.parser.CommMessageObjectExtractor;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.symbo.IconDynamique;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.project.impl.DistantProjectFactoryMemory;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;


public class TestParser {
	
	public TestParser() {
		
		//String path = getClass().getResource("TEST_OWSIGProjects.xml").getFile();
		File file = new File("test/project/TEST_OWSIGProjects.xml");	
		try {
			
			DistantProjectManager.registerDistantProjectManager(new DistantProjectManager(new DistantProjectFactoryMemory()));
			
			InputStream in = new FileInputStream(file);
			List array = DistantProjectObjectExtractor.getObject(in, System.getProperty("user.dir")+"/ressources_test/project");
			
			System.out.println();
			System.out.println("---> CURRENT PROJECT : "+DistantProjectManager.getManager().getCurrentDistantProject().getUniqueProjectName());
			System.out.println();
			
			for(int i=0;i<array.size();i++){
				DistantProject project = (DistantProject)array.get(i);
				System.out.println("PROJECT : "+project.getUniqueProjectName());
				
				List layers = project.getClassifiedDistantLayersList();
				for(int j=0;j<layers.size();j++){
					DistantLayer layer = (DistantLayer)layers.get(j);
					
					if(layer instanceof DistantLayer){			
						System.out.println("\t\t FILE : "+layer.getFile());
						System.out.println("\t\t GEOFILENAME : "+layer.getGeoFileName());
						System.out.println("\t\t PATHGEOFILE : "+layer.getPathGeoFile());
						System.out.println("\t\t LAYERNAME : "+layer.getLayerName());
						System.out.println("\t\t EXTENSION : "+layer.getExtension());
						System.out.println("\t\t POSITION : "+layer.getPosition());
						if(layer.getSymbology() != null){
							System.out.println("\t\t SYMBOLOGY FOUND");
							System.out.println("\t\t COLOR : "+layer.getSymbology().getColor());
							System.out.println("\t\t COLOR FILLED : "+layer.getSymbology().getColorFilled());
							System.out.println("\t\t IS FILLED : "+layer.getSymbology().isFilled());
							System.out.println("\t\t SIZE : "+layer.getSymbology().getSize());
							
							if(layer.getSymbology().getIconSymbology() != null){
								System.out.println("\t\t DEFAULT ICON NAME : "+layer.getSymbology().getIconSymbology().getDefaultIconName());
								if(layer.getSymbology().getIconSymbology().getDynamisme() != null){
									System.out.println("\t\t FIELD DYNAMIC REFERENCE : "+layer.getSymbology().getIconSymbology().getDynamisme().getFieldReference());
									
									Enumeration enume = layer.getSymbology().getIconSymbology().getDynamisme().getTableIconDynamic().keys();
									while(enume.hasMoreElements()){
										String value = (String)enume.nextElement();
										IconDynamique iconDynamique = (IconDynamique)layer.getSymbology().getIconSymbology().getDynamisme().getTableIconDynamic().get(value);
										System.out.println("\t\t ICONDYNAMIC VALUE : "+value+" ICON_NAME : "+iconDynamique.getIconName());
										if(iconDynamique.isButtonEdit())
											System.out.println("\t\t BUTTON_EDIT Select : "+iconDynamique.isSelectableButton()+"Action : "+iconDynamique.isActionableButton()+" ICON_BUT_NAME : "+iconDynamique.getIconButtonName()+" ICON_BUT_SELECT_NAME : "+iconDynamique.getIconButtonSelectName());
										System.out.println();
									}
								}
							}
						}else{
							System.out.println("\t\t SYMBOLOGY NOT FOUND");
						}
						System.out.println("\t\t THE LAYER IS NORMAL");
					}
					
					if(layer instanceof DistantInfoableLayer){
						DistantInfoableLayer layerInf = (DistantInfoableLayer)layer;
						if(layerInf.getDistantInteraction() == null){
							System.out.println("\t\t INTERACTION : NAN");
						}else{
							System.out.println("\t\t INTERACTION : OUI");
							System.out.println("\t\t FIELD LINKER : "+layerInf.getDistantInteraction().getGeoFieldLinker());
							System.out.println("\t\t CLASS LINKER : "+layerInf.getDistantInteraction().getLinkedClasse());
						}
						System.out.println("\t\t THE LAYER IS INFOABLE");
					}
					if(layer instanceof DistantEditableLayer){
						DistantEditableLayer layerEd = (DistantEditableLayer)layer;
						System.out.println("\t\t THE LAYER IS EDITABLE");
					}
					System.out.println();
				}
				
				if(project.getGPSManager() != null){
					System.out.println("\t GPSMANAGER IS NOT NULL");
					DistantGPSManager gps = project.getGPSManager();
					System.out.println("\t ViewGPSPosition : "+gps.isViewGPSPosition());
					System.out.println("\t SimulationFile : "+gps.getSimulationFile());
					System.out.println("\t TrackerFile : "+gps.getTrackerFile());
					System.out.println("\t AutoGenerationSHP : "+gps.getAutoGenerationFile());
					
				}else{
					System.out.println("\t GPSMANAGER IS NULL");
				}	
				System.out.println();
				System.out.println();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String args[]){
		new TestParser();
	}
}
