/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.gvsig.gui;

import java.awt.Cursor;
import java.awt.Frame;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.gvsig.gui.OWMapControl;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGInformation;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmobileis.common.util.exception.DatabaseException;

import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.driver.BoundedShapes;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerFileVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;

public class DistantInfoPointToolListener implements PointToolListener {
	
	public static final String ID_FIELD_NAME = "ID";

	private static Logger logger = Logger.getLogger(DistantInfoPointToolListener.class);

	private OWMapControl mapCtrl;
	private Frame parentFrame;
	
	public DistantInfoPointToolListener(Frame infoFrame) {
		parentFrame = infoFrame;
	}

	
	private void addPair(ArrayList al, String a, String b) { al.add(a); al.add(b); }

	public void point(PointEvent event) {
		try{
			if (mapCtrl.isProcessing()) {
				return;
			}
			
			if (event.getEvent().getID() != MouseEvent.MOUSE_PRESSED) {
				return;
			}
			
			int x = event.getPoint().x;
			int y = event.getPoint().y;
			
			Point2D wp = mapCtrl.getViewPort().toMapPoint(x, y);
			/*FLayer[] info_lyrs = mapCtrl.getInfoableLayers(true);
			
			if ((info_lyrs == null) || (info_lyrs.length != 1)) {
				MessageDialog._showModalErrorMessageDialog(
						"OpenMIS info",
						"Select one vector layer",
						parentFrame
						);
				return;
			}*/
			int[] intersect_indices = null;
			FLyrVect lyr = null;
			ResultPositionPoint result = testAllProjectLayersForPoint(wp);
			if(result != null){
				intersect_indices = result.getIntersect_indices();
				lyr = result.getFLyrVect();
			}
	
			if (intersect_indices == null) {
				/*MessageDialog._showModalErrorMessageDialog(
						"OpenMIS info", "No elements found",
						parentFrame
						);*/
				return;
			}
	
			int index = intersect_indices[0];
			
			String the_geofn = "[NONE]";
			String the_id = "[NONE]";
			
			// ------------ check ID field ------------
			int idFieldIndex = 0;
			try {
				ReadableVectorial rv = lyr.getSource();
				SelectableDataSource sds = rv.getRecordset();
				idFieldIndex = sds.getFieldIndexByName(ID_FIELD_NAME);
				
				logger.debug("CommInfoPointListener, idFieldIndex = " + idFieldIndex);
				
				Value v = sds.getRow(index)[idFieldIndex];
				the_id = v.toString();
				
				logger.debug("CommInfoPointListener, the_id = " + the_id);
	
			} catch (Exception ex) {
				logger.error("Did not find field called " + ID_FIELD_NAME);
				MessageDialog._showModalErrorMessageDialog(
						"OpenMIS info", "Field not found: " + ID_FIELD_NAME,
						parentFrame
						);
				return;
			}
			// ----------------------------------------
	
			if (lyr instanceof FLayerFileVectorial) {
				try {
					//File aux = new File(((FLayerFileVectorial) lyr).getFilePath());
					the_geofn = result.getDistantLayer().getGeoFileName();
					
					logger.debug("CommInfoPointListener, the_geofn = " + the_geofn);
				} catch (Exception ex) {
					logger.error("While getting file name: " + ex.getMessage());
				}
			} else {
				MessageDialog._showModalErrorMessageDialog(
						"OpenMIS info", "Bad layer type (?)",
						parentFrame
						);
				return;
			}
			
			ServiceComGvSIGInformation service = (ServiceComGvSIGInformation)DistantServiceManager.getManager().getDistantServiceByURI(
					ServiceComGvSIGInformation.DISTANT_URI);
			
			if(service != null){
				service.setGeoFileName(the_geofn);
				service.setID(the_id.trim());
				try {
					DistantServiceManager.getManager().runDistantService(service.getDistantServiceUri());
				} catch (CommException e) {
					e.printStackTrace();
				}
			}
		}catch(Exception ex){
			ex.printStackTrace();
		}

	}
	
	private int[] getIntersectsIndicesForLayer(FLyrVect lyr, Point2D wp){
		if(lyr != null){
			double tol = mapCtrl.getViewPort().toMapDistance(15);
			
			Rectangle2D click_rect = new Rectangle2D.Double(
					wp.getX() - 0.5 * tol,
					wp.getY() - 0.5 * tol,
					tol, tol);
			VectorialDriver vd = lyr.getSource().getDriver();
		
			try {
				return ((BoundedShapes) vd).getIntersectingIndices(click_rect, 1);
			} catch (IOException e) {
				logger.error("While getting intersect_indices: " + e.getMessage());
			}
		}
		return null;
	}

	
	private ResultPositionPoint testAllProjectLayersForPoint(Point2D wp) throws DatabaseException{
		FLyrVect lyr = null;
		int[] intersect_indices = null;

		DistantProject project = DistantProjectManager.getManager().getCurrentDistantProject();
		List layers = project.getEditableLayers();
		for(int i=0;i<layers.size();i++){
			DistantEditableLayer layer = (DistantEditableLayer)layers.get(i);
			lyr = (FLyrVect)mapCtrl.getLayerWithGeoFileName(layer.getGeoFileName());
			intersect_indices = getIntersectsIndicesForLayer(lyr, wp);
			if(intersect_indices == null){
				/*lyr = (FLyrVect)mapCtrl.getLayerWithGeoFileName(layer.getEditableGeoFileName());
				intersect_indices = getIntersectsIndicesForLayer(lyr, wp);
				if(intersect_indices != null)
					return new ResultPositionPoint(intersect_indices, lyr, layer);*/
			}else{
				return new ResultPositionPoint(intersect_indices, lyr, layer);
			}
		}
		
		layers = project.getInfoableLayers();
		for(int i=0;i<layers.size();i++){
			DistantInfoableLayer layer = (DistantInfoableLayer)layers.get(i);
			lyr = (FLyrVect)mapCtrl.getLayerWithGeoFileName(layer.getGeoFileName());
			intersect_indices = getIntersectsIndicesForLayer(lyr, wp);
			if(intersect_indices != null)
				return new ResultPositionPoint(intersect_indices, lyr, layer);
		}
		return null;
	}

	public Cursor getCursor() {
		return Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	}

	public boolean cancelDrawing() {
		return false;
	}

	public void setMapControl(MapControl mc) {
		mapCtrl = (OWMapControl)mc;
	}
	

	private class ResultPositionPoint {
		private int[] intersect_indices = null;
		private FLyrVect lyr = null;
		private DistantLayer layer = null;
		
		public ResultPositionPoint(int[] intersect_indices, FLyrVect lyr, DistantLayer layer){
			this.intersect_indices = intersect_indices;
			this.lyr = lyr;
			this.layer = layer;
		}
		
		public int[] getIntersect_indices() {
			return intersect_indices;
		}
		
		public FLyrVect getFLyrVect() {
			return lyr;
		}
		
		public DistantLayer getDistantLayer() {
			return layer;
		}
	}
}
