/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.gvsig.gui;

import java.io.File;

import org.dinopolis.gpstool.gpsinput.GPSException;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.location.GPSManager;
import org.gvsig.mobile.location.tracklog.RawDataManager;
import org.gvsig.mobile.location.tracklog.TracklogManager;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.gvsig.gps.OpenMISGPSPositionListener;
import org.openmis.tellus.gvsig.gui.AbstOpenMISMainWindow;
import org.openmis.tellus.gvsig.gui.InitToolBar;
import org.openmis.tellus.gvsig.gui.OWMapControl;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGForeground;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.synchro.common.data.EventFileModification;
import org.openmobileis.common.util.PropertiesManager;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.gpe.layer.FlyrGpeVect;
import es.prodevelop.gvsig.mobile.gpe.writer.ExportGpeWriter;
import es.prodevelop.gvsig.mobile.gui.dialog.AcceptCancelDialog;
import es.prodevelop.gvsig.mobile.gui.tool.TBActionButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBComboButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBSelectableButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBSwitcherButton;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBar;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBarGroupButton;
import es.prodevelop.gvsig.mobile.gui.tool.behavior.PointBehavior;

public class OpenMISInitToolBar implements InitToolBar{
	private static OpenMISInitToolBar instance = null;
	
	private ToolBar openmisBar;
	public TBActionButton gotoOpenMIS;
	public TBSelectableButton infoOpenMIS;
	public TBComboButton editComboButton;
	
	public TBSwitcherButton httpGpsposButton;	//boutton qui envoi les positions GPS  GoogleMap
	
	public TBSwitcherButton gpsConnectToolButton, gpsTracklogToolButoon;
	
	private AbstOpenMISMainWindow window;
	
	public OpenMISInitToolBar(AbstOpenMISMainWindow window){
		this.window = window;
	}
	
	public static OpenMISInitToolBar getInstance(){
		return instance;
	}

	public static void registerInitToolBar(OpenMISInitToolBar newinstance){
		instance = newinstance;
	}
	
	public AbstOpenMISMainWindow getMainWindow(){
		return this.window;
	}
	
	public void initToolsCanvas(int positions[]){
		
		
		ToolBarGroupButton openmisGroupButton =	new ToolBarGroupButton("group_info_floating_tab.png","group_info_selected.png", window, positions[0]); 
		openmisBar = new ToolBar(window, openmisGroupButton, positions[0]);
		// Addition of the ToolBar to the toolbars array of MainWindow
		window.addToolBar(openmisBar);
		
		/*
		 * Creation of listeners
		 */
		DistantInfoPointToolListener iplOW = new DistantInfoPointToolListener(window);
		iplOW.setMapControl(window.getMapControl());
		PointBehavior pbOW = new PointBehavior(iplOW); // Info by point
		
		// GOTO OpenMIS application
		gotoOpenMIS = new TBActionButton(openmisBar,	"zoom_next_black.png", "zoom_next_orange.png", "Go to OpenMIS", window); 
		
		
		// Info OpenMIS application
		infoOpenMIS = new TBSelectableButton(openmisBar, "info_black.png", "info_orange.png", "Info OpenMIS", pbOW, window);
		
		
		// Edit OpenMIS application
		//editOpenMIS = new TBSelectableButton(openmisBar,"edition_geometry_insert_point_black.png", "edition_geometry_insert_point_orange.png", GvsigMobileGUIConstants.TB_OPENMIS_EDIT, pbOWEd, this); 

		
		// Connect
		gpsConnectToolButton = new TBSwitcherButton(openmisBar,"gps_activity_black.png", "gps_activity_orange.png", GvsigMobileGUIConstants.TB_GPS_CONNECT, window);	
		//gpsTracklogToolButoon = new TBSwitcherButton(navBar,"finish_flag_black.png", "finish_flag_orange.png",GvsigMobileGUIConstants.TB_GPS_START_TRACKLOG, window);
		
		httpGpsposButton = new TBSwitcherButton(openmisBar, "finish_flag_black.png", "finish_flag_orange.png", "Envoi coordones GPS distant", window);
		
		openmisBar.reorder(window.getWidth());
		window.getMapControl().addGroupButton(openmisGroupButton, false);
	}
	
	public void doGoToOpenMIS(){
		try{
			DistantServiceManager.getManager().runDistantService(ServiceComGvSIGForeground.DISTANT_URI);
		}catch(Exception ex){
			ex.printStackTrace();
		}	
	}
	
	
	public void doTrackingLog(){
		gpsTracklogToolButoon.reverse();
		window.refreshSwitchableTool(gpsTracklogToolButoon);
		window.getMapControl().switchTracklog();
	}
	
	
	private OpenMISGPSPositionListener positionementList = null;
	
	public OpenMISGPSPositionListener getOpenMISGPSPositionListener(){
		if(positionementList == null){
			String url = PropertiesManager.getManager().getProperty("openmis.googlemap.url.update");
			String id = PropertiesManager.getManager().getProperty("openmis.googlemap.id.technicien");
			
			if( url != null && url.length() > 0 && id != null && id.length() > 0){
				int idTech = Integer.parseInt(id);
				positionementList = new OpenMISGPSPositionListener(url, idTech);	
			}
		}
		return positionementList; 
	}
	
	public void doSendGPSPositionnement(){
		
		if(httpGpsposButton != null){		
			httpGpsposButton.reverse();
			window.refreshSwitchableTool(httpGpsposButton);
			
			if(httpGpsposButton.isOn())
				GPSManager.getGPSManager().addGPSRelevantEventsListener(getOpenMISGPSPositionListener());		
			else
				GPSManager.getGPSManager().removeGPSRelevantEventsListener(getOpenMISGPSPositionListener());
		
		}
	
	}
	
	public void doGPSConnection(){

		gpsConnectToolButton.reverse();
		window.refreshSwitchableTool(gpsConnectToolButton);

		boolean is_conn = GPSManager.getGPSManager().isConnected();
		
		if (is_conn) {
			GPSManager.getGPSManager().disconnectFromGPS();
			RawDataManager.closeRawData(); /////
			
			window.getMapControl().getMapContext().getAcetateLayer().setGPSTrailerVisible(false);
			window.getMapControl().getMapContext().getAcetateLayer().clearGPSTrailer();
			
			//----------------  ACTIVATION GPS ------------------------
			window.getMapControl().getMapCursorSupport().switchViewGPSPosition(); // PUT TO FALSE TO ERASE THE ICON GPS'S VIEW
			//----------------- ACTIVATION GPS ------------------------
			
			
			window.getMapControl().repaint();
			
			//----------------  ACTIVATION GPS ------------------------
			try {
				
				DistantProject project = DistantProjectManager.getManager().getCurrentDistantProject();
				if(project.getGPSManager() != null)
				{
					DistantGPSManager gps = project.getGPSManager();
					if(gps.isAutoGenerationFile()){		//AUTO GENERATION FILE REQUESTED
						File track = gps.getTrackerFile();
						
						FlyrGpeVect fl = FlyrGpeVect.createGpeLayer(track, gps.getAutoGenerationFile().getName(), window.getMapControl().project.getMapContext());
						FLyrVect vectLayer = (FLyrVect)fl;
		
						if (vectLayer != null){
							ExportGpeWriter gpeWriter = new ExportGpeWriter();
							gpeWriter.write((FLyrVect) vectLayer, gps.getAutoGenerationFilePath(), "SHP");
							
							
							((OWMapControl)window.getMapControl()).processEventFileModification(
									"line_"+vectLayer.getName(), EventFileModification.ACTION_ADD, true);
							((OWMapControl)window.getMapControl()).processEventFileModification(
									"point_"+vectLayer.getName(), EventFileModification.ACTION_ADD, true);
						}
					}			
				}
				
				if(gpsTracklogToolButoon != null && gpsTracklogToolButoon.isOn()){
					gpsTracklogToolButoon.reverse();
					window.getMapControl().switchTracklog();
				}

			}catch (Exception e) {
				e.printStackTrace();
			}
			//----------------  ACTIVATION GPS ------------------------
			
		} else {
			
			
			
			//----------------  ACTIVATION GPS ------------------------
			try{
				DistantProject project = DistantProjectManager.getManager().getCurrentDistantProject();
				if(project.getGPSManager() != null)
				{
					DistantGPSManager gps = project.getGPSManager();
					
					if(gps.isSimulationFile()){	  //SIMULATION FILE FOUND
						GPSManager.getGPSManager().setDeviceType(GPSManager.FILE_DEVICE);
						GPSManager.getGPSManager().setSimFolderName(gps.getSimulationFile().getParent());
						GPSManager.getGPSManager().setSimFileName(gps.getSimulationFile().getName());
						GPSManager.getGPSManager().setVSim(100);
						GPSManager.getGPSManager().setShowIcon(gps.isViewGPSPosition());
						GPSManager.getGPSManager().setRecenter(true);
					}
					
					if(gps.isTrackerFile()){	//TRACKER FILE FOUND
						TracklogManager.setFolderName(gps.getTrackerFile().getParent());
						TracklogManager.setFileName(gps.getTrackerFile().getName());
						TracklogManager.setType(TracklogManager.TRACKLOG_TYPE_GPX);
					}
				}

			}catch(Exception ex){
				ex.printStackTrace();
			}
			//----------------  ACTIVATION GPS ------------------------
			
			
			
			try {
				GPSManager.getGPSManager().connectToGPS();
				
				if(RawDataManager.getSaveRawData())
					RawDataManager.startSaving(); 
				
			} catch (GPSException e) {
				AcceptCancelDialog.informUser(
						Messages.getText("MainWindow.121"), //$NON-NLS-1$
						e.getMessage(), 
						window);
				gpsConnectToolButton.reverse();
			}
		
			//----------------  ACTIVATION GPS ------------------------	
			
			try{
				DistantProject project = DistantProjectManager.getManager().getCurrentDistantProject();
				if(project.getGPSManager() != null){
					window.getMapControl().getMapCursorSupport().setViewGPSPosition(project.getGPSManager().isViewGPSPosition());
					window.getMapControl().getMapContext().getAcetateLayer().setGPSTrailerVisible(project.getGPSManager().isViewGPSPosition());
				}else{
					window.getMapControl().getMapCursorSupport().setViewGPSPosition(true);	//DEFAULT TRUE
					window.getMapControl().getMapContext().getAcetateLayer().setGPSTrailerVisible(true);	//DEFAULT TRUE
				}
				
				
			}catch(Exception ex){
				ex.printStackTrace();
			}
						
			//----------------  ACTIVATION GPS ------------------------

		}
	
	}
}
