/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.gvsig.init;





import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.openmis.application.demo.gvsig.services.demo.DemoServiceComGvSiGSynchro;
import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGDelete;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGEventFileModif;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGForeground;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGInformation;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGLoadFile;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGLoadGPSManager;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGLoadProject;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGSituation;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGStorage;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGUnloadProject;
import org.openmis.tellus.init.SIGOpenMISInit;
import org.openmis.tellus.project.DistantProjectFactory;
import org.openmis.tellus.project.impl.DistantProjectFactoryMemory;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.Plateform;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Manuel Gomez
 * @since	JDK 1.1
 * @version 1.0.
 */
public class WebServerOpenMISInit extends SIGOpenMISInit {

	public WebServerOpenMISInit() {
		super();
	}

	public void preLoadingInit() throws ServiceException {
		try {
			Plateform plateform = ApplicationContextManager.getManager().getApplicationContext().getPlateform();
			if(plateform.getOS().equals(Plateform.POCKETPCOS)){
			    //----------------------- Log file ---------------------------
			    Properties  props = new Properties();
			    props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+"/WEB-INF/log/pdalog.txt");
			    File file = new File(System.getProperty("user.dir")+"/WEB-INF/log");
			    if (file.exists())
			    	FileOpenCloseLogManager.registerLogManager(props); //log to a file
			    else LogManager.registerLogManager(null); //log to a file
			    LogManager.traceDebug(0,"init log");
			}    
			
			
			PropertiesManager.getManager().addPropertiesFile(System.getProperty("user.dir")+"/WEB-INF/conf/properties/gvsig.properties");
				
			
			super.preLoadingInit();
			
	    } catch (Throwable ex)  {
	        throw new ServiceException(ex);
	    }
		
	}
	
	
	public DistantServiceManager getDistantServiceManager() throws CommException {
		int port = Integer.parseInt(PropertiesManager.getManager().getProperty("port.openmis.application"));
	    return (new DistantServiceManager(
	    		"localhost"
	    		,port
	    		));
	}
	
	public DistantProjectFactory getDistantProjectFactory() {
		return new DistantProjectFactoryMemory();
	}
	
	
	public List getListDistantServiceToLoad() {
		
		List list = new ArrayList();
		list.add(new ServiceComGvSIGForeground());
		list.add(new ServiceComGvSIGInformation());
		list.add(new ServiceComGvSIGSituation());
		list.add(new ServiceComGvSIGStorage());
		list.add(new ServiceComGvSIGLoadFile());
		list.add(new ServiceComGvSIGLoadGPSManager());
		list.add(new ServiceComGvSIGLoadProject());
		list.add(new ServiceComGvSIGDelete());
		list.add(new DemoServiceComGvSiGSynchro());
		list.add(new ServiceComGvSIGEventFileModif());
		list.add(new ServiceComGvSIGUnloadProject());
		return list;
	}
	
	
	public void postLoadingInit() throws ServiceException {
		
		
		/*String url = PropertiesManager.getManager().getProperty("openmis.googlemap.url.update");
		String id = PropertiesManager.getManager().getProperty("openmis.googlemap.id.technicien");
		
		if( url != null && url.length() > 0 && id != null && id.length() > 0){
			int idTech = Integer.parseInt(id);
			GPSManager.getGPSManager().addGPSRelevantEventsListener(new OpenMISGPSPositionListener(url, idTech));	
		}*/
		
		
		super.postLoadingInit();
	}
	
	

}
