/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.manager;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.common.data.Report;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class InterventionManager {//implements ManagerClassType{

	private static InterventionManager manager;
	private InterventionFactory factory;
		
	public InterventionManager(InterventionFactory factory) {
		this.factory = factory;
	}
		
	public static InterventionManager getManager () {
		return manager;
	}
		
	public static void removeManager()	{
		manager = null;
	}
		  
	public InterventionFactory getFactory() {
		return this.factory;
  	}

	public static void registerManager(InterventionManager newManager) {
		manager = newManager;
		ApplicationContextManager.getManager().addManager(manager);
	}
	
	public void store(Intervention inter)  throws DatabaseException{
		this.getFactory().store(inter);
	}
	
	public Intervention getIntervention(String id) throws DatabaseException{
		return this.getFactory().getIntervention(id);
	}
	
	public Array getAllInterventions() throws DatabaseException{
		return this.getFactory().getAllInterventions();
	}
	
	public Array getInterventionsForStatut(int statut)throws DatabaseException{
		return this.getFactory().getInterventionsForStatut(statut);
	}
	
	public Array getInterventionsForStatutAndUser(int statut, String userID)throws DatabaseException{
		return this.getFactory().getInterventionsForStatutAndUser(statut, userID);
	}

	public void delete(String id) throws DatabaseException {
		this.getFactory().delete(id);
	}
	
	public Report getReportFromReportIDAndInterID(String reportID, String interID) throws DatabaseException{
		Intervention inter = getFactory().getIntervention(interID);
		Array reports = inter.getArrayReport();
		for(int i=0;i<reports.size();i++){
			Report report = (Report)reports.get(i);
			if(report.getIDReport().equals(reportID)){
				return report;
			}
		}
		return null;
	}
	
	public Class getClassTypeManaged(){
		return DefaultIntervention.class;
	}
}
