/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server;

import org.openmis.application.demo.manager.InterventionManager;
import org.openmis.application.demo.server.db.DemoCreateDatabase;
import org.openmis.application.demo.server.db.DemoUserManagerFactory;
import org.openmis.application.demo.server.db.MSSQLInterventionFactoryServer;
import org.openmis.application.demo.server.project.init.SIGSynchroProjectManager;
import org.openmis.application.demo.server.project.init.factory.SIGProjectSynchroBittorrent;
import org.openmis.application.demo.server.project.init.factory.SIGProjectsSynchoDirect;
import org.openmis.application.demo.server.synchro.services.InterventionSyncServiceFactory;
import org.openmis.application.demo.terminal.services.DemoNavigationBarDescription;
import org.openmis.bittorrent.tracker.OpenMISTrackerBittorrent;
import org.openmis.bittorrent.tracker.TrackerBittorrent;
import org.openmis.tellus.synchro.server.listeners.LayersSyncNumberManagerDelegate;
import org.openmis.tellus.synchro.server.listeners.SynchroLayerTargetProcessing;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.profile.ProfileDataManager;
import org.openmobileis.common.user.profile.impl.XmlFileProfilDataFactory;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.database.JdbcPoolManagerDB;
import org.openmobileis.common.util.database.ManagerDB;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDBManager;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSynchroManager;
import org.openmobileis.modules.common.log.server.LogSyncService;
import org.openmobileis.services.navigation.NavigationBarManager;
import org.openmobileis.services.servlet.OpenMISInit;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;


/**
 * @author Manuel Gomez
 */
public class ServerOpenMISInit implements OpenMISInit {

	public ServerOpenMISInit() {
		super();
	}

	public void preLoadingInit() throws ServiceException {
		try {
			 //Log file
		   /* Properties  props = new Properties();
		    props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+"/WEB-INF/log/serverlog.txt");
		    File file = new File(System.getProperty("user.dir")+"/WEB-INF/log");
		    if (file.exists())
		    	FileOpenCloseLogManager.registerLogManager(props); //log to a file
		    else LogManager.registerLogManager(null); //log to a file
		    
		    LogManager.traceDebug(0,"init log");
			*/
			
		      //init Synchro Server side.
			  PropertiesManager.getManager().addPropertiesFile(System.getProperty("user.dir")+"/WEB-INF/conf/properties/server.properties");

		      PropertiesManager.getManager().addProperty("org.openmobileis.common.user.profil.xmlprofilfile.path",System.getProperty("user.dir") + "/WEB-INF/conf/defaultprofils.xml");
		      PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile",System.getProperty("user.dir") + "/WEB-INF/conf/numsync.txt");
		      try {
			      SynchroFastObjectDBManager.getManager().getCurrentFODB(); //init synchro db to be use as default db
			    } catch (Throwable ex)  {
			      throw new ServiceException(ex);
		      }
			    		  
			  //---------------------- register profil and user -----------------------------------------------------
			  //UserManager.getManager().registerUserManagerFactoryForGroup("default", new MonoUserUserManagerFactory());
			  UserManager.getManager().registerUserManagerFactoryForGroup("default", new DemoUserManagerFactory());
			  ProfileDataManager.getManager().registerProfilDataFactoryForGroup("default", new XmlFileProfilDataFactory());
			  
			  //------------------- Register SyncNumberManager ------------------------------------------------------
			  SyncNumberManager.getManager().registerDelegate(new LayersSyncNumberManagerDelegate());
		      
		      SynchroTargerManager.getManager().registerProxyTargetForTerminal(new GenericProxySynchroTarget(), null);
		      
		      SynchroTargerManager.getManager().addSynchroTargetListener(new LogSyncService(), null, null);		      
		      SynchroTargerManager.getManager().addSynchroTargetListener(new SynchroLayerTargetProcessing(), null, null);
		     
		      try{

		    	  //---------- BITTORRENT SYNCHRO ------
				  /*TrackerBittorrent tracker = new OpenMISTrackerBittorrent(
							"192.168.0.100"
							, OpenMISTrackerBittorrent.DEFAULT_PORT);
		    	  
			      SIGSynchroProjectManager.registerManager(new SIGSynchroProjectManager(new SIGProjectSynchroBittorrent(tracker)));*/
		    	  
	    		  //---------- DIRECT SYNCHRO ------
	    		  SIGSynchroProjectManager.registerManager(new SIGSynchroProjectManager(new SIGProjectsSynchoDirect()));
		    	  
		    	  //--------- SIG SYNCHRO INITIALIZATION
			      SIGSynchroProjectManager.getManager().initialize();
			      
			      String dbDefaultPort = PropertiesManager.getManager().getProperty("server.database.dbDefaultPort");
			      String dbServer = PropertiesManager.getManager().getProperty("server.database.dbServer");
			      String dbHost = PropertiesManager.getManager().getProperty("server.database.dbHost");
			      String databaseName = PropertiesManager.getManager().getProperty("server.database.databaseName");
			      
			      String dbServerResult = dbServer+"://"+dbHost+":"+dbDefaultPort+";DatabaseName="+databaseName+";SelectMethod=cursor";
				    
			      //Construit la base de base : Tellus_poc
			      DemoCreateDatabase createBdPDA = new DemoCreateDatabase();
			      createBdPDA.createDatabaseIfNotExist(databaseName
				    		, PropertiesManager.getManager().getProperty("server.database.dbDriver")
				    		, dbServer
				    		, dbHost
				    		, dbDefaultPort
				    		, PropertiesManager.getManager().getProperty("server.database.dbLogin")
				    		, PropertiesManager.getManager().getProperty("server.database.dbPassword")
		    			);
				    
				    
			      	//init synchro database access.
			      	JdbcPoolManagerDB dbmanager = new JdbcPoolManagerDB("JDBCPoolManagerDB"
			    		    , PropertiesManager.getManager().getProperty("server.database.dbDriver")
			    		    , dbServerResult	       
			    		    , PropertiesManager.getManager().getProperty("server.database.dbLogin")
					        , PropertiesManager.getManager().getProperty("server.database.dbPassword")
					        , Integer.parseInt(PropertiesManager.getManager().getProperty("server.database.pool.monitorInterval"))
					        , Integer.parseInt(PropertiesManager.getManager().getProperty("server.database.pool.maxConns"))
					        , Integer.parseInt(PropertiesManager.getManager().getProperty("server.database.pool.idleTimeout"))
					        , Integer.parseInt(PropertiesManager.getManager().getProperty("server.database.pool.checkoutTimeout"))
					        , Integer.parseInt(PropertiesManager.getManager().getProperty("server.database.pool.maxCheckout"))
					        , PropertiesManager.getManager().getProperty("server.database.shutDownQuery")
					     );
				    //JdbcPoolManagerDB dbmanager = new JdbcPoolManagerDB();
				    ManagerDB.registerManager(dbmanager);
				    
				    //---- Load the Interventions collection -----
				    FODBSynchroManager.getManager().registerCollection(new InterventionSyncServiceFactory());
				    
				    
				    //------------ ENREGISTRE LE MANAGER DE LISTE D'INTER POUR LA VISU COTE SERVER
				    InterventionManager.registerManager(new InterventionManager(new MSSQLInterventionFactoryServer()));
				    NavigationBarManager.initializeNavigationBar(new DemoNavigationBarDescription());
		      }catch(Exception ex){
		    	  LogManager.traceError(0, ex);
		      }
      
		      
		 } catch (Throwable ex)  {
		        throw new ServiceException(ex);
		 }

	}

	public void postLoadingInit() throws ServiceException {
		

	}

}
