/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.db;

import java.sql.ResultSet;
import java.util.Calendar;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionFactory;
import org.openmis.application.demo.server.db.googleMap.GoogleMapInterventionManager;
import org.openmis.application.demo.server.synchro.services.InterventionSyncServiceQuery;
import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.database.JdbcPoolManagerDB;
import org.openmobileis.common.util.database.ManagerDB;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class MSSQLInterventionFactoryServer implements InterventionFactory {

	private InterventionSyncServiceQuery query;
	
	public MSSQLInterventionFactoryServer() {
		query = new InterventionSyncServiceQuery((JdbcPoolManagerDB)ManagerDB.getManager());
		this.init();
	}


	public void delete(String id) throws DatabaseException {
		String[] params ={id};
		try{

			query.DeleteInter(params);
			
		}catch(Throwable ex){
			  throw new DatabaseException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}
		
		//GOOGLE MAP
		GoogleMapInterventionManager.getManager().delete(id);
	}
	
	public Array getInterventionsForStatutAndUser(int statut, String userID) throws DatabaseException {
		String[] params ={userID, Integer.toString(statut)};
		Array listInter = new Array();
		
		try{
			
			
			ResultSet result = query.getInterventionsForIDTechAndStatut(params);
			while (result.next())	{
				  DefaultIntervention inter = query.convertResultSetToIntervention(result);
				  if(inter != null){
				    listInter.add(inter);  
				  }
			}
			
			return listInter;
			
		}catch(Throwable ex){
			  throw new DatabaseException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}
	}
	
	public Array getInterventionsForStatut(int statut) throws DatabaseException {
		String[] params ={Integer.toString(statut)};
		Array listInter = new Array();
		
		try{
			
			
			ResultSet result = query.getInterventionForStatut(params);
			while (result.next())	{
				  DefaultIntervention inter = query.convertResultSetToIntervention(result);
				  if(inter != null){
				    listInter.add(inter);  
				  }
			}
			
			return listInter;
			
		}catch(Throwable ex){
			  throw new DatabaseException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}
	}


	public Array getAllInterventions() throws DatabaseException {
		Array listInter = new Array();
		
		try{
			
			
			ResultSet result = query.getInterventions();
			while (result.next())	{
				  DefaultIntervention inter = query.convertResultSetToIntervention(result);
				  if(inter != null){
				    listInter.add(inter);  
				  }
			}
			
			return listInter;
			
		}catch(Throwable ex){
			  throw new DatabaseException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}

	}


	public Intervention getIntervention(String id) throws DatabaseException {
		String[] params ={id};
		DefaultIntervention inter = null;
		try{
			
			
			ResultSet result = query.getInterventionForIDInter(params);
			if (result.next())	{
				  inter = query.convertResultSetToIntervention(result);
			}
			
			return inter;
			
		}catch(Throwable ex){
			  throw new DatabaseException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}
	}


	public void store(Intervention i) throws DatabaseException {
		DefaultIntervention inter = (DefaultIntervention)i;
		String[] params = { inter.getIDInter() };
		boolean insert = false;
		try {
			ResultSet result = query.SelectInter(params);

			if (result.next()) {
				String[] params2 = query.convertInterForUpdate(inter);
				query.UpdateInter(params2);
				insert = false;
			
			}else{
				String[] params2 = query.convertInterForInsert(inter);
				query.InsertInter(params2);
				insert = true;
			}
		}catch (Throwable ex) {
			throw new DatabaseException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}
		
		try{
			User user1 = UserManager.getManager().getUser("default", "1");
			User user2 = UserManager.getManager().getUser("default", "2");
			
			String name="";
			if(inter.getCodeTech() != null){
				if(inter.getCodeTech().equals(user1.getId())){
					name = user1.getLogin();
				}else{
					name = user2.getLogin();
				}
			}
			
			if(insert == false){			
		
				//GOOGLE MAP
				if(inter.getStatut() == Intervention.STATUT_INTERVENTION_FAITE){	//remove from view in googleMap
					GoogleMapInterventionManager.getManager().delete(inter.getIDInter());
				}else{				
					GoogleMapInterventionManager.getManager().update(inter.getIDInter(), inter.getStatut(), inter.getNomClient()+" - "+name);
				}
			}else{
				//GOOGLE MAP
				GoogleMapInterventionManager.getManager().store(inter.getIDInter(), inter.getCoordx(), inter.getCoordy(), inter.getStatut(), inter.getNomClient()+" - "+name);
			}
			
			Thread.sleep(500);
			
		}catch (Throwable ex) {
			throw new DatabaseException(ex);
		}

	}
	
	
	private void init(){
		
		GoogleMapInterventionManager.getManager().deleteAll();
		
		Calendar cal = Calendar.getInstance();
		
		Intervention inter = new DefaultIntervention("1", -0.3518887155548489, 39.46699896026074);
		inter.setNomClient("MFP MICHELINS/ DIVERS DETECTION INCENDIE");
		inter.setDemande("Rparation de la chaufferie");
		inter.setConsigne("Fermer le gaz avant");
	
		cal.set(Calendar.YEAR, 2009);
		cal.set(Calendar.MONTH, 5);
		cal.set(Calendar.DAY_OF_MONTH, 8);
		cal.set(Calendar.HOUR_OF_DAY, 8);
		cal.set(Calendar.MINUTE, 0);
		inter.setDateDebut(cal.getTime().getTime());
		
		cal.set(Calendar.YEAR, 2009);
		cal.set(Calendar.MONTH, 5);
		cal.set(Calendar.DAY_OF_MONTH, 8);
		cal.set(Calendar.HOUR_OF_DAY, 10);
		cal.set(Calendar.MINUTE, 0);
		inter.setDateFin(cal.getTime().getTime());
		
		inter.setResume("CHAUFFERIE");
		inter.setDuree(120);
		inter.setEquipement("Chaudire X5200 VXW");
		inter.setAdress("Calle del Doctor Manuel Candela, 46021, Valence, Espagne");
		inter.setCodeTech("1");
		inter.setStatut(Intervention.STATUT_INTERVENTION);
		inter.setPriorite(Intervention.PRIORITE_NORMAL);
		try{
			this.store(inter);
		}catch(Throwable ex){
			LogManager.traceError(LogServices.WEBSERVICE, ex);
		}
		
		
		
		inter = new DefaultIntervention("2",-0.34184798953660883,39.463478408559794);
		inter.setNomClient("SEPEL EUREXPO/ SONORISATION");
		inter.setDemande("Voir intervention Mr XXX");
		inter.setConsigne("Consigne ");
		
		cal.set(Calendar.YEAR, 2009);
		cal.set(Calendar.MONTH, 5);
		cal.set(Calendar.DAY_OF_MONTH, 8);
		cal.set(Calendar.HOUR_OF_DAY, 10);
		cal.set(Calendar.MINUTE, 0);
		inter.setDateDebut(cal.getTime().getTime());
		
		cal.set(Calendar.YEAR, 2009);
		cal.set(Calendar.MONTH, 5);
		cal.set(Calendar.DAY_OF_MONTH, 8);
		cal.set(Calendar.HOUR_OF_DAY, 12);
		cal.set(Calendar.MINUTE, 0);
		inter.setDateFin(cal.getTime().getTime());
		
		inter.setResume("SONO");
		inter.setDuree(120);
		inter.setEquipement("SONORISATION");
		inter.setAdress("Calle del Maestro Valls, Valence, Espagne");
		inter.setCodeTech("1");
		inter.setStatut(Intervention.STATUT_INTERVENTION);
		inter.setPriorite(Intervention.PRIORITE_NORMAL);
		try{
			this.store(inter);
		}catch(Throwable ex){
			LogManager.traceError(LogServices.WEBSERVICE, ex);
		}
		
		inter = new DefaultIntervention("3",-0.3344606089026959, 39.46486946536976);
		inter.setNomClient("LYONNAISE DE BANQUE/ COURANT FAIBLE");
		inter.setDemande("Detection incendie - Visite de maintenance semestrielle dsefumage");
		inter.setConsigne("Verification extraxtion et exutoire de fume");
		
		cal.set(Calendar.YEAR, 2009);
		cal.set(Calendar.MONTH, 5);
		cal.set(Calendar.DAY_OF_MONTH, 8);
		cal.set(Calendar.HOUR_OF_DAY, 8);
		cal.set(Calendar.MINUTE, 0);
		inter.setDateDebut(cal.getTime().getTime());
		
		cal.set(Calendar.YEAR, 2009);
		cal.set(Calendar.MONTH, 5);
		cal.set(Calendar.DAY_OF_MONTH, 8);
		cal.set(Calendar.HOUR_OF_DAY, 10);
		cal.set(Calendar.MINUTE, 0);
		inter.setDateFin(cal.getTime().getTime());
		
		inter.setResume("L.B.CIC/ DESENFUMAGE/ SEMESTRIELLE");
		inter.setDuree(120);
		inter.setEquipement("COURANT FAIBLE");
		inter.setAdress("Calle de Ernesto Anastasio, 46011, Valence, Espagne");
		inter.setCodeTech("2");
		inter.setStatut(Intervention.STATUT_INTERVENTION);
		inter.setPriorite(Intervention.PRIORITE_NORMAL);
		try{
			this.store(inter);
		}catch(Throwable ex){
			LogManager.traceError(LogServices.WEBSERVICE, ex);
		}
	}

}
