/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.project.init.factory;

import java.io.File;

import org.openmis.bittorrent.common.exception.BittorrentException;
import org.openmis.bittorrent.tracker.TrackerBittorrent;
import org.openmis.tellus.layers.DistantLayer;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class SIGProjectSynchroBittorrent extends SIGProjectsSynchro {
	
	private TrackerBittorrent tracker = null;
	
	public SIGProjectSynchroBittorrent(TrackerBittorrent tracker) throws ServiceException {
		super();
		this.tracker = tracker;
	}
	
	public File[] addFilesToSynchronisation(DistantLayer layer, String projectName) {
		try{
			if(layer.getExtension().equals(DistantLayer.EXT_ECW)){	
				
				File torrent = getTorrentFile(layer.getFile().getParent(), layer.getGeoFileName());
				tracker.addFileToTracker(
						layer.getFile()
						,torrent.getAbsolutePath());
				
				  return new File[]{torrent};
				  
			  }else if(layer.getExtension().equals(DistantLayer.EXT_SHP)){
				  File torrents[] = new File[3];
			
				  torrents[0] = getTorrentFile(layer.getFile().getParent(), layer.getGeoFileName()); //with SHP extension
				  tracker.addFileToTracker(
							layer.getFile()
							,torrents[0].getAbsolutePath());
				  				  
					torrents[1] = getTorrentFile(layer.getFile().getParent(), layer.getDBFFileName());
					tracker.addFileToTracker(
								new File(layer.getFile().getParent(), layer.getDBFFileName())
								,torrents[1].getAbsolutePath());
					
					torrents[2] = getTorrentFile(layer.getFile().getParent(), layer.getSHXFileName());
					tracker.addFileToTracker(
								new File(layer.getFile().getParent(), layer.getSHXFileName())
								,torrents[2].getAbsolutePath());
				  
				  return torrents;
				  
			  }
			  
		}catch(BittorrentException ex){
			LogManager.traceError(0, ex);
		}
		
		return null;
	}
	
	private File getTorrentFile(String directory, String fileName){
		return new File(directory, fileName+".torrent");
	}

}
