/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.project.init.factory;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;

import org.openmis.application.demo.server.project.init.SIGSynchroProjectFactory;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.project.impl.DistantProjectFactoryMemory;
import org.openmis.tellus.project.parser.DistantProjectObjectExtractor;
import org.openmis.tellus.synchro.common.data.DefaultEventFileModification;
import org.openmis.tellus.synchro.common.data.EventFileModification;
import org.openmis.tellus.synchro.common.service.ServiceEventFileModification;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmis.tellus.layers.DistantLayer;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public abstract class SIGProjectsSynchro implements SIGSynchroProjectFactory{

	public static final String TELLUS_OWPROJECT_NAME = "TELLUS_OWPROJECT";
	
	private File fileOWProject = null;
	private String projectsDirectory = null;
	private Array projectsName = null;
	
	public SIGProjectsSynchro() throws ServiceException{
		this.projectsDirectory = PropertiesManager.getManager().getProperty("projects.server.folder"); 	  
		this.fileOWProject = new File(projectsDirectory+File.separator+TELLUS_OWPROJECT_NAME+File.separator+"OWSIGProjects.xml");
		this.projectsName = new Array();
	}

	
	public Array getListProjectName() {
		return projectsName;
	}
	
	public File getFileOWProject() {
		return fileOWProject;
	}
	
	public String getProjectOWDirectory() {
		return projectsDirectory;
	}
	
	
	public void initialize() throws ServiceException {
	    try{
	    	  this.projectsName.clear();
		      if(fileOWProject != null && fileOWProject.exists()){
		    	  
		    	  //----------- Add OWProject.xml as project indispenseble to synchronize on terminal
			      doEventFileModification(EventFileModification.ACTION_ADD
			    		  , TELLUS_OWPROJECT_NAME
			    		  , fileOWProject.getName()
			    		  , fileOWProject.getParent());
			      
			      this.projectsName.add(TELLUS_OWPROJECT_NAME);
			      
			      //---------- Parse the OWProject.xml to find projects to synchronize on terminal
		    	  DistantProjectManager.registerDistantProjectManager(new DistantProjectManager(new DistantProjectFactoryMemory()));
		    	  InputStream in = new FileInputStream(fileOWProject);
				  DistantProjectObjectExtractor.getObject(in, projectsDirectory);
				  
				  //---------- Generate All EventFile for synchronisation init ----
				  this.parseOWProject();
				  
		      }else{
		    	  throw new ServiceException("FILE OWPROJECT NOT FOUND : "+fileOWProject.getAbsolutePath());
		      }
	    }catch(Exception ex){
	    	throw new ServiceException(ex);
	    }	
	}
	
	private void parseOWProject() throws ServiceException {
		try{
		  	  Array array = DistantProjectManager.getManager().getDistantProjects();
			  for(int i=0;i<array.size();i++){
				  DistantProject project = (DistantProject)array.get(i);
				  List list = project.getClassifiedDistantLayersList();
				  for(int j=0;j<list.size();j++){
					 DistantLayer layer = (DistantLayer)list.get(j);
					  
		 		     File[] files = this.addFilesToSynchronisation(layer, project.getUniqueProjectName());
		 		     if(files != null){
					     for(int h=0;h<files.length;h++){ 
					    	 File file = files[h];
					    	 if(file != null){
				    		     doEventFileModification(EventFileModification.ACTION_ADD
				    		    		 , project.getUniqueProjectName()
				    		    		 , file.getName()
				    		    		 , file.getParent());
					    	 }
					     }
		 		     }
					  
					  if(project.getGPSManager() != null){
						  DistantGPSManager gps = project.getGPSManager();
						  
						  if(gps.isSimulationFile()){
							  doEventFileModification(EventFileModification.ACTION_ADD
									  , project.getUniqueProjectName()
			    		    		  , gps.getSimulationFile().getName()
			    		    		  , gps.getSimulationFile().getParent());
						  }
					  }
				  }
				  this.projectsName.add(project.getUniqueProjectName()); 
			  }
		}catch(Exception ex){
			throw new ServiceException(ex);
		}

	}
	
		
	private void doEventFileModification(int actionType, String projectName, String fileName, String serverPath) throws Exception {
		  DefaultEventFileModification event = new DefaultEventFileModification();
	      event.setActionType(actionType);
	      event.setSideModification(EventFileModification.SIDE_SERVER);
	      event.setProjectName(projectName);
	      event.setFileName(fileName);
	      //event.setServerPath(serverPath);
	      new ServiceEventFileModification(event);
	}
	
	public abstract File[] addFilesToSynchronisation(DistantLayer layer, String projectName);
	
}
