/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.services;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.common.data.Report;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class ServiceDemoDisplayHistoInter extends TemplateService implements NavigationBarService {

	private static int uniqID = 0;
	private final static String templateName = "demotemplates/display_inter_histo.htm";

	public String runTemplate(HttpServletRequest req, HttpServletResponse res,TemplateModelRoot templateData) throws ServletException,IOException {
		try{
			templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
			String interID = req.getParameter("interID");
			if(interID != null && interID.length()>0){
				Intervention inter = InterventionManager.getManager().getIntervention(interID);
				if(inter != null){
					String errorMess = req.getParameter("error");
					if(errorMess != null && errorMess.length()>0)
						templateData.put("error", new SimpleScalar(errorMess));

					
					templateData.put("interID", new SimpleScalar(inter.getIDInter()));
					templateData.put("nomClient", new SimpleScalar(inter.getNomClient()));
					templateData.put("demande", new SimpleScalar(inter.getDemande()));
					templateData.put("consigne", new SimpleScalar(inter.getConsigne()));
					templateData.put("equipement", new SimpleScalar(inter.getEquipement()));
					templateData.put("resume", new SimpleScalar(inter.getResume()));
					
					templateData.put("userID", new SimpleScalar(inter.getCodeTech()));
					User user = UserManager.getManager().getUser("default", inter.getCodeTech());
					templateData.put("NomTech", new SimpleScalar(user.getLogin()));
					
					long datelong = inter.getDateDebut();
					Date date = new Date(datelong);
					SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yyyy HH:mm", java.util.Locale.FRANCE);
					String dateForm = dateFomrat.format(date);
					templateData.put("dateInterDeb", new SimpleScalar(dateForm));
					
					datelong = inter.getDateFin();
					date = new Date(datelong);
					dateForm = dateFomrat.format(date);
					templateData.put("dateInterFin", new SimpleScalar(dateForm));

					
					//**************************** Duree Inter *******************************************//
					int hour, min;
					int temp = inter.getDuree();
					hour = temp/60; 
					min = temp - (hour*60);
					if(min <10 && hour <10){		
						templateData.put("duree", new SimpleScalar("0"+hour+":0"+min));
					}else if(min <10){
						templateData.put("duree", new SimpleScalar(hour+":0"+min));	
					}else if(hour <10){
						templateData.put("duree", new SimpleScalar("0"+hour+":"+min));
					}else{
						templateData.put("duree", new SimpleScalar(hour+":"+min));	
					}
					
					//------------------------
					
					String adress = inter.getAdress();
					if(adress != null){
						templateData.put("isAdress", new SimpleScalar("1"));
						templateData.put("adresse", new SimpleScalar(adress));
					}
					Array arrayReports = inter.getArrayReport();
					if(arrayReports != null && arrayReports.size()>0){
						templateData.put("isReport", new SimpleScalar("1"));
						SimpleList list = new SimpleList();
					    list = displayListReport(arrayReports, list);
					    templateData.put("respList",list);
					}
				}
			}
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service ServiceDemoDisplayIntervention run(): "+ex);
		}
		
		return templateName;
	}


	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/inter/display/histo";
	}


	public String getNavigationBarLabel(HttpServletRequest req) {
		return "inter";
	}


	public boolean displayFormExitMessage() {
		return false;
	}


	public boolean displayRecursive() {
		return false;
	}
	
	private SimpleList displayListReport(Array array, SimpleList list) {
		SimpleHash listElement;
		Report report = null;
		
		for(int i = 0; i < array.size(); i++) {
				listElement = new SimpleHash();
				report = (Report)array.get(i);			  
				listElement.put("reportID", report.getIDReport());
				
				long datelong = report.getDateLong();
				Date date = new Date(datelong);
				SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yy HH:mm", java.util.Locale.FRANCE);
				String dateForm = dateFomrat.format(date);
				listElement.put("date",dateForm);
				
				list.add(listElement);		
		}
		return list;
	}

}
