/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class ServiceDemoListTech extends TemplateService implements NavigationBarService {

	private static int uniqID = 0;
	private final static String templateName = "demotemplates/list_tech.htm";
	
	public ServiceDemoListTech() {
		super();
	}


	public String runTemplate(HttpServletRequest req, HttpServletResponse res,TemplateModelRoot templateData) throws ServletException,IOException {
		try{
						
			templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
			String errorMess = req.getParameter("error");
			if(errorMess != null && errorMess.length()>0)
				templateData.put("error", new SimpleScalar(errorMess));
			
			
			User user1 = UserManager.getManager().getUser("default", "1");
			User user2 = UserManager.getManager().getUser("default", "2");
			
			Array array = new Array();
			array.add(user1);
			array.add(user2);
			
			User user = new User("3","tutu", "X");
			array.add(user);
			user = new User("4","titi", "X");
			array.add(user);
			
			SimpleList list = new SimpleList();
		    list = displayList(array, list);
		    templateData.put("respList",list);
		    
			
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service ServiceDemoListTech run(): "+ex);
		}
		
		return templateName;
	}


	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/listtech";
	}


	public String getNavigationBarLabel(HttpServletRequest req) {
		return "techniciens";
	}


	public boolean displayFormExitMessage() {
		return false;
	}


	public boolean displayRecursive() {
		return false;
	}

	
 	private SimpleList displayList (Array array, SimpleList list) {
		SimpleHash listElement;
		User user = null;
		
		for(int i = 0; i < array.size(); i++) {
				listElement = new SimpleHash();
				user = (User)array.get(i);			  
				listElement.put("userID", user.getId());
				listElement.put("login",user.getLogin());
				listElement.put("pwd",user.getPassword());
				
				list.add(listElement);	
		}
		return list;
	}

}
