/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.services;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.collection.Orderable;
import org.openmobileis.common.util.collection.Sort;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.FastScalar;
import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModel;
import freemarker.template.TemplateModelRoot;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class ServiceServerDemoListDayInter extends TemplateService implements NavigationBarService {

	private static int uniqID = 0;
	private final static String templateName = "demotemplates/list_inter_day.htm";

	public String runTemplate(HttpServletRequest req, HttpServletResponse res,TemplateModelRoot templateData) throws ServletException,IOException {
		try{
			templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
			String userID = req.getParameter("userID");
			String daySt = req.getParameter("day");
			
			String errorMess = req.getParameter("error");
			if(errorMess != null && errorMess.length()>0)
				templateData.put("error", new SimpleScalar(errorMess));
			
			User user1 = UserManager.getManager().getUser("default", "1");
			User user2 = UserManager.getManager().getUser("default", "2");
			if(user1.getId().equals(userID)){
				templateData.put("userName", new SimpleScalar(user1.getLogin()));
			}else{
				templateData.put("userName", new SimpleScalar(user2.getLogin()));
			}
			
			Array array = InterventionManager.getManager().getInterventionsForStatutAndUser(Intervention.STATUT_INTERVENTION, userID);
			Sort.shellsort(array,(Orderable)new TriOrderableInterventionDate());
			
			int day = -1;
			if(daySt != null){
				day = Integer.parseInt(daySt);
				
			}else{
				day = -1;
			}
			

			if(day != -1){
				
				String dayName = ServiceServerDemoPlanning.getNameDayForDay(day);
				if(dayName != null){
					dayName = dayName+" "+day+" Juin";
					templateData.put("Day", new SimpleScalar(dayName));
				}
				

				Array listes[] = displayListPerDay(array, day);
				//------------------------------------
				if(listes[0].size() == 0){
					SimpleHash listElement = new SimpleHash();
					listElement.put("empty", "1");
					listElement.put("heure", "8");
					listes[0].add(listElement);	//add both
					
					listElement = new SimpleHash();
					listElement.put("empty", "1");
					listElement.put("heure", "10");
					listes[0].add(listElement);
				}
				if(listes[0].size() == 1){
					SimpleHash listElement = new SimpleHash();
					listElement.put("empty", "1");
					
					SimpleHash temp = (SimpleHash)listes[0].get(0);
					TemplateModel m = temp.get("heure");
					if(m.equals(new FastScalar("8"))){
						listElement.put("heure", "10");
					}else{
						listElement.put("heure", "8");
					}
					
					listes[0].add(listElement);	//add one
				}
				
				if(listes[1].size() == 0){
					SimpleHash listElement = new SimpleHash();
					listElement.put("empty", "1");
					listElement.put("heure", "14");
					listes[1].add(listElement);	//add both
					
					listElement = new SimpleHash();
					listElement.put("empty", "1");
					listElement.put("heure", "16");
					listes[1].add(listElement);
				}	
				if(listes[1].size() == 1){
					SimpleHash listElement = new SimpleHash();
					listElement.put("empty", "1");
					
					SimpleHash temp = (SimpleHash)listes[1].get(0);
					TemplateModel m = temp.get("heure");
					if(m.equals(new FastScalar("14"))){
						listElement.put("heure", "16");
					}else{
						listElement.put("heure", "14");
					}
					
					listes[1].add(listElement);	//add one
				}
				//------------------------------------
				
				SimpleList SListes[] = new SimpleList[2];
				SListes[0] = new SimpleList();				
				SListes[0].add((SimpleHash)listes[0].get(0));
				SListes[0].add((SimpleHash)listes[0].get(1));
				
				SListes[1] = new SimpleList();				
				SListes[1].add((SimpleHash)listes[1].get(0));
				SListes[1].add((SimpleHash)listes[1].get(1));
				
			    templateData.put("respListMatin",SListes[0]);
			    templateData.put("respListAprem",SListes[1]);
				
				
			    
			    //-----------------------------------------------------------
			    
				Array demandes = InterventionManager.getManager().getInterventionsForStatut(Intervention.STATUT_DEMANDE);
				
				SimpleList list = new SimpleList();
				SimpleHash listElement;
		
				if (demandes != null && demandes.size() > 0) {
		
					for (int i = 0; i < demandes.size(); i++) {
						Intervention inter = (Intervention)demandes.get(i);
						listElement = new SimpleHash();
						if(inter.getNomClient() == null && inter.getNomClient().length() == 0){
							listElement.put("value", inter.getResume());
						}else{
							listElement.put("value", inter.getNomClient());
						}
						
						listElement.put("select", false);
						listElement.put("interID", inter.getIDInter());	
						list.add(listElement);
					}
					
				}else{
					templateData.put("isDemande", new SimpleScalar("0"));
				}
				templateData.put("respListDemandes", list);
			    
				
				templateData.put("userID", new SimpleScalar(userID));
				templateData.put("dayOnly", new SimpleScalar(daySt));
			    
			}
			
  
		}catch(Throwable ex){
			LogManager.traceError(0, ex);
		}
		
		return templateName;
	}


	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/listinterday";
	}


	public String getNavigationBarLabel(HttpServletRequest req) {
		return "Journee";
	}


	public boolean displayFormExitMessage() {
		return false;
	}


	public boolean displayRecursive() {
		return false;
	}
	
	
	private Array[] displayListPerDay (Array array, int day) {
		SimpleHash listElement;
		Intervention inter = null;
		Array listes[] = new Array[2];
		listes[0] = new Array();
		listes[1] = new Array();
		
		for(int i = 0; i < array.size(); i++) {

			inter = (Intervention)array.get(i);
			
			//------------------------------
			Calendar cal = Calendar.getInstance();
			Date dateCheck = new Date(inter.getDateDebut());
			cal.setTime(dateCheck);
			int dayInter = cal.get(Calendar.DAY_OF_MONTH);
			int hour = cal.get(Calendar.HOUR_OF_DAY);
			
			if(day == dayInter){
				if(hour <= 12){					
					listElement = addInList(inter);
					
					if(hour < 10){
						listElement.put("heure", "8");
					}else{
						listElement.put("heure", "10");
					}
					
					listes[0].add(listElement);
	
				}else{
					listElement = addInList(inter);
					
					if(hour < 16){
						listElement.put("heure", "14");
					}else{
						listElement.put("heure", "16");
					}
					listes[1].add(listElement);
				}
			}
			
		}
		return listes;
	}
 	
 	private SimpleHash addInList(Intervention inter){
 		SimpleHash listElement = new SimpleHash();
		  
		listElement.put("interID", inter.getIDInter());
		listElement.put("nom",inter.getNomClient());
		listElement.put("resume",inter.getResume());
			
		long datelong = inter.getDateDebut();
		Date date = new Date(datelong);
		SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yyyy HH:mm", java.util.Locale.FRANCE);
		String dateForm = dateFomrat.format(date);
		listElement.put("dateDebut",dateForm);
		
		datelong = inter.getDateFin();
		date = new Date(datelong);
		dateForm = dateFomrat.format(date);
		listElement.put("dateFin",dateForm);
		
		return listElement;
 	}

}
