/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.services;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.collection.Orderable;
import org.openmobileis.common.util.collection.Sort;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class ServiceServerDemoListHistoInter extends TemplateService implements NavigationBarService {

	private static int uniqID = 0;
	private final static String templateName = "demotemplates/list_inter_histo.htm";
	
	public ServiceServerDemoListHistoInter() {
		super();
	}


	public String runTemplate(HttpServletRequest req, HttpServletResponse res,TemplateModelRoot templateData) throws ServletException,IOException {
		try{
			templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
			String userID = req.getParameter("userID");
			
			String errorMess = req.getParameter("error");
			if(errorMess != null && errorMess.length()>0)
				templateData.put("error", new SimpleScalar(errorMess));
			
			Array array = InterventionManager.getManager().getInterventionsForStatut(Intervention.STATUT_INTERVENTION_FAITE);
			SimpleList list = new SimpleList();
			
			if(array != null && array.size() > 0){
				Sort.shellsort(array,(Orderable)new TriOrderableInterventionDate());
			    list = displayList(array, list);
			    templateData.put("respList",list);
		    
			}else{ 
		        templateData.put("isHisto",new SimpleScalar("Aucune intervention trouv&eacute;e")); 
		     }
		    
			
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service ServiceServerDemoListHistoInter run(): "+ex);
		}
		
		return templateName;
	}


	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/listintercloture";
	}


	public String getNavigationBarLabel(HttpServletRequest req) {
		return "historique";
	}


	public boolean displayFormExitMessage() {
		return false;
	}


	public boolean displayRecursive() {
		return false;
	}

	
 	private SimpleList displayList (Array array, SimpleList list) {
		SimpleHash listElement;
		Intervention inter = null;
		
		for(int i = 0; i < array.size(); i++) {
				listElement = new SimpleHash();
				inter = (Intervention)array.get(i);			  
				listElement.put("interID", inter.getIDInter());
				listElement.put("nom",inter.getNomClient());
				listElement.put("resume",inter.getResume());
					
				long datelong = inter.getDateDebut();
				Date date = new Date(datelong);
				SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yyyy HH:mm", java.util.Locale.FRANCE);
				String dateForm = dateFomrat.format(date);
				listElement.put("dateDebut",dateForm);
				
				datelong = inter.getDateFin();
				date = new Date(datelong);
				dateForm = dateFomrat.format(date);
				listElement.put("dateFin",dateForm);
				
				list.add(listElement);	
		}
		return list;
	}

}
