/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.services;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class ServiceServerDemoPlanning extends TemplateService implements NavigationBarService {

	private static int uniqID = 0;
	private final static String templateName = "demotemplates/planning_tech.htm";
	
	private Array jours = null;
	
	public static String getNameDayForDay(int day){
		if(day == 8)
			return "lundi";
		if(day == 9)
			return "mardi";
		if(day == 10)
			return "mercredi";
		if(day == 11)
			return "jeudi";
		if(day == 12)
			return "vendredi";
		if(day == 13)
			return "samedi";
		
		return null;
	}
	
	public String runTemplate(HttpServletRequest req, HttpServletResponse res,TemplateModelRoot templateData) throws ServletException,IOException {
		try{
			templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
			
			jours = new Array();
			jours.add("8");
			jours.add("9");
			jours.add("10");
			jours.add("11");
			jours.add("12");
			jours.add("13");
			
			SimpleList list = new SimpleList();
		    list = displayListDate(jours, list);
		    templateData.put("respListDay",list);
		    
			User user1 = UserManager.getManager().getUser("default", "1");
			User user2 = UserManager.getManager().getUser("default", "2");
		    
		    Array array = new Array();
			array.add(new String[]{"A", user1.getLogin(), user1.getId()});
			array.add(new String[]{"A", user2.getLogin(), user2.getId()});
			array.add(new String[]{"B","tutu", "-1"});
			array.add(new String[]{"B","titi", "-1"});
			
			list = new SimpleList();
		    list = displayListTech(array, list);
		    templateData.put("respListTech",list);
		    
		    
		    
		}catch(Exception e){
			LogManager.traceError(0, e);
		}

		return templateName;
	}

	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/planning";
	}


	public String getNavigationBarLabel(HttpServletRequest req) {
		return "planning";
	}


	public boolean displayFormExitMessage() {
		return false;
	}


	public boolean displayRecursive() {
		return false;
	}
	
	private SimpleList displayListTech (Array array, SimpleList list) {
		SimpleHash listElement;
		String[] params = null;
		
		for(int i = 0; i < array.size(); i++) {
				listElement = new SimpleHash();
				params = (String[])array.get(i);			  
				listElement.put("secteur",params[0]);
				listElement.put("name",params[1]);
				listElement.put("userID",params[2]);
				
				Array arrayState = new Array();
				
				for(int h=0;h<jours.size();h++){

					//not the user titi and tutu
					if(params[2].equals("-1")){
						arrayState.add(h, new String[]{"2","2",(String)jours.get(h)});
					}else{
						arrayState.add(h, new String[]{"0","0",(String)jours.get(h)});	//reinit
					}
				}
				
			    try {
					Array inters = InterventionManager.getManager().getInterventionsForStatutAndUser(Intervention.STATUT_INTERVENTION, params[2]);
					for(int j=0;j<inters.size();j++){
						Intervention inter = (Intervention)inters.get(j);
						
						Calendar cal = Calendar.getInstance();		
						
						if(inter.getDateDebut() != 0){
							Date date = new Date(inter.getDateDebut());
							cal.setTime(date);
							int day = cal.get(Calendar.DAY_OF_MONTH);
							int hour = cal.get(Calendar.HOUR_OF_DAY);
								
							for(int h=0;h<jours.size();h++){
								int weekDay = Integer.parseInt((String)jours.get(h));
								if(day == weekDay){
									String compt[] =  (String[])arrayState.get(h);
									
									if(hour <= 12){
										if("0".equals(compt[0])){
											arrayState.add(h, new String[]{"1", compt[1], compt[2]});
										}
										if("1".equals(compt[0])){
											arrayState.add(h, new String[]{"2", compt[1], compt[2]});
										}
									}else{
										if("0".equals(compt[1])){
											arrayState.add(h, new String[]{compt[0], "1", compt[2]});
										}
										if("1".equals(compt[1])){
											arrayState.add(h, new String[]{compt[0], "2", compt[2]});
										}
									}
									
								}
							}
						}
						
					}
					
				} catch (DatabaseException e) {
					LogManager.traceError(0, e);
				}
				
				SimpleList listInter = new SimpleList();
				listInter = displayListInter(arrayState, listInter);
			    
				listElement.put("respListInter",listInter);
				
				list.add(listElement);	
		}
		return list;
	}
	
	private SimpleList displayListInter (Array array, SimpleList list) {
		SimpleHash listElement;
		String[] params = null;
		
		for(int i = 0; i < array.size(); i++) {
				listElement = new SimpleHash();
				params = (String[])array.get(i);		  
				listElement.put("matin",params[0]);
				listElement.put("aprem",params[1]);
				listElement.put("day",params[2]);
				list.add(listElement);	
		}
		return list;
	}
	
 	private SimpleList displayListDate (Array array, SimpleList list) {
		SimpleHash listElement;
		String day = null;
		
		for(int i = 0; i < array.size(); i++) {
				listElement = new SimpleHash();
				day = (String)array.get(i);			  
				listElement.put("day",day);	
				list.add(listElement);	
		}
		return list;
	}

}
