/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.synchro.services;

import java.sql.ResultSet;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.server.db.googleMap.GoogleMapInterventionManager;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.database.JdbcPoolManagerDB;
import org.openmobileis.common.util.database.ManagerDB;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;
import org.openmobileis.synchro.algo.replication.AlwaysUpdateServerSynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.utils.DefaultSynchroAtomicObject;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class InterventionSyncServiceFactory implements FODBSyncTarget {


	private InterventionSyncServiceQuery query = null;
	
	public InterventionSyncServiceFactory() {
		query = new InterventionSyncServiceQuery((JdbcPoolManagerDB)ManagerDB.getManager());
	}


	public void connect(Credential cred) throws UserNotFoundException,ServiceException {
	}


	public void deleteCollectionObject(String id, long currentSynchroNumber)throws OpenMSPException {
	}


	public void disconnect() {
	}


	public Array getAllCollectionObject() throws OpenMSPException {
		
		//*** TO PICK THE PARAMETERS TO FIND THE RIGHT CONNECTION POOL**//
	    SessionContext context = SessionContextManager.getManager().getSessionContext();
		String repId = context.getUserId();
		//String group = context.getUserGroup();
		
		String[] params ={repId, Integer.toString(Intervention.STATUT_INTERVENTION)};
		Array listInter = new Array();
		
		try{
			
			
			ResultSet result = query.getInterventionsForIDTechAndStatut(params);
			while (result.next())	{
				  DefaultIntervention inter = query.convertResultSetToIntervention(result);
				  if(inter != null){
				    listInter.add(inter);  
				  }
			}
			
			return listInter;
			
		}catch(Throwable ex){
			  throw new OpenMSPException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}
	}


	public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long syncnumber)throws OpenMSPException {
	    SynchroAtomicObject[] sync = new  SynchroAtomicObject[1];
	    sync[0] = new DefaultSynchroAtomicObject("", "");
	    return sync;
	}


	public String getCollectionName() {
		return "demoInter";
	}


	public Object getCollectionObjectWithId(String id) throws OpenMSPException {
		return null;
	}


	public SynchroConflicResolver getConflicResolver() {
		 return new AlwaysUpdateServerSynchroConflicResolver();
	}


	public int getUpdateMaxNbRow() {
		return 0;
	}


	public void setSendSynchroMetaData(Object metadata) throws OpenMSPException {

	}


	public void updateCollectionObject(Object obj, long currentSynchroNumber)throws OpenMSPException {
		DefaultIntervention inter = (DefaultIntervention)obj;
		String[] params = { inter.getIDInter() };
		boolean insert = false;
		try {
			ResultSet result = query.SelectInter(params);

			if (result.next()) {
				String[] params2 = query.convertInterForUpdate(inter);
				query.UpdateInter(params2);
				insert = false;
				
			}else{
				String[] params2 = query.convertInterForInsert(inter);
				query.InsertInter(params2);
				insert = true;
			}
		}catch (Throwable ex) {
			throw new OpenMSPException(ex);
		}finally {
			query.getDbManager().garbageOpenedConnection();
		}

		try{
			User user1 = UserManager.getManager().getUser("default", "1");
			User user2 = UserManager.getManager().getUser("default", "2");
			
			String name="";
			if(inter.getCodeTech() != null){
				if(inter.getCodeTech().equals(user1.getId())){
					name = user1.getLogin();
				}else{
					name = user2.getLogin();
				}
			}
			if(insert == false){
				//GOOGLE MAP
				if(inter.getStatut() == Intervention.STATUT_INTERVENTION_FAITE){	//rmove from view in googleMap
					GoogleMapInterventionManager.getManager().delete(inter.getIDInter());
				}else{
					GoogleMapInterventionManager.getManager().update(inter.getIDInter(), inter.getStatut(), inter.getNomClient()+" - "+name);
				}
			}else{
				//GOOGLE MAP
				GoogleMapInterventionManager.getManager().store(inter.getIDInter(), inter.getCoordx(), inter.getCoordy(), inter.getStatut(), inter.getNomClient()+" - "+name);
			}
		}catch (Throwable ex) {
			throw new OpenMSPException(ex);
		}
	}
	
	public void updateSynchroDB(FastObjectDB db) throws OpenMSPException {

	}
	
}

