/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.server.synchro.services;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.database.JdbcPoolManagerDB;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.openmsp.client.db.DBImportFileCoder;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class InterventionSyncServiceQuery extends AbstractQueryManager {

	/*protected static String createTableInter = "CREATE TABLE Interventions ("
		+"Id_Inter [varchar] (12) NOT NULL ,"
		+"Libelle [varchar] (60) NULL ,"
		+"Objectif [ntext]  NULL ,"
		+"Statut [int] NULL ,"
		+"Nom_client [varchar] (100) NULL ,"
		+"Adresse [ntext] NULL ,"
		+"Equipement [ntext] NULL ,"
		+"Consigne [ntext] NULL ,"
		+"Debut_Prevu [datetime] NULL ,"
		+"Fin_Prevu [datetime] NULL ,"
		+"Charge [int] NULL ,"
		+"Code_Tech [varchar] (10) NULL ,"
		+"Priorite [int] NULL ,"
		+"Coordx [varchar] (25) NULL ,"
		+"Coordy [varchar] (25) NULL"
		+");";*/
	
	protected static String createTableInter = "CREATE TABLE Interventions ("
		+"Id_Inter [varchar](20)NOT NULL ,"
		+"Libelle [ntext]  NULL ,"
		+"Objectif [ntext]  NULL ,"
		+"Statut [int] NULL ,"
		+"Nom_client [ntext] NULL ,"
		+"Adresse [ntext] NULL ,"
		+"Equipement [ntext] NULL ,"
		+"Consigne [ntext] NULL ,"
		+"Debut_Prevu [datetime] NULL ,"
		+"Fin_Prevu [datetime] NULL ,"
		+"Charge [int] NULL ,"
		+"Code_Tech [varchar](20)  NULL ,"
		+"Priorite [int] NULL ,"
		+"Coordx [ntext] NULL ,"
		+"Coordy [ntext] NULL"
		+");";
	
	protected static String droptableinter = "DROP TABLE Interventions";
	
	protected static String queryGetInterventions="SELECT Id_Inter, Libelle, Objectif, Statut, Nom_client, Adresse, Equipement, Consigne, Debut_Prevu, Fin_Prevu, Charge, Code_Tech, Priorite,Coordx, Coordy FROM Interventions";
	protected static String queryGetInterventionsForIDInter="SELECT Id_Inter, Libelle, Objectif, Statut, Nom_client, Adresse, Equipement, Consigne, Debut_Prevu, Fin_Prevu, Charge, Code_Tech, Priorite,Coordx, Coordy FROM Interventions WHERE Id_Inter='%0%'";
	protected static String queryGetInterventionsForIDTech="SELECT Id_Inter, Libelle, Objectif, Statut, Nom_client, Adresse, Equipement, Consigne, Debut_Prevu, Fin_Prevu, Charge, Code_Tech, Priorite,Coordx, Coordy FROM Interventions WHERE Code_Tech='%0%'";
	
	protected static String queryGetInterventionsForIDTechAndStatut="SELECT Id_Inter, Libelle, Objectif, Statut, Nom_client, Adresse, Equipement, Consigne, Debut_Prevu, Fin_Prevu, Charge, Code_Tech, Priorite,Coordx, Coordy FROM Interventions WHERE Code_Tech='%0%' AND Statut='%1%'";
	protected static String queryGetInterventionsForStatut="SELECT Id_Inter, Libelle, Objectif, Statut, Nom_client, Adresse, Equipement, Consigne, Debut_Prevu, Fin_Prevu, Charge, Code_Tech, Priorite,Coordx, Coordy FROM Interventions WHERE Statut='%0%'";
	
	protected static String queryCreateInter = "INSERT INTO Interventions (Id_Inter, Libelle, Objectif, Statut, Nom_client, Adresse, Equipement, Consigne, Debut_Prevu, Fin_Prevu, Charge, Code_Tech, Priorite,Coordx, Coordy) VALUES ('%0%', '%1%', '%2%', '%3%', '%4%', '%5%', '%6%', '%7%', '%8%', '%9%', '%10%', '%11%', '%12%', '%13%', '%14%')";
	protected static String queryUpdateInter = "UPDATE Interventions SET Libelle='%0%', Objectif='%1%', Statut='%2%', Nom_client='%3%', Adresse='%4%', Equipement='%5%', Consigne='%6%', Debut_Prevu='%7%', Fin_Prevu='%8%', Charge='%9%', Code_Tech='%10%', Priorite='%11%', Coordx='%12%', Coordy='%13%' WHERE Id_Inter='%14%'";
	
	protected static String querySelectInter = "SELECT Id_Inter FROM Interventions WHERE Id_Inter='%0%'";
	
	protected static String queryDeleteInter = "DELETE FROM Interventions WHERE Id_Inter='%0%'";
	
	public InterventionSyncServiceQuery(JdbcPoolManagerDB pool) {
		this.registerManagerDB(pool);
		 try{
			  this.remove();
		 }catch(Throwable ex){
			 LogManager.traceError(0,ex);
		 } 
		 try{
		      this.init();  
		 }catch(Throwable ex){
			 LogManager.traceError(0,ex);
		 } 
	}
	
	  public void remove() throws ServiceException{
		    try{
		    	try {
					this.executeUpdate(droptableinter, null);
				} finally {
					this.getDbManager().garbageOpenedConnection();
				}
		    }catch(Throwable ex){
		      throw new ServiceException(ex);
		    }    
	  }
	
	  public void init() throws ServiceException{
		    try{
		    	try {
					Connection con = this.getDbManager().getConnection();
					DatabaseMetaData dbmd = con.getMetaData();
					String[] tableType = {"TABLE"};
					ResultSet result = dbmd.getTables (null,null,"Interventions", tableType);
					if (!result.next()) {
						this.executeUpdate(createTableInter, null);
					}
				} finally {
					this.getDbManager().garbageOpenedConnection();
				}
		    }catch(Throwable ex){
		      throw new ServiceException(ex);
		    }    
	  }
	  
	  
	  
	public ResultSet getInterventionsForIDTechAndStatut (String[] param) throws DatabaseException {
		return this.executeQuery(queryGetInterventionsForIDTechAndStatut, param);
	}
	
	public ResultSet getInterventions () throws DatabaseException {
		return this.executeQuery(queryGetInterventions, null);
	}
	
	public ResultSet getInterventionForIDInter (String[] param) throws DatabaseException {
		return this.executeQuery(queryGetInterventionsForIDInter, param);
	}
	
	public ResultSet getInterventionForStatut (String[] param) throws DatabaseException {
		return this.executeQuery(queryGetInterventionsForStatut, param);
	}
	
	public void InsertInter (String[] param) throws DatabaseException {
		for(int i=0;i<param.length;i++)
			System.out.println(param[i]);
		this.executeUpdate(queryCreateInter, param);
	}
	
	public ResultSet SelectInter (String[] param) throws DatabaseException {
		return this.executeQuery(querySelectInter, param);
	}
	
	public void UpdateInter (String[] param) throws DatabaseException {
		this.executeUpdate(queryUpdateInter, param);
	}
	
	public void DeleteInter (String[] param) throws DatabaseException {
		this.executeUpdate(queryDeleteInter, param);
	}
	
	
	public DefaultIntervention convertResultSetToIntervention(ResultSet result)throws ServiceException	{
	    
	    try	{
	    	
    	  String temp;
	      DefaultIntervention intervention =  new DefaultIntervention(result.getString(1));

	      temp = result.getString(2);
	      if(temp == null){
	    	  intervention.setResume("");
	      }else{
	    	  intervention.setResume(DBImportFileCoder.getCoder().removeAntiSlash(temp));  
	      }
	      
	      temp = result.getString(3);
	      if(temp == null){
	    	  intervention.setDemande("");
	      }else{
	    	  intervention.setDemande(DBImportFileCoder.getCoder().removeAntiSlash(temp));
	      }
	      
	      intervention.setStatut(result.getInt(4));
      
	      temp = result.getString(5);
	      if(temp == null){
	    	  intervention.setNomClient("");
	      }else{
	    	  intervention.setNomClient(DBImportFileCoder.getCoder().removeAntiSlash(temp));
	      }
	      
	      temp = result.getString(6);
	      if(temp == null){
	    	  intervention.setAdress("");
	      }else{
	    	  intervention.setAdress(DBImportFileCoder.getCoder().removeAntiSlash(temp));
	      }
	      
	      temp = result.getString(7);
	      if(temp == null){
	    	  intervention.setEquipement("");
	      }else{
	    	  intervention.setEquipement(DBImportFileCoder.getCoder().removeAntiSlash(temp));
	      }
	      
	      temp = result.getString(8);
	      if(temp == null){
	    	  intervention.setConsigne("");
	      }else{
	    	  intervention.setConsigne(DBImportFileCoder.getCoder().removeAntiSlash(temp));
	      }
	        
	      String DateStart = result.getString(9);
	      String DateEnd = result.getString(10);

	      intervention.setDateDebut(convertStringDateToLong(DateStart));
	      intervention.setDateFin(convertStringDateToLong(DateEnd));
	      
	      intervention.setDuree(result.getInt(11));
	      
	      intervention.setCodeTech(result.getString(12));
	      
	      intervention.setPriorite(result.getInt(13));
	      
	      intervention.setCoordx(Double.parseDouble(result.getString(14)));
	      intervention.setCoordy(Double.parseDouble(result.getString(15)));
			
		  return intervention;
			} catch (Throwable ex)	{
				throw new ServiceException(ex);
			}
	  }
	
	  public static long convertStringDateToLong(String dateString){
		    long mydate = 0;
		    try{
		      
		      SimpleDateFormat dateFomrat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", java.util.Locale.FRANCE);
		      Date date = dateFomrat.parse(dateString);
		        
		      mydate = date.getTime();
		          
		    }catch(Throwable ex){
		      //System.out.println("Erreur during execution of the service : convertStringDatetoLong(), Date invalide pour la convertion"+ex);
		    }
		    return mydate; 
	  }
	  
	  public String[] convertInterForInsert(DefaultIntervention inter) throws ServiceException {
		    String[] params = new String[15];
			  
			  try	{
				     
				  params[0] = inter.getIDInter();		      
			      if(inter.getResume() == null){
			        params[1] = "";
			      }else{
			        params[1] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getResume());
			        if (params[1].length() >= 60)	{
			        	params[1] = params[1].substring(0, 59);
			        }
			      }
			      if(inter.getDemande() == null){
			        params[2] = "";
			      }else{
			        params[2] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getDemande());
			      }
			      params[3] = Integer.toString(inter.getStatut());
			 
			      if(inter.getNomClient() == null){
			    	  params[4] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[4] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getNomClient());  
			      }
			      
			      if(inter.getAdress() == null){
			    	  params[5] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[5] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getAdress());  
			      }
			      
			      if(inter.getEquipement() == null){
			    	  params[6] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[6] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getEquipement());  
			      }
			      
			      if(inter.getConsigne() == null){
			    	  params[7] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[7] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getConsigne());  
			      }
			         
			      params[8] = convertLongDateToString(inter.getDateDebut());
			      params[9] = convertLongDateToString(inter.getDateFin());
			      
			      params[10] = Integer.toString(inter.getDuree());
			     
			      if(inter.getCodeTech() == null){
			    	  params[11] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[11] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getCodeTech());
			      }
			      
			      params[12] = Integer.toString(inter.getPriorite());
			      params[13] = Double.toString(inter.getCoordx());
			      params[14] = Double.toString(inter.getCoordy());
			      
			      return params;
				
			} catch (Throwable ex)	{
				throw new ServiceException(ex);
			}
	  }
	  
	  
	  
	  public String[] convertInterForUpdate(DefaultIntervention inter) throws ServiceException {
		    String[] params = new String[15];
			  
			  try	{

			      if(inter.getResume() == null){
			        params[0] = "";
			      }else{
			        params[0] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getResume());
			        if (params[0].length() >= 60)	{
			        	params[0] = params[1].substring(0, 59);
			        }
			      }
			      if(inter.getDemande() == null){
			        params[1] = "";
			      }else{
			        params[1] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getDemande());
			      }
			      params[2] = Integer.toString(inter.getStatut());
			 
			      if(inter.getNomClient() == null){
			    	  params[3] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[3] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getNomClient());  
			      }
			      
			      if(inter.getAdress() == null){
			    	  params[4] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[4] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getAdress());  
			      }
			      
			      if(inter.getEquipement() == null){
			    	  params[5] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[5] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getEquipement());  
			      }
			      
			      if(inter.getConsigne() == null){
			    	  params[6] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[6] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getConsigne());  
			      }
			         
			      params[7] = convertLongDateToString(inter.getDateDebut());
			      params[8] = convertLongDateToString(inter.getDateFin());
			      
			      params[9] = Integer.toString(inter.getDuree());
			     
			      if(inter.getCodeTech() == null){
			    	  params[10] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB("");
			      }else{
			    	  params[10] = DBImportFileCoder.getCoder().cleanAndFormatStringforDB(inter.getCodeTech());
			      }
			      
			      params[11] = Integer.toString(inter.getPriorite());
			      params[12] = Double.toString(inter.getCoordx());
			      params[13] = Double.toString(inter.getCoordy());
			      
			      
				  params[14] = inter.getIDInter();
			      
			      return params;
				
			} catch (Throwable ex)	{
				throw new ServiceException(ex);
			}
	  }
	  
	
	  public static String convertLongDateToString(long dateLong){
		    String stringDate = "";
		    try{
		      
		  		Date date = new Date(dateLong);
		  		SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss", java.util.Locale.FRANCE);
		  		stringDate = dateFomrat.format(date);
		          
		    }catch(Throwable ex){
		      //System.out.println("Erreur during execution of the service : convertLongDateToString(), Date invalide pour la convertion"+ex);
		    }
		    return stringDate; 
		  }

}
