/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.openmis.application.demo.terminal.com.services.DemoOpenMISSynchroDistantService;
import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.init.SIGOpenMISInit;
import org.openmis.tellus.project.DistantProjectFactory;
import org.openmis.tellus.project.impl.DistantProjectFactoryFODB;
import org.openmis.tellus.service.distant.OpenMISEventFileModifDistantService;
import org.openmis.tellus.service.distant.OpenMISLoadFileDistantService;
import org.openmis.tellus.service.distant.OpenMISLoadGPSManagerDistantService;
import org.openmis.tellus.service.distant.OpenMISLoadProjectDistantService;
import org.openmis.tellus.service.distant.OpenMISUnLoadProjectDistantService;
import org.openmis.tellus.services.impl.AliveDistantService;
import org.openmis.tellus.synchro.terminal.services.tellus.GvSIGDirectSyncFactory;
import org.openmis.tellus.synchro.terminal.services.tellus.OpenMISBittorrentSyncFactory;
import org.openmis.tellus.synchro.terminal.services.tellus.OpenMISDirectSyncFactory;
import org.openmis.tellus.synchro.terminal.services.tellus.TellusSynchronisationManager;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.Plateform;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDBManager;
import org.openmobileis.modules.common.log.terminal.LogsSynchroListener;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.journal.SimpleLogRenderer;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class TerminalOpenMISInit extends SIGOpenMISInit {

	public TerminalOpenMISInit() {
		super();
	}
	
	
	public void preLoadingInit() throws ServiceException {
		try {		
			Plateform plateform = ApplicationContextManager.getManager().getApplicationContext().getPlateform();
			if(plateform.getOS().equals(Plateform.POCKETPCOS)){
			    //----------------------- Log file ---------------------------
			    Properties  props = new Properties();
			    props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+"/WEB-INF/log/pdalog.txt");
			    File file = new File(System.getProperty("user.dir")+"/WEB-INF/log");
			    if (file.exists())
			    	FileOpenCloseLogManager.registerLogManager(props); //log to a file
			    else LogManager.registerLogManager(null); //log to a file
			    LogManager.traceDebug(0,"init log");
			}    
		    
			PropertiesManager.getManager().addPropertiesFile(System.getProperty("user.dir")+"/WEB-INF/conf/properties/terminal.properties");  	      
		    PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile",System.getProperty("user.dir") + "/WEB-INF/conf/numsync.txt");
		    try {
		    	FastObjectDBManager.registerManager(new SynchroFastObjectDBManager()); //init synchro db to be use as default db
		    } catch (Throwable ex)  {
		      throw new ServiceException(ex); 
		    }
			    
			//--------------------- Log Listener ----------------------------------------
			OpenMSPSynchroManager.getManager().addListener(new LogsSynchroListener(),null);
			JournalManager.getManager().registerJournalLogRenderer(new SimpleLogRenderer("FWKLogSynchroListener", "Logs"));
			
			TellusSynchronisationManager.registerManager(
					new TellusSynchronisationManager(new OpenMISDirectSyncFactory()));
			
			/*TellusSynchronisationManager.registerManager(
					new TellusSynchronisationManager(new OpenMISBittorrentSyncFactory()));*/
			
			super.preLoadingInit();
			
	    } catch (Throwable ex)  {
	        throw new ServiceException(ex);
	    }
		
	}

	public void postLoadingInit() throws ServiceException {
		try{

			/*String path = PropertiesManager.getManager().getProperty("xml.terminal.folder.projects");
			File file = new File(path+File.separator+"OWSIGProjects.xml");		
			InputStream in = new FileInputStream(file);
			DistantProjectObjectExtractor.getObject(in, path);*/

			
			
			/*DistantProject project = new DefaultDistantProject("SANDRA");
			project.addDistantLayer(new DefaultDistantLayer(new File(path+File.separator+"Sandra/Valencia_4326.ecw")));
			project.addDistantLayer(new DefaultDistantLayer(new File(path+File.separator+"Sandra/ouvrageArt_epsg4326.shp")));
			project.addDistantLayer(new DefaultDistantLayer(new File(path+File.separator+"select_manza_epsg4326.shp")));
			project.addDistantLayer(new DefaultDistantLayer(new File(path+File.separator+"select_s46_epsg4326.shp")));
			project.addDistantLayer(new DefaultDistantLayer(new File(path+File.separator+"select_c_epsg4326.shp")));
			project.addDistantLayer(new DefaultDistantEditableLayer(new File(path+File.separator+"intervents_epsg4326.shp"), "ID", DefaultIntervention.class));
			DistantProjectManager.getManager().storeDistantProject(project);
			DistantProjectManager.getManager().setProjectAsCurrent(project.getUniqueProjectName());*/
			
			super.postLoadingInit();
			
		}catch(Throwable ex){
			LogManager.traceError(LogServices.DATABASESERVICE, ex);
		}
	}
	
	public DistantProjectFactory getDistantProjectFactory() {
		return new DistantProjectFactoryFODB();
	}
	
	public DistantServiceManager getDistantServiceManager() throws CommException{
		int port = Integer.parseInt(PropertiesManager.getManager().getProperty("port.gvsig.application"));
		return new DistantServiceManager("localhost", port);
	}
	
	public List getListDistantServiceToLoad() {
		List list = new ArrayList();
		list.add(new OpenMISLoadFileDistantService());
		list.add(new OpenMISLoadGPSManagerDistantService());
		list.add(new OpenMISLoadProjectDistantService());
		list.add(new AliveDistantService());	
		list.add(new DemoOpenMISSynchroDistantService());
		list.add(new OpenMISEventFileModifDistantService());
		list.add(new OpenMISUnLoadProjectDistantService());
		return list;
	}

}
