/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.com.services;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.service.distant.template.OpenMISInformationDistantTemplateService;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class ServiceComGetCoordGeographic extends OpenMISInformationDistantTemplateService {
	
	private String idObject = null;

	
	public void runTemplate(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {	
		this.idObject = req.getParameter("idObject");
	}
	
	public DistantInfoableLayer getDistantInfoableLayer() {
		try {
			return (DistantInfoableLayer) DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(DefaultIntervention.class);
		} catch (DatabaseException e) {
			e.printStackTrace();
		}
		return null;
	}

	
	public String getObjectID() {
		return this.idObject;
	}
	
	
	public void runCall(String idObject, DistantInfoableLayer layer, DistantCallMessageResponse res) {
		Intervention inter = null;
		try {
			inter = InterventionManager.getManager().getIntervention(idObject);
		} catch (DatabaseException e) {
			LogManager.traceError(0,"Error CommController getObjectFromInfosParams() : "+e);
			res.setErrorMessage("Internal error in requesting DB");
		}
		//ici genere le SOAP file
		if(inter != null){
			res.addParameter("ID", inter.getIDInter(), DistantMessage.TYPE_STRING, "ID");
			res.addParameter("nomClient", inter.getNomClient(), DistantMessage.TYPE_STRING, "Client name");
			res.addParameter("objective", inter.getDemande(), DistantMessage.TYPE_STRING, "Objective");
			
			String uriAsk = "/inter/display?interID="+inter.getIDInter();
			String url = "http://localhost:9090/services/com/service/launcher?uri="+uriAsk;	//A REVOIR
			res.addParameter("LINK", url, DistantMessage.TYPE_STRING, "View more info");
			
			long datelong = inter.getDateDebut();
			Date date = new Date(datelong);
			SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yy HH:mm", java.util.Locale.FRANCE);
			String dateForm = dateFomrat.format(date);
			res.addParameter("dateStart", dateForm, DistantMessage.TYPE_STRING, "Start date");

			int hour, min;
			int temp = inter.getDuree();
			hour = temp/60; 
			min = temp - (hour*60);
			if(min <10 && hour <10){	
				res.addParameter("duree","0"+hour+":0"+min,DistantMessage.TYPE_STRING,"Time duration");
			}else if(min <10){
				res.addParameter("duree",hour+":0"+min,DistantMessage.TYPE_STRING,"Time duration");
			}else if(hour <10){
				res.addParameter("duree","0"+hour+":"+min,DistantMessage.TYPE_STRING,"Time duration");
			}else{
				res.addParameter("duree",hour+":"+min,DistantMessage.TYPE_STRING,"Time duration");
			}
			

		}else{
			res.setErrorMessage("Object Not Found with the ID in SI database");
		}

	}

}
