/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.com.services;

import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.service.distant.template.OpenMISStoreGeoDistantTemplateService;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.embedded.util.SystemAPI;
import org.openmobileis.services.common.ServiceManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class ServiceComStoreGeometry extends OpenMISStoreGeoDistantTemplateService {

	private String idObject = null;
	private boolean gps = true;

	public void runTemplate(HttpServletRequest req, HttpServletResponse res)throws ServletException, IOException {
		idObject = req.getParameter("idObject");	
		String gpsStr = req.getParameter("gps");
		if("1".equals(gpsStr) || "true".equals(gpsStr)){
			gps = true;
		}else{
			gps = false;
		}
	}
	
	public DistantEditableLayer getDistantEditableLayer() {
		try {
			return (DistantEditableLayer) DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(DefaultIntervention.class);
		} catch (Exception e) {
			removeIntervention();
		}
		return null;
	}
	
	public boolean positionToGPS() {
		return gps;
	}
	
	public String getObjectID() {
		return idObject;
	}
	
	private void removeIntervention(){
		try {
			InterventionManager.getManager().delete(this.idObject);
		} catch (DatabaseException ex) {
			LogManager.traceError(0,"Error during execution of ServiceComStoreGeometry  with delete inter : "+idObject);
		}
	}
	
	public void response(DistantSendMessageResponse res, double coordx, double coordy) {
		if(res.isError()){	//si erreur on delete l'inter car pas presente sur gvSIG
			removeIntervention();
		}else{
			if(coordx != -1 && coordy != -1){
				try {
					Intervention inter = InterventionManager.getManager().getIntervention(idObject);
					if(inter != null){
						inter.setCoordx(coordx);
						inter.setCoordy(coordy);
						InterventionManager.getManager().store(inter);
					}
				}catch (Exception e) {
					LogManager.traceError(0,e);
				}
			}
		}
	}
	
	public void runCall(DistantEditableLayer layer, DistantCallMessageResponse res) {
		String uri = "/inter/edit?isFromGvSIG=1";
		String url = "";
		if(uri != null && uri.length()> 0){
			Properties webserverProps = null;
			try {
				webserverProps = PropertiesManager.getManager().getPropertiesFromPath(System.getProperty("user.dir")+"/WEB-INF/conf/properties/webserver.properties");
			} catch (IOException e) {
				e.printStackTrace();
			}
			String port = webserverProps.getProperty("port");
			String addr = "localhost";
			url = "http://"+addr+":"+port+ServiceManager.getManager().getServiceBaseURI()+uri;
			
		}
		
		try{

			LogManager.traceInfo(0, "Open Pocket IE (ppcsystemapi.dll): "
					+PropertiesManager.getManager().getProperty("org.openmobileis.embedded.execprog.browser")
					+" with params : "
					+url);
			
			SystemAPI.getManager().execProgram(
					PropertiesManager.getManager().getProperty("org.openmobileis.embedded.execprog.browser")
					,url);//params
			
		}catch(Throwable ex){
			LogManager.traceError(0,"Error SIGTemplateService, Library pccsystemapi not FOUND : "+ex);
			res.setErrorMessage("not found SystemAPI");
		}
	}

}
