/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.fodb;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmis.application.demo.terminal.data.DemoRepresentant;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.BadDataFormatException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.db.query.soda.FODBSodaObjectSet;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDB;
import org.openmobileis.modules.common.database.fodb.FODBTerminalUserFactory;


/**
 * @author Romain Beaugrand
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class DemoFODBRepresentantFactory extends FODBTerminalUserFactory {

	public DemoFODBRepresentantFactory() {
	  super();			
		String dbPath = PropertiesManager.getManager().getProperty("fastobjectdb.database.path");
		if (dbPath == null)	{
			LogManager.traceError(0, "Client Factory FODB ERROR : FODB path property is not set. Can't create db");
		}
		try	{
			FastObjectDB db = FastObjectDBManager.getManager().getCurrentFODB();
			
			if (!db.isCollectionExist(this.getCollectionName()))	{
				db.createCollection(this.getCollectionName(), this.getDataType());
				FODBStringIndexDescriptor NameDescriptor = new FODBStringIndexDescriptor("Name", FODBIndexDescriptor.MULTIPLE, "getNomTech()", 12, 50);
				db.addIndex(this.getCollectionName(), NameDescriptor);
			}
			//always register notification listener because it's not persistent.
			((SynchroFastObjectDB)db).registerSynchroFODBReturnListener(this.getCollectionName(), this.getSynchroListener());

		} catch (Throwable ex)	{
			LogManager.traceError(0, "Client Factory FODB ERROR : Unknown error during creation");
			LogManager.traceError(0, ex);
		}
	  //this.init();
	}
	
	/*private void init() {
	  CorimFODBRepresentant rep = new CorimFODBRepresentant("1234567891");
	  rep.setInstallTerminalUser(true);
	  rep.setNomTech("Zozo");
	  rep.setPrenomTech("toto");
	  rep.setLangue("fr");
	  rep.setCodeCourtTech("123");
	  try {
	    super.addTerminalUser(rep);
	    super.setInstallTerminalUser(rep.getTerminalUserId());
	  } catch (Exception ex) {
	    LogManager.traceWarning(LogServices.DATABASESERVICE, ex);
	    // MARCHE PAS !!!
	  }
	}*/
		
/*	protected RepresentantSynchroFODBReturnListener getSynchroListener() {
//		return new CorimRepresentantSynchroFODBReturnListener();
	  return new RepresentantSynchroFODBReturnListener();
	}*/
	
	public Array getAllRepresentantByName(String name)	{
		try {
			Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
			q.constrain(this.getDataType());
			Query subq = q.descend("getNomTech()");
			subq.constrain(name).like();
			ObjectSet set = q.execute(); 
			return ((FODBSodaObjectSet)set).getArrayFromSet();
		} catch (Exception e) {
			LogManager.traceError(LogServices.DATABASESERVICE, e);
		} 
		return null;
  }

	protected int getMaxTerminalUserIdLength(){
		return 12;
	}
	
	protected Class getDataType(){
		return DemoRepresentant.class;
	}
	
	protected void initDB(FastObjectDB db) throws FODBException, BadDataFormatException{
	  
	}
	
}
