/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.fodb;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionFactory;
import org.openmis.application.demo.terminal.data.MyLogRenderer;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FODBIntIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.query.soda.FODBSodaObjectSet;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDB;
import org.openmobileis.synchro.journal.JournalManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class FODBInterventionFactoryClient implements InterventionFactory{

	public FODBInterventionFactoryClient() {
		super();
		try {
			FastObjectDB db = FastObjectDBManager.getManager().getCurrentFODB();
			
			JournalManager.getManager().registerJournalLogRenderer(new MyLogRenderer(this.getDBName(),"Interventions","NbrInterRecu"));
			if (!db.isCollectionExist(this.getDBName()))	{
				db.createCollection(this.getDBName(), DefaultIntervention.class);
				
				FODBStringIndexDescriptor id = new FODBStringIndexDescriptor("id", FODBStringIndexDescriptor.UNIQUE, "getIDInter()", 12, 26);
				db.addIndex(this.getDBName(), id);
				
				FODBIntIndexDescriptor statut = new FODBIntIndexDescriptor("statut", FODBStringIndexDescriptor.MULTIPLE, "getStatut()", 12, 4);
				db.addIndex(this.getDBName(), statut);
			}
		      // always register notification listener because it's not persistent.
		      ((SynchroFastObjectDB) db).registerSynchroFODBReturnListener(this.getDBName(), this.getSynchroListener(this.getDBName()));
		      
		      //this.init();
		} catch (Throwable ex)  {
			LogManager.traceError(0,ex);
		}
	}
	
	protected InterventionSynchroFODBReturnListener getSynchroListener(String databaseName) {
		return new InterventionSynchroFODBReturnListener(databaseName);
	}
	
	private String getDBName(){
		return "demoInter";
	}
	
	
	public Array getInterventionsForStatutAndUser(int statut, String userID) throws DatabaseException {
		//Not used on terminal
		return null;
	}
	
	public void store(Intervention i) throws DatabaseException{

		FastObjectDB db = FastObjectDBManager.getManager().getCurrentFODB();
		boolean test = this.exists(i.getIDInter());
		if(test == false){
		  db.add(this.getDBName(), i);
		}else{
		  db.replace(this.getDBName(), i);
		}

	}
  
  private boolean exists(String id)  throws DatabaseException{
			Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
			q.constrain(this.getDBName());
			Query subq = q.descend("getIDInter()");
			subq.constrain(id).equal();
			ObjectSet set = q.execute();
			if (set.size() > 0) {
				return true;
			}
			return false;
	}
  
  
  public Intervention getIntervention(String id) throws DatabaseException	{
		Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
		q.constrain(this.getDBName());
		Query subq = q.descend("getIDInter()");
		subq.constrain(id).equal();
		ObjectSet set = q.execute();  
		if (set.hasNext())	{
			return (Intervention) set.next();
		}
		return null;  
	}
  
  public Array getInterventionsForStatut(int statut) throws DatabaseException {
		Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
		q.constrain(this.getDBName());
		Query subq = q.descend("getStatut()");
		subq.constrain(new Integer(statut)).equal();
		ObjectSet set = q.execute();   
		return ((FODBSodaObjectSet)set).getArrayFromSet();  
}
  
	public Array getAllInterventions() throws DatabaseException {
		Query q = FastObjectDBManager.getManager().getCurrentFODB().query();
		q.constrain(this.getDBName());
		q.descend("getIDInter()");
		ObjectSet set = q.execute();   
		return ((FODBSodaObjectSet)set).getArrayFromSet();  
	}
	
	public void delete(String id) throws DatabaseException {
		Intervention inter = this.getIntervention(id);
		if (inter != null)	{
			FastObjectDBManager.getManager().getCurrentFODB().delete(this.getDBName(),inter);
		}
	}
}
