/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.modules;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DateFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.openmobileis.common.util.log.LogManager;



public class OWCalendar extends Frame {
	private static final long serialVersionUID = -6070460129417383057L;

	class WindowsEventHandler extends WindowAdapter {
		public void windowClosing(WindowEvent evt) {
			maskWindows();
		}
	   public void windowIconified(WindowEvent e){
		   maskWindows();
	   }         
	}
        
    private GregorianCalendar calendar = new GregorianCalendar();
    private GregorianCalendar calendarActu = (GregorianCalendar)GregorianCalendar.getInstance();
    private long dateChecked, actualDateValue;
    
    private SimpleDateFormat formatLabelPres = new SimpleDateFormat("MMM yyyy");
    private DateFormatSymbols formatDateSymb = new DateFormatSymbols();
    private SimpleDateFormat formatDateRecup = new SimpleDateFormat("dd/MM/yyyy");
    
    private final Label screenYearandMonth = new Label("");     
    private final Label[] daysOfWeek = new Label[7];  
    private final Button[] daysNumbers = new Button[42];
    
    private int firstDayOfWeek = calendar.getFirstDayOfWeek();
    Color WHITE = new Color(255,255,255);
    
    private final ActionListener changeMonth = new ActionListener() {
        public void actionPerformed(final ActionEvent ae) {
        	final int nb;
        	if("next".equals(ae.getActionCommand())){
        		nb=1;
        	}else{
        		nb=-1;
        	}
            calendar.add(Calendar.MONTH, nb);
            reloadTheMonthandYear();
            reloadTheNumberofDays();
        }
    };
    
    private final ActionListener getDayValue = new ActionListener() {
    	public void actionPerformed(final ActionEvent ae) {
    		dateChecked = Long.parseLong(ae.getActionCommand());
    		maskWindows();
    	}
    };
    
    private final ActionListener closeListener = new ActionListener() {
    	public void actionPerformed(final ActionEvent ae) {
    		maskWindows();
    	}
    };
    
    public synchronized long getDateChecked(){
    	return this.dateChecked;
    }
    
    public synchronized void showFrame(long actualDatevalue){
    	this.actualDateValue = actualDatevalue;
    	
    	reloadTheMonthandYear();
    	reloadTheDaysofWeek();
        reloadTheNumberofDays(); 
    	this.setVisible(true);
    }
    
    public OWCalendar(int x, int y, String closeButtonValue) {
    	try{
        	this.addWindowListener(new WindowsEventHandler());
        	
        	Panel main = new Panel();
        	main.setLayout(new BorderLayout());
            
        	final Panel month = new Panel();
            
        	final Button previous = new Button("<");
            Font bold = new Font("bold",Font.BOLD,11);
            previous.addActionListener(changeMonth);
            previous.setBackground(WHITE);
            previous.setFont(bold);
            
            final Button next = new Button(">");
            next.setBackground(WHITE);
            next.setFont(bold);
            
            next.addActionListener(changeMonth);
            next.setActionCommand("next");
            
            month.add(previous);
            month.add(screenYearandMonth);
            month.add(next);
            main.add(month, BorderLayout.NORTH);
            

            final Panel day = new Panel();
            day.setLayout(new GridLayout(7, 7));
            
            for (int i = 0; i < 7; ++i)
                day.add(daysOfWeek[i] = new Label(""));
            
            for (int i = 0; i < daysNumbers.length; ++i){
            	daysNumbers[i] = new Button();
            	daysNumbers[i].addActionListener(getDayValue);
                day.add(daysNumbers[i]);	
            }
                        
            main.add(day, BorderLayout.CENTER);          
            //------------------------ Update All the Components   
            reloadTheMonthandYear();
            reloadTheDaysofWeek();
            reloadTheNumberofDays(); 
            
            final Button close = new Button(closeButtonValue);
            close.setBackground(WHITE);
            close.addActionListener(closeListener);
            main.add(close, BorderLayout.SOUTH);
            this.add(main);
            this.pack();
            this.setSize(x,y);
            this.setVisible(false);
    	}catch(Throwable ex){
    		LogManager.traceError(0,"Error during execution of service Calendrier, Calendrier() : "+ex );
    	}
    	
    }
    
    
    //------------------------------------------------ Put visible or not the Windows, to win time when screen on the calendar component
    public void maskWindows()	{
		try{
	    	//super.dispose();
			this.setVisible(false);
			synchronized(this)	{
				this.notifyAll();
			}
    	}catch(Throwable ex){
    		LogManager.traceError(0,"Error during execution of service Calendrier, maskWindows() : "+ex );
    	}

	}
    
     
    //----------------------------------------------- To screen the currently Year and month 
    private void reloadTheMonthandYear() {
       try{
       	   //space before to center the Label
    	   screenYearandMonth.setText("    "+formatLabelPres.format(calendar.getTime()));
           Font bold = new Font("bold",Font.BOLD,12);
           screenYearandMonth.setFont(bold);
	   	}catch(Throwable ex){
			LogManager.traceError(0,"Error during execution of service Calendrier, updateMonthYear() : "+ex );
	   	} 
    }
    
    
//  -------------------------------------------- To screen the number of the days
    private void reloadTheNumberofDays() {
        try{
        	
        	final Date tmp = calendar.getTime();
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            final int firstDay = calendar.get(Calendar.DAY_OF_WEEK);
            final int LocalFirstDay = (firstDay - firstDayOfWeek + 7) % 7 + 1;
            boolean full = false;
            for (int i = 0; i < daysNumbers.length; ++i) {
                //---------------------------------------- Put visible or not the component
                final boolean isNotEmpty = i < LocalFirstDay - 1 || full;
                daysNumbers[i].setVisible(!isNotEmpty);
                //----------------------------------------Screen the Day
                if (!isNotEmpty) {
                    final int dayOfMonth = calendar.get(Calendar.DAY_OF_MONTH);
                    daysNumbers[i].setLabel(String.valueOf(dayOfMonth));                
                    daysNumbers[i].setBackground(WHITE);
                    Font nobold = new Font("nobold",Font.PLAIN,11);
                    daysNumbers[i].setFont(nobold);
                    //----------------------------------- to put in value the current day 
                    if(formatDateRecup.format(calendarActu.getTime()).equals(formatDateRecup.format(calendar.getTime()))){
                    	Color gray = new Color(192,224,178);
                    	daysNumbers[i].setBackground(gray);
                    	Font bold = new Font("bold",Font.BOLD,11);
                    	daysNumbers[i].setFont(bold);
                    }
                    //---------------------------------- to put in value the selected day
                    if(actualDateValue > 0){
                        if(formatDateRecup.format(calendar.getTime()).equals(formatDateRecup.format(new Date(actualDateValue)))){
                        	Color blue = new Color(70,185,234);
                        	daysNumbers[i].setBackground(blue);
                        	Font bold = new Font("bold",Font.BOLD,11);
                        	daysNumbers[i].setFont(bold);
                        }
                    }
                    
                    daysNumbers[i].setActionCommand(Long.toString(calendar.getTime().getTime()));
                    calendar.add(Calendar.DAY_OF_MONTH, 1);
                    full = 1 == calendar.get(Calendar.DAY_OF_MONTH);
                }
            }
            calendar.setTime(tmp);
        }catch(Throwable ex){
			LogManager.traceError(0,"Error during execution of service Calendrier, updateDaysNumber() : "+ex );
	   	} 
    }
    
    
    //---------------------------------------------- To screen the Day of the Week
    private void reloadTheDaysofWeek() {
        try{
        	final String[] weekDays = formatDateSymb.getShortWeekdays();
            for (int i = 1; i < weekDays.length; ++i) {
                final int index = (i - 2 + firstDayOfWeek) % 7 + 1;
                daysOfWeek[i - 1].setText(weekDays[index]);
            }	
        }catch(Throwable ex){
			LogManager.traceError(0,"Error during execution of service Calendrier, updateDaysOfWeek() : "+ex );
	   	} 
    }
    
    
    

    public static void main(final String[] args)throws ParseException {  
    	OWCalendar cal = new OWCalendar(230, 300,"close");
    	cal.show();
    }
}