/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.modules;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.ParseException;

import org.openmobileis.common.util.log.LogManager;



public class OWTime extends Frame {
	private static final long serialVersionUID = -6070460129417383057L;

	class WindowsEventHandler extends WindowAdapter {
		public void windowClosing(WindowEvent evt) {
			maskWindows();
		}
	   public void windowIconified(WindowEvent e){
		   maskWindows();
	   }         
	}
	
	private int minChecked, hourChecked, actualminValue, actualhourValue;

    Color WHITE = new Color(255,255,255);
    Color GRAYLITTLE = new Color(240,240,240);
    Color GREEN = new Color(192,224,178);
    Color BLUE = new Color(70,185,234);
    final Button[] hours = new Button[24];
    final Button[] mins = new Button[12];
    
    private final ActionListener hourListener = new ActionListener() {
    	public void actionPerformed(final ActionEvent ae) {
    		for(int i=0;i<hours.length;i++){
    			hours[i].setBackground(WHITE);
    		}
    		hourChecked = Integer.parseInt(ae.getActionCommand());
    		hours[hourChecked].setBackground(BLUE);
    	}
    };
    
    private final ActionListener minListener = new ActionListener() {
    	public void actionPerformed(final ActionEvent ae) {
    		String chaine = ae.getActionCommand();
    		int index = chaine.indexOf(":");
    		minChecked = Integer.parseInt(ae.getActionCommand().substring(index+1, chaine.length()));
    		maskWindows();	
    	}
    };
    
    private final ActionListener closeListener = new ActionListener() {
    	public void actionPerformed(final ActionEvent ae) {
    		maskWindows();
    	}
    };
    
    public synchronized int[] getTimeChecked(){
    	int temp[]={hourChecked, minChecked};
    	return temp; 
    }
    
    public synchronized void showFrame(int actualHourvalue, int actualminValue){
    	this.actualhourValue = actualHourvalue;
    	this.actualminValue = actualminValue;
    	this.hourChecked = actualhourValue;
    	reloadButtonValue();
    	this.setVisible(true);
    }
    
    public void reloadButtonValue(){
   		for(int i=0;i<hours.length;i++){
			hours[i].setBackground(WHITE);
		}
   		for(int i=0;i<mins.length;i++){
   			mins[i].setBackground(GRAYLITTLE);
		}
   		if(actualhourValue != -1){
   			hours[actualhourValue].setBackground(GREEN);	
   		}
   		if(actualminValue != -1){
   			mins[actualminValue/5].setBackground(GREEN);	
   		}
    }
    
    public OWTime(int x, int y, String closeButtonValue, String labelHour, String labelMin) {
    	try{
        	this.addWindowListener(new WindowsEventHandler());
        	
        	Panel main = new Panel();
        	//GridBagLayout gbl = new GridBagLayout();
        	//this.setLayout(gbl);
            //GridBagConstraints constraints = new GridBagConstraints();
        	main.setLayout(new BorderLayout());
        	
        	/*final Panel bighourPanel = new Panel();
        	bighourPanel.setLayout(new GridLayout(1,1));
        	bighourPanel.add(new Label(labelHour));        	
        	main.add(bighourPanel, BorderLayout.PAGE_START);*/
        	
        	final Panel hoursPanel = new Panel();
        	hoursPanel.setLayout(new GridLayout(4, 8));
        	for(int i=0;i<7;i++){
        		if(i==0)
        			hoursPanel.add(new Label(labelHour));
        		hoursPanel.add(new Label());
        	}
        	for(int i=0;i<hours.length;i++){
        		hours[i] = new Button(Integer.toString(i));
                Font bold = new Font("bold",Font.BOLD,11);
                hours[i].addActionListener(hourListener);
                hours[i].setBackground(WHITE);
                hours[i].setFont(bold);
                
                hoursPanel.add(hours[i]);
        	}
        	//bighourPanel.add(hoursPanel);
        	//main.add(bighourPanel, BorderLayout.NORTH);
        	main.add(hoursPanel, BorderLayout.NORTH);
            
            /*final Panel bigminPanel = new Panel();
            bigminPanel.setLayout(new GridLayout(1, 1));
            bigminPanel.add(new Label(labelMin));
            main.add(bigminPanel, BorderLayout.CENTER); */
            
            final Panel minPanel = new Panel();
            minPanel.setLayout(new GridLayout(3, 6));        
        	for(int i=0;i<5;i++){
        		if(i==0)
        			minPanel.add(new Label(labelMin));
        		minPanel.add(new Label());
        	}
        	for(int i=0;i<mins.length;i++){
        		if((i*5)<10){
        			mins[i] = new Button(":0"+Integer.toString(i*5));	
        		}else{
        			mins[i] = new Button(":"+Integer.toString(i*5));
        		}     		
                Font bold = new Font("bold",Font.BOLD,11);
                mins[i].addActionListener(minListener);
                mins[i].setBackground(GRAYLITTLE);
                mins[i].setFont(bold);
                
                minPanel.add(mins[i]);
        	}
        	//bigminPanel.add(minPanel);
        	//main.add(bigminPanel, BorderLayout.SOUTH); 
        	main.add(minPanel, BorderLayout.CENTER);
            
            final Button close = new Button(closeButtonValue);
            close.setBackground(WHITE);
            close.addActionListener(closeListener);
            main.add(close, BorderLayout.SOUTH);

            
            this.add(main);
            this.pack();
            this.setSize(x,y);
            this.setVisible(false);
    	}catch(Throwable ex){
    		LogManager.traceError(0,"Error during execution of service Calendrier, Calendrier() : "+ex );
    	}
    	
    }
    
    
    //------------------------------------------------ Put visible or not the Windows, to win time when screen on the time component
    public void maskWindows()	{
		try{
	    	//super.dispose();
			this.setVisible(false);
			synchronized(this)	{
				this.notifyAll();
			}
    	}catch(Throwable ex){
    		LogManager.traceError(0,"Error during execution of service Time, maskWindows() : "+ex );
    	}

	}
    
 
    

    public static void main(final String[] args)throws ParseException {  
    	OWTime cal = new OWTime(230, 300,"close","hour","minute");
    	cal.show();
    }
}