/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.services;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.Report;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

public class ServiceDemoDisplayReport extends TemplateService implements NavigationBarService {
	
	private final static String templateName = "demotemplates/display_report.htm";

	public String runTemplate(HttpServletRequest req, HttpServletResponse res,TemplateModelRoot templateData) throws ServletException,IOException {
		try{
			String reportID = req.getParameter("reportID");
			String interID = req.getParameter("interID");
			
			if(reportID != null && reportID.length()>0 && interID != null && interID.length()>0){
				Report report = InterventionManager.getManager().getReportFromReportIDAndInterID(reportID, interID);
				templateData.put("cptRendu", new SimpleScalar(report.getCptRendu()));
				templateData.put("reportID", new SimpleScalar(report.getIDReport()));
				long datelong = report.getDateLong();
				Date date = new Date(datelong);
				SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yy HH:mm", java.util.Locale.FRANCE);
				String dateForm = dateFomrat.format(date);
				templateData.put("dateReport", new SimpleScalar(dateForm));
			}
			
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service ServiceDemoDisplayReport run(): "+ex);
		}
		
		return templateName;
	}

	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/report/display";
	}

	public String getNavigationBarLabel(HttpServletRequest req) {
		return "report";
	}

	public boolean displayFormExitMessage() {
		return false;
	}

	public boolean displayRecursive() {
		return false;
	}

}
