/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.services;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.services.MainSubEditService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

public class ServiceDemoEditIntervention extends MainSubEditService implements NavigationBarService {
	private static int uniqID = 0;
	private String idInter, gps;
	boolean isFromGvSIG = false;
	
	public final static String STRING_PRIORITE_FAIBLE = "Faible";
	public final static String STRING_PRIORITE_NORMAL = "Normal";
	public final static String STRING_PRIORITE_HAUTE = "Haute";

	protected void fillTemplateWithSessionData(Object sessionDatas,TemplateModelRoot templateData) throws ServiceException {
		
		templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
		Intervention inter = (Intervention)sessionDatas;
		this.idInter = inter.getIDInter();
		templateData.put("interID", new SimpleScalar(inter.getIDInter()));
		templateData.put("demande", new SimpleScalar(inter.getDemande()));
		
		//templateData.put("equipement", new SimpleScalar(inter.getEquipement()));
		//templateData.put("resume", new SimpleScalar(inter.getResume()));
		templateData.put("adress", new SimpleScalar(inter.getAdress()));
		templateData.put("nomClient", new SimpleScalar(inter.getNomClient()));
		
		//------------------------ RESUME ----------------------------------------
		Array resumes = new Array();
		resumes.add("POMPE A CHALEUR");
		resumes.add("CHAUFFE-EAU SOLAIRE");
		resumes.add("CLIMATISATION");
		
		SimpleList listResume = new SimpleList();
		for (int i = 0; i < resumes.size(); i++) {
			String string = (String)resumes.get(i);
			SimpleHash listElement = new SimpleHash();
			listElement.put("value", string);
			
			String resume = inter.getResume();

			if (string.equals(resume)) {
				listElement.put("select", true);
			} else {
				listElement.put("select", false);
			}
			listResume.add(listElement);
		}
	
		templateData.put("respListResume", listResume);
		
		
		//------------------------ EQUIPEMENT -------------------------------------
		Array equi = new Array();
		equi.add("AIXIA_PAC");
		equi.add("DAIKIN_PAC");
		equi.add("CESI_CHES");
		equi.add("PHOENIX_CHES");
		equi.add("CARRIER_CLIM");
		equi.add("LG_CLIM");
		
		SimpleList listEqui = new SimpleList();
		for (int i = 0; i < equi.size(); i++) {
			String string = (String)equi.get(i);
			SimpleHash listElement = new SimpleHash();
			listElement.put("value", string);
			
			String equip = inter.getEquipement();

			if (string.equals(equip)) {
				listElement.put("select", true);
			} else {
				listElement.put("select", false);
			}
			listEqui.add(listElement);
		}
	
		templateData.put("respListEqui", listEqui);
		
		
		
		//------------------------ PRIORITE ----------------------------------------
		Array arrayLabel = new Array();
		arrayLabel.add(STRING_PRIORITE_FAIBLE);
		arrayLabel.add(STRING_PRIORITE_NORMAL);
		arrayLabel.add(STRING_PRIORITE_HAUTE);
		SimpleList list = new SimpleList();
		SimpleHash listElement;

		if (arrayLabel != null) {

			for (int i = 0; i < arrayLabel.size(); i++) {
				String string = (String)arrayLabel.get(i);
				listElement = new SimpleHash();
				listElement.put("value", string);
				
				String prio = STRING_PRIORITE_FAIBLE;
				if(inter.getPriorite() == Intervention.PRIORITE_NORMAL){
					prio = STRING_PRIORITE_NORMAL;
				}
				if(inter.getPriorite() == Intervention.PRIORITE_HAUTE){
					prio = STRING_PRIORITE_HAUTE;
				}
				if (string.equals(prio)) {
					listElement.put("select", true);
				} else {
					listElement.put("select", false);
				}
				list.add(listElement);
			}
		}
		templateData.put("respListStatut", list);
		
		/*String adress = inter.getAdress();
		if(adress != null){
			templateData.put("adresse", new SimpleScalar(adress));
		}
		 */
		
		int hour = 0;
		int min = 0;
		// **************************** Start Date *****************************************//
		
		SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");
		Calendar cal = Calendar.getInstance();		
		
		if(inter.getDateDebut() != 0){
			cal.setTime(new Date(inter.getDateDebut()));
			hour = cal.get(Calendar.HOUR_OF_DAY);
			min = cal.get(Calendar.MINUTE);
		}else{
			hour = cal.get(Calendar.HOUR_OF_DAY);
			min = 0;
		}
		templateData.put("datedebut", new SimpleScalar(formatDate.format(cal.getTime())));	
		templateData.put("datedebutLong", new SimpleScalar(Long.toString(cal.getTime().getTime())));		  
		
		if(min <10 && hour <10){		
			templateData.put("heuredebut", new SimpleScalar("0"+hour+":0"+min));
		}else if(min <10){
			templateData.put("heuredebut", new SimpleScalar(hour+":0"+min));	
		}else if(hour <10){
			templateData.put("heuredebut", new SimpleScalar("0"+hour+":"+min));
		}else{
			templateData.put("heuredebut", new SimpleScalar(hour+":"+min));	
		}
		templateData.put("heuredebutValue", new SimpleScalar(Integer.toString(hour)));
		templateData.put("mindebutValue", new SimpleScalar(Integer.toString(min)));
		
		//**************************** Duree Inter *******************************************//
		int temp = inter.getDuree();
		hour = temp/60; 
		min = temp - (hour*60);
		if(min <10 && hour <10){		
			templateData.put("duree", new SimpleScalar("0"+hour+":0"+min));
		}else if(min <10){
			templateData.put("duree", new SimpleScalar(hour+":0"+min));	
		}else if(hour <10){
			templateData.put("duree", new SimpleScalar("0"+hour+":"+min));
		}else{
			templateData.put("duree", new SimpleScalar(hour+":"+min));	
		}
		templateData.put("heuredureeValue", new SimpleScalar(Integer.toString(hour)));
		templateData.put("mindureeValue", new SimpleScalar(Integer.toString(min)));
		
		

	}

	protected void storeSessionObjectInDB(Object sessionDatas,HttpServletRequest req, TemplateModelRoot templateData)throws ServiceException {
		boolean isErrors = false;
		
		Intervention inter = (Intervention)sessionDatas;
	    
		String demande = inter.getDemande();
		if(demande == null || demande.length() <= 0){
			super.setInputRequestParameterError(req,"the objective must be present");
			isErrors = true;
		}
		
		String resume = inter.getResume();
		if(resume == null || resume.length() <= 0){
			super.setInputRequestParameterError(req,"the resume must be present");
			isErrors = true;
		}
		
		/*int duree = inter.getDuree();
		if (duree == 0)	{
		  super.setInputRequestParameterError(req,"duree incorrect");
		  isErrors = true;
		}
		*/
		
		/*long start = inter.getDateDebut();
		if (start == 0)	{
		  super.setInputRequestParameterError(req,"date incorrect");
		  isErrors = true;
		}*/
		
		if(isErrors != true){
		    try	{
		    	InterventionManager.getManager().store(inter);
		    }catch (Throwable ex)	{
		    	throw new ServiceException(ex);
		    }
		}

	}

	protected void fillSessionObjectWithRequestData(Object sessionDatas,HttpServletRequest req) throws ServiceException {
		Intervention inter = (Intervention) sessionDatas;
		req.getSession(true);
		if("1".equals(req.getParameter("isFromGvSIG"))){
			isFromGvSIG = true;
		}
		if("1".equals(req.getParameter("gps"))){
			gps = "1";
		}
					
	    String startDate = req.getParameter("startDate");
	    String startHour = req.getParameter("startHour");
	    String startMinutes = req.getParameter("startMinutes");
	    
	    String dureeHour = req.getParameter("dureeHour");
	    String dureeMinutes = req.getParameter("dureeMinutes");
		
	    if(dureeHour!= null && dureeMinutes != null && dureeHour.length() >0 && dureeMinutes.length()>0){
	    	inter.setDuree((Integer.parseInt(dureeHour)*60) + Integer.parseInt(dureeMinutes));
	    }
	    
	    if(startDate!= null && startHour != null && startMinutes != null && startDate.length()>0 && startHour.length()>0 && startMinutes.length()>0){
	    	Calendar cal = Calendar.getInstance();
			cal.setLenient(false);
			cal.setTime(new Date(Long.parseLong(startDate)));
			cal.set(Calendar.HOUR_OF_DAY,Integer.parseInt(startHour));
			cal.set(Calendar.MINUTE,Integer.parseInt(startMinutes));
			
			//Date test = cal.getTime();
			long date = cal.getTime().getTime();
			inter.setDateDebut(date);
	    }
		  
		String temp = (String)req.getParameter("typeStatutSelect");
		if (temp != null && temp.length()>0)	{
			if(temp.equals(STRING_PRIORITE_NORMAL)){
				inter.setPriorite(Intervention.PRIORITE_NORMAL);
			}
			if(temp.equals(STRING_PRIORITE_FAIBLE)){
				inter.setPriorite(Intervention.PRIORITE_FAIBLE);
			}
			if(temp.equals(STRING_PRIORITE_HAUTE)){
				inter.setPriorite(Intervention.PRIORITE_HAUTE);
			}

		}
	    
		temp = (String)req.getParameter("demande");
		if (temp != null&& temp.length()>0)	{
		  inter.setDemande(temp);
		}
		temp = (String)req.getParameter("equipement");
		if (temp != null&& temp.length()>0)	{
		  inter.setEquipement(temp);
		}
		
		temp = (String)req.getParameter("resume");
		if (temp != null&& temp.length()>0)	{
		  inter.setResume(temp);
		}
		
		temp = (String)req.getParameter("adress");
		if (temp != null&& temp.length()>0)	{
		  inter.setAdress(temp);
		}
		
		/*temp = (String)req.getParameter("nomClient");
		if (temp != null&& temp.length()>0)	{
		  inter.setNomClient(temp);
		}*/
		try{
			//TerminalUser tuser = TerminalUserManager.getManager().getInstallTerminalUser();
			//User user = UserManager.getManager().getUser("default", "1");
			
			String codeTech = ((Intervention)InterventionManager.getManager().getAllInterventions().get(0)).getCodeTech();
			if(codeTech != null){
				String name = "";
				if(codeTech.equals("1")){
					name = "toto";
				}
				if(codeTech.equals("2")){
					name = "tata";
				}
				inter.setNomClient("DEMANDE TECHNICIEN : "+name+" - "+inter.getIDInter());
			}
			
		}catch(Exception e){
			LogManager.traceError(0, e);
		}
		
		/*TerminalUser user = TerminalUserManager.getManager().getInstallTerminalUser();
		inter.setCodeTech(user.getTerminalUserId());*/
		inter.setStatut(Intervention.STATUT_DEMANDE);

		SessionContext session = SessionContextManager.getManager().getSessionContext();
		Hashtable hashDate = (Hashtable)session.getAttribute("O3.calendar.date.checked");
		Hashtable hashTime = (Hashtable)session.getAttribute("O3.calendar.time.checked");
		long dateLong = 0;
		int minutes = 0;
		int hour = 0;
		if(hashDate != null && hashDate.size() > 0){
		   java.util.Enumeration enume = hashDate.keys();
		   String strDate = (String)hashDate.get(enume.nextElement());
			if (strDate != null && strDate.length() != 0) {
				try {
					dateLong = Long.parseLong(strDate);
					
					Calendar cal = Calendar.getInstance();
					cal.setLenient(false);
					if(hashDate.containsKey("debut")){	
						//------- keep hours and minutes of the old date
						hour = Integer.parseInt(req.getParameter("startHour"));
						minutes = Integer.parseInt(req.getParameter("startMinutes"));
						//----- construct the new date with the old parameters
						cal.setTime(new Date(dateLong));
						cal.set(Calendar.HOUR_OF_DAY,hour);
						cal.set(Calendar.MINUTE,minutes);
						
						long date = cal.getTime().getTime();
						inter.setDateDebut(date);
					}
				} catch (NumberFormatException ex) {
					LogManager.traceError(LogServices.WEBSERVICE, "invalid strDate : " + strDate);
				}
			}
		}
		if(hashTime != null && hashTime.size() > 0){
		   java.util.Enumeration enume = hashTime.keys();
		   int[] time = (int[])hashTime.get(enume.nextElement());
			if (time != null && time.length != 0) {
				try {
					hour = time[0];
					minutes = time[1];
					//--------Date et Heure
					Calendar cal = Calendar.getInstance();
					cal.setLenient(false);
					if(hashTime.containsKey("debut")){					
						cal.setTime(new Date(Long.parseLong(req.getParameter("startDate"))));
						cal.set(Calendar.HOUR_OF_DAY,hour);
						cal.set(Calendar.MINUTE,minutes);
						
						long date = cal.getTime().getTime();
						inter.setDateDebut(date);
					}
					if(hashTime.containsKey("duree")){					
						inter.setDuree((hour*60) + minutes);
					}
				} catch (NumberFormatException ex) {
					LogManager.traceError(LogServices.WEBSERVICE, "invalid time[] : " + time);
				}
			}
		}
		
	  session.removeAttribute("O3.calendar.date.checked");
	  session.removeAttribute("O3.calendar.time.checked");

	}

	protected String getSessionDatasName() {
		return "demo.edition.inter";
	}

	protected String getTemplateName() {
		return "demotemplates/edit_inter.htm";
	}

	protected Object createSessionObject(HttpServletRequest req)throws ServiceException {
		try{
			String idInter = req.getParameter("interID");
			
			if(idInter != null && idInter.length()>0){
				Intervention inter = InterventionManager.getManager().getIntervention(idInter);
				if(inter != null){
					return inter;
				}else{
					return new DefaultIntervention(idInter);
				}
			}else{
				return new DefaultIntervention();			
			}	
			
		}catch (Throwable ex)	{
	    	throw new ServiceException(ex);
	    }	

	}

	protected String getDisplayServiceURI() {
		if(isFromGvSIG){
			isFromGvSIG = false;
			if("1".equals(gps)){
				gps=null;
				return ServiceManager.getManager().getServiceBaseURI()+"/com/store/object?redirectURL=/demandeInter/liste&gps=1&idObject="+idInter;
			}else{
				return ServiceManager.getManager().getServiceBaseURI()+"/com/store/object?redirectURL=/demandeInter/liste&idObject="+idInter;				
			}
	
		}else{
			return ServiceManager.getManager().getServiceBaseURI()+"/index";
		}
		
	}

	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/inter/edit";
	}

	public String getNavigationBarLabel(HttpServletRequest req) {
		return "demande inter";
	}

	public boolean displayFormExitMessage() {
		return true;
	}

	public boolean displayRecursive() {
		return false;
	}

}
