/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.services;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;

import org.openmis.application.demo.common.data.DefaultReport;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.common.data.Report;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.services.MainSubEditService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

public class ServiceDemoEditReport extends MainSubEditService implements NavigationBarService {

	private static int uniqID = 0;

	protected void fillTemplateWithSessionData(Object sessionDatas, TemplateModelRoot templateData) throws ServiceException {
		templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
		Report report = (Report)sessionDatas;
		
		templateData.put("comment", new SimpleScalar(report.getCptRendu()));
		templateData.put("reportID", new SimpleScalar(report.getIDReport()));
		templateData.put("interID", new SimpleScalar(report.getIDInter()));
		
		
		int hour = 0;
		int min = 0;
		// **************************** Start Date *****************************************//
		
		SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");
		Calendar cal = Calendar.getInstance();		
		
		if(report.getDateLong() != 0){
			cal.setTime(new Date(report.getDateLong()));
			hour = cal.get(Calendar.HOUR_OF_DAY);
			min = cal.get(Calendar.MINUTE);
		}else{
			hour = cal.get(Calendar.HOUR_OF_DAY);
			min = 0;
		}
		templateData.put("datedebut", new SimpleScalar(formatDate.format(cal.getTime())));	
		templateData.put("datedebutLong", new SimpleScalar(Long.toString(cal.getTime().getTime())));		  
		
		if(min <10 && hour <10){		
			templateData.put("heuredebut", new SimpleScalar("0"+hour+":0"+min));
		}else if(min <10){
			templateData.put("heuredebut", new SimpleScalar(hour+":0"+min));	
		}else if(hour <10){
			templateData.put("heuredebut", new SimpleScalar("0"+hour+":"+min));
		}else{
			templateData.put("heuredebut", new SimpleScalar(hour+":"+min));	
		}
		templateData.put("heuredebutValue", new SimpleScalar(Integer.toString(hour)));
		templateData.put("mindebutValue", new SimpleScalar(Integer.toString(min)));

	}

	protected void storeSessionObjectInDB(Object sessionDatas, HttpServletRequest req, TemplateModelRoot templateData) throws ServiceException {
		boolean isErrors = false;
		
		Report report = (Report)sessionDatas;
	    
		String cptRendu = report.getCptRendu();
		if(cptRendu == null || cptRendu.length() <= 0){
			super.setInputRequestParameterError(req,"the comment must be present");
			isErrors = true;
		}
		
		/*long start = report.getDateLong();
		if (start == 0)	{
		  super.setInputRequestParameterError(req,"date incorrect");
		  isErrors = true;
		}*/
		
		if(isErrors != true){
		    try	{
				Intervention inter = InterventionManager.getManager().getIntervention(report.getIDInter());
				Array array = inter.getArrayReport();	//get old array reports from inter
				if(array == null)
					array = new Array();
				boolean exist = false; 
				for(int i=0;i<array.size();i++){	//search if report already exist in intervention
					Report temp = (Report)array.get(i);
					if(temp.getIDReport().equals(report.getIDReport())){
						array.replace(i, report);	//replace it
						exist = true;
					}
				}
				if(!exist)
					array.add(report);	//put new report in array if not exist in intervention
				
				inter.setArrayReport(array);	//put new array reports in inter
				InterventionManager.getManager().store(inter);	//store inter
				
		    }catch (Throwable ex)	{
		    	throw new ServiceException(ex);
		    }
		}
	}

	protected void fillSessionObjectWithRequestData(Object sessionDatas, HttpServletRequest req) throws ServiceException {
		Report report = (Report) sessionDatas;
		req.getSession(true);
		
	    String startDate = req.getParameter("startDate");
	    String startHour = req.getParameter("startHour");
	    String startMinutes = req.getParameter("startMinutes");
		
	    if(startDate!= null && startHour != null && startMinutes != null && startDate.length()>0 && startHour.length()>0 && startMinutes.length()>0){
	    	Calendar cal = Calendar.getInstance();
			cal.setLenient(false);
			cal.setTime(new Date(Long.parseLong(startDate)));
			cal.set(Calendar.HOUR_OF_DAY,Integer.parseInt(startHour));
			cal.set(Calendar.MINUTE,Integer.parseInt(startMinutes));
			
			//Date test = cal.getTime();
			long date = cal.getTime().getTime();
			report.setDateLong(date);
	    }
		  
		String temp = (String)req.getParameter("comment");
		if (temp != null)	{
		  report.setCptRendu(temp);
		}
		
		SessionContext session = SessionContextManager.getManager().getSessionContext();
		Hashtable hashDate = (Hashtable)session.getAttribute("O3.calendar.date.checked");
		Hashtable hashTime = (Hashtable)session.getAttribute("O3.calendar.time.checked");
		long dateLong = 0;
		int minutes = 0;
		int hour = 0;
		if(hashDate != null && hashDate.size() > 0){
		   java.util.Enumeration enume = hashDate.keys();
		   String strDate = (String)hashDate.get(enume.nextElement());
			if (strDate != null && strDate.length() != 0) {
				try {
					dateLong = Long.parseLong(strDate);
					
					Calendar cal = Calendar.getInstance();
					cal.setLenient(false);
					if(hashDate.containsKey("debut")){	
						//------- keep hours and minutes of the old date
						hour = Integer.parseInt(req.getParameter("startHour"));
						minutes = Integer.parseInt(req.getParameter("startMinutes"));
						//----- construct the new date with the old parameters
						cal.setTime(new Date(dateLong));
						cal.set(Calendar.HOUR_OF_DAY,hour);
						cal.set(Calendar.MINUTE,minutes);
						
						long date = cal.getTime().getTime();
						report.setDateLong(date);
					}
				} catch (NumberFormatException ex) {
					LogManager.traceError(LogServices.WEBSERVICE, "invalid strDate : " + strDate);
				}
			}
		}
		if(hashTime != null && hashTime.size() > 0){
		   java.util.Enumeration enume = hashTime.keys();
		   int[] time = (int[])hashTime.get(enume.nextElement());
			if (time != null && time.length != 0) {
				try {
					hour = time[0];
					minutes = time[1];
					//--------Date et Heure
					Calendar cal = Calendar.getInstance();
					cal.setLenient(false);
					if(hashTime.containsKey("debut")){					
						cal.setTime(new Date(Long.parseLong(req.getParameter("startDate"))));
						cal.set(Calendar.HOUR_OF_DAY,hour);
						cal.set(Calendar.MINUTE,minutes);
						
						long date = cal.getTime().getTime();
						report.setDateLong(date);
					}
				} catch (NumberFormatException ex) {
					LogManager.traceError(LogServices.WEBSERVICE, "invalid time[] : " + time);
				}
			}
		}
		
	  session.removeAttribute("O3.calendar.date.checked");
	  session.removeAttribute("O3.calendar.time.checked");
	}

	protected String getSessionDatasName() {
		return "demo.edition.report";
	}

	protected String getTemplateName() {
		return "demotemplates/edit_report.htm";
	}

	protected Object createSessionObject(HttpServletRequest req) throws ServiceException {
		try{
			String idReport = req.getParameter("reportID");
			String idInter = req.getParameter("interID");
			
			if(idReport != null && idReport.length()>0){
				Report report = InterventionManager.getManager().getReportFromReportIDAndInterID(idReport, idInter);
				if(report != null)
					return report;
			}else{
				Report report = new DefaultReport(idInter);
				return report;
			}	
			
		}catch (Throwable ex)	{
	    	throw new ServiceException(ex);
	    }	
		return null;
	}
	

	protected String getDisplayServiceURI() {
		return ServiceManager.getManager().getServiceBaseURI()+"/inter/display";
	}

	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/report/edit";
	}

	public String getNavigationBarLabel(HttpServletRequest req) {
		return "edit report";
	}

	public boolean displayFormExitMessage() {
		return true;
	}

	public boolean displayRecursive() {
		return false;
	}

}

