/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.com.inter.agents.conduit.ConduitParameter;
import org.openmis.com.inter.agents.conduit.HTTPClientLocalCommunicationConduit;
import org.openmis.tellus.synchro.common.data.DefaultSynchroProject;
import org.openmis.tellus.synchro.common.data.SynchroGeograhicProfil;
import org.openmis.tellus.synchro.common.data.SynchroProject;
import org.openmis.tellus.synchro.common.utils.FileLayerUtilities;
import org.openmis.tellus.synchro.terminal.project.ProjectProfilManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import HTTPClient.HTTPResponse;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class ServiceDemoListProjectSynchro extends TemplateService implements NavigationBarService {
	
	private final static String templateName = "demotemplates/synchroList.htm";

	public String runTemplate(HttpServletRequest req, HttpServletResponse res,TemplateModelRoot templateData) throws ServletException,IOException {
		
		try{
			ConduitParameter form_data[] = new ConduitParameter[1];
			form_data[0] = new ConduitParameter("projectsRequest", "REQUEST FOR LIST OF SIG PROJECTS");
			
			int port = Integer.parseInt(PropertiesManager.getManager().getProperty("sig.server.port"));
			String host = PropertiesManager.getManager().getProperty("sig.server.host");
			String uri = PropertiesManager.getManager().getProperty("sig.server.projects.uri");
			
			HTTPResponse resp =  HTTPClientLocalCommunicationConduit.sendRequest(form_data
					, port
					, host
					, uri);
			String soapResponse =  resp.getHeader("projectsResponse");
			
			if(soapResponse != null && soapResponse.length() > 0){
				Array array = (Array)FileLayerUtilities.unserializeData(soapResponse);
				
				SimpleList list = new SimpleList();
			    list = displayList(array, list);
			    templateData.put("respList",list);
			    
		        SynchroGeograhicProfil profil = new SynchroGeograhicProfil();

				for (int i=0;i<array.size();i++){
					String prjName = (String)array.get(i);
		        	SynchroProject project = new DefaultSynchroProject(prjName);
			        profil.addSynchroProject(project);
				}
				
				ProjectProfilManager.getManager().addSynchroGeograhicProfil(profil);
			}	
			
		}catch(Exception ex){
			LogManager.traceError(0, ex);
			templateData.put("error", new SimpleScalar("Connexion au serveur impossible"));
		}
		
		return templateName;
	}


	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/synchrolist";
	}


	public String getNavigationBarLabel(HttpServletRequest req) {
		return "projets";
	}


	public boolean displayFormExitMessage() {
		return false;
	}


	public boolean displayRecursive() {
		return false;
	}
	
 	private SimpleList displayList (Array array, SimpleList list) {
		SimpleHash listElement;
		
		for(int i = 0; i < array.size(); i++) {
			listElement = new SimpleHash();
			String prj = (String)array.get(i);	
			listElement.put("projectName", prj);
			list.add(listElement);		
		}
		return list;
	}


}
