/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.application.demo.terminal.services;

import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.application.demo.common.data.DefaultIntervention;
import org.openmis.application.demo.common.data.Intervention;
import org.openmis.application.demo.manager.InterventionManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServerException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * @author Manuel Gomez
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ServiceDemoListeDemandeInter extends TemplateService implements NavigationBarService {

  private final static String templateName = "demotemplates/liste_Demande_inter.htm";
  private static int uniqID = 0;


   /* (non-Javadoc)
    * @see org.openmobileis.embedded.services.common.TemplateService#runTemplate(org.openmobileis.embedded.webserver.http.HttpServletRequest, org.openmobileis.embedded.webserver.http.HttpServletResponse, freemarker.template.TemplateModelRoot)
    */
   public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData){
     
   try	{
		templateData.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
		String errorMess = req.getParameter("error");
		if(errorMess != null && errorMess.length()>0)
			templateData.put("error", new SimpleScalar("Error GvSIG appended - "+errorMess));
		
		
 		SimpleList list = new SimpleList();

 		Array allInter = InterventionManager.getManager().getInterventionsForStatut(Intervention.STATUT_DEMANDE);
 		
	      if( (allInter != null) && (allInter.size() != 0) ){
	        list = displayListRapport(allInter, list);
	        templateData.put("respList",list);
	        
	      }else{
	        
	        templateData.put("isDemande",new SimpleScalar("Aucune demande d'intervention r&eacute;alis&eacute;e")); 
	      }
 			
       return templateName;
       
 		} catch (Throwable ex)	{
 			LogManager.trace(new ServerException("Error during execution of Service "+this.getName(), ex));
 			return null;
 		}
   }
   

 	public String getNavigationBarLabel(HttpServletRequest req) {
 			
 		return "demande";
 	}

 	public boolean displayFormExitMessage() {
 	
 		return false;
 	}
 	
 	public String getName() {
 	  return "DemandeInter.Liste";
 	}
 	
	  public String getServiceUri() {
		    return ServiceManager.getManager().getServiceBaseURI()+"/demandeInter/liste";
	  }

 	public boolean displayRecursive() {
 		
 		return false;
 	}
 	
 	
 	private SimpleList displayListRapport (Array RappList, SimpleList list) {
		SimpleHash listElement;
		DefaultIntervention tempInter = null;
		int sizeid = RappList.size();
		
		for(int i = 0; i < sizeid; i++) {
				listElement = new SimpleHash();
				tempInter = (DefaultIntervention)RappList.get(i);
				  
				listElement.put("interID", tempInter.getIDInter());
				long dateDemande = tempInter.getDateDebut();
				Date date = new Date(dateDemande);
				SimpleDateFormat dateFomrat = new SimpleDateFormat("dd/MM/yyyy HH:mm", java.util.Locale.FRANCE);
				String dateForm = dateFomrat.format(date);
				listElement.put("dateDem",dateForm);	
				listElement.put("object", tempInter.getResume());
				
				
				listElement.put("interID", tempInter.getIDInter());
				listElement.put("nom",tempInter.getNomClient());
				listElement.put("resume",tempInter.getResume());
					
				long datelong = tempInter.getDateDebut();
				date = new Date(datelong);
				dateFomrat = new SimpleDateFormat("dd/MM/yyyy HH:mm", java.util.Locale.FRANCE);
				dateForm = dateFomrat.format(date);
				listElement.put("dateDebut",dateForm);
				
				datelong = tempInter.getDateFin();
				date = new Date(datelong);
				dateForm = dateFomrat.format(date);
				listElement.put("dateFin",dateForm);
				
				
				list.add(listElement);
				
			}
		
		return list;
	}
 	

}
