/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap;

import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.BoundedShapes;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.RasterAdapter;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.RasterFileAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FAcetateLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerGenericRaster;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.cancel.Cancellable;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.FBitSet;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPortListener;

import org.apache.log4j.Logger;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import java.io.IOException;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * This class keeps and manages geographic data (basically the viewport and the
 * array of layers). Therefore deals with zooms, extents, selection, etc, and
 * gives access to the management of teh viewport and the layer drawing.
 *
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort
 * @see es.prodevelop.gvsig.mobile.fmap.layer.FLayer
 * 
 * @author jldominguez
 *
 */
public class MapContext {
    private static Logger logger = Logger.getLogger(MapContext.class);
    public static final double[] CHANGEM = {
            1000, 1, 0.01, 0.001, 1609.344, 0.9144, 0.3048, 0.0254
        };
    public static final double[] CHANGE = {
            100000, 100, 1, 0.1, 160934.4, 91.44, 30.48, 2.54
        };
    public static final int EQUALS = 0;
    public static final int DISJOINT = 1;
    public static final int INTERSECTS = 2;
    public static final int TOUCHES = 3;
    public static final int CROSSES = 4;
    public static final int WITHIN = 5;
    public static final int CONTAINS = 6;
    public static final int OVERLAPS = 7;
    public static double ZOOMINFACTOR = 2;
    public static double ZOOMOUTFACTOR = 0.5;
    public static double ASSUMED_DPI = 96.0;
    
    private ArrayList layers = new ArrayList();
    private FAcetateLayer acetateLyr = null;
    private ViewPort viewPort;

    public MapContext(ViewPort vp) {
        setViewPort(vp);
    }

    /**
     * @return All layers in the current view
     */
    public ArrayList getLayers() {
        return layers;
    }

    /**
     * Gets i-th layer
     *
     * @param index
     * @return the i-th layer
     */
    public FLayer getLayer(int index) {
        if (index >= layers.size()) {
            return null;
        }

        return (FLayer) layers.get(index);
    }

    /**
     * Get current view scale.
     *
     * @return Returns d when current scale is 1:d (assumes dpi = ASSUMED_DPI)
     */
    public long getViewScale() {
        IProjection proj = viewPort.getProjection();

        if (viewPort.getImageSize() == null) {
            return -1;
        }

        double w = ((viewPort.getImageSize().width / ASSUMED_DPI) * 2.54);

        if (viewPort.getAdjustedExtent() == null) {
            return 0;
        }

        if (proj == null) {
            return (long) (viewPort.getAdjustedExtent().getWidth() / w * CHANGE[getViewPort()
                                                                                    .getMapUnits()]);
        }

        return Math.round(proj.getScale(viewPort.getAdjustedExtent().getMinX(),
                viewPort.getAdjustedExtent().getMaxX(),
                viewPort.getImageSize().width, ASSUMED_DPI));
    }

    /**
     *
     * @return Get current projection
     */
    public IProjection getProjection() {
        return getViewPort().getProjection();
    }

    /**
     * Sets a new projection.
     *
     * @param proj
     *            the new projection
     */
    public void setProjection(IProjection proj) {
        if (getViewPort() != null) {
            getViewPort().setProjection(proj);
        }
    }

    /**
     * Draws visible layers in the provided Graphics object.
     *
     * @param g
     *            the object where layers will be drawn.
     * @param cancel
     *            cancel object (follow this parameter to see if it is used.
     */
    public void draw(Graphics2D g, Cancellable cancel) {
        if (viewPort.getAdjustedExtent() == null) {
            return;
        }

        long t1 = System.currentTimeMillis();
        drawLayers(g, cancel, viewPort);
        
        long t2 = System.currentTimeMillis();

        logger.info("MapContext - Drawing time: " + (t2 - t1) + " mseg.");
        System.gc();
    }


	/**
     * Get visible layers (the ones that will be drawn)
     *
     * @return visible layers
     */
    private FLayer[] getVisibleLayers() {
        ArrayList resp = new ArrayList();
        FLayer item;

        for (int i = 0; i < layers.size(); i++) {
            item = (FLayer) layers.get(i);

            if (item.isVisible()) {
                resp.add(item);
            }
        }

        return (FLayer[]) resp.toArray(new FLayer[0]);
    }

    /**
     * Draws the current layers on the provided Graphics object.
     *
     * @param g
     *            the graphics object where they will be drawn.
     *
     * @param cancel
     *            the cancel obejct (follow this parameter to see if it is used)
     * @param vp
     *            the viewport (in case it's not the same as this object's vp)
     */
    private void drawLayers(Graphics2D g, Cancellable cancel, ViewPort vp) {
        FLayer[] vls = getVisibleLayers();
        int size = vls.length;

        for (int i = 0; i < size; i++) {
            FLayer lyr = vls[i];

            if (cancel.isCanceled()) {
                return;
            }

            try {
                lyr.draw(g, vp);
            }
            catch (FMapDriverException e) {
                logger.error("While drawing layer: " + lyr.getName());
                logger.error("Error message: " + e.getMessage());
            }
        }
    }

    /**
     * Gets the viewport
     *
     * @return the viewport
     */
    public ViewPort getViewPort() {
        return viewPort;
    }

    /**
     * Sets the viewport
     *
     * @param v
     *            the new viewport
     */
    public void setViewPort(ViewPort v) {
        viewPort = v;
    }

    /**
     * Computes the full extent adding the extents of the current layers.
     *
     * @return The current full extent
     *
     * @throws FMapDriverException
     */
    public Rectangle2D getFullExtent() throws FMapDriverException {
        int size = layers.size();

        if (size == 0) {
            return null;
        }

        FLayer lyr = (FLayer) layers.get(0);
        Rectangle2D resp = lyr.getFullExtent();

        for (int i = 1; i < size; i++)
            lyr = (FLayer) layers.get(i);

        resp.add(lyr.getFullExtent());

        return resp;
    }

    /**
     * Removes one layer from this mapcontext
     *
     * @param lyr
     *            the layer to be removed
     */
    public void removeLayer(FLayer lyr) {
        layers.remove(lyr);
        lyr.clean();
    }

    /**
     * Add a layer to this mapcontext object
     *
     * @param lyr
     *            the layer to be added
     * @param repaint
     *            whether or not the view has to be repainted
     *
     */
    public void addLayer(FLayer lyr, boolean repaint) {
        layers.add(lyr);

        if (layers.size() == 1) {
            setViewPortFromLayer((FLayer) layers.get(0), repaint);
        }
    }

    /**
     * Set the provided layer's extent as the new extent (this happens when that
     * layer is added to an empty view)
     *
     * @param layer
     *            the layer where the extent will be used
     * @param repaint
     *            whether or not the view has to be refreshed
     */
    private void setViewPortFromLayer(FLayer layer, boolean repaint) {
        try {
            Rectangle2D newext = getExtentFromFullExtentAndImageSize(layer.getFullExtent(),
                    viewPort.getImageSize());
            viewPort.setExtent(newext, repaint);
        }
        catch (FMapDriverException e) {
            logger.error("Unable to set first view port from layer: " +
                layer.getName(), e);
        }
    }

    /**
     *
     * Computes the minimum rectangle with the width/height ratio that contains
     * the provided extent.
     *
     * @param fext
     *            the extent to be covered
     * @param imgdim
     *            the size of the screen
     * @return the minimum rectangle with the width/height ratio that contains
     *         the provided extent.
     */
    private Rectangle2D getExtentFromFullExtentAndImageSize(Rectangle2D fext,
        Dimension imgdim) {
        double ext_ratio = fext.getWidth() / fext.getHeight();
        double img_ratio = (1.0 * imgdim.width) / (1.0 * imgdim.height);

        if (ext_ratio == img_ratio) {
            // same ratio
            return fext;
        }
        else {
            if (ext_ratio > img_ratio) {
                // cinemascope
                double newexth = fext.getWidth() / img_ratio;
                double yoff = 0.5 * (newexth - fext.getHeight());

                return new Rectangle2D.Double(fext.getMinX(),
                    fext.getMinY() - yoff, fext.getWidth(), newexth);
            }
            else {
                // canadian flag
                double newextw = fext.getHeight() * img_ratio;
                double xoff = 0.5 * (newextw - fext.getWidth());

                return new Rectangle2D.Double(fext.getMinX() - xoff,
                    fext.getMinY(), newextw, fext.getHeight());
            }
        }
    }

    /**
     * Changes the order of the layers in those indices.
     *
     * @param _a
     *            one of the indices
     * @param _b
     *            the other index
     */
    public void swapFromBottom(int _a, int _b) {
        FLayer[] lyrs = (FLayer[]) layers.toArray(new FLayer[0]);
        int last_index = lyrs.length - 1;

        int a = last_index - _a;
        int b = last_index - _b;

        FLayer aux = lyrs[a];
        lyrs[a] = lyrs[b];
        lyrs[b] = aux;
        layers = new ArrayList();

        for (int i = 0; i < lyrs.length; i++) {
            layers.add(lyrs[i]);
        }
    }

    /**
     * Closes the datasources of the layers. This is called when the application
     * is closed.
     */
    public void closeDataSources() {
        for (int i = 0; i < layers.size(); i++) {
            FLayer lyr = (FLayer) layers.get(i);

            if (lyr instanceof FLayerGenericRaster) {
                FLayerGenericRaster _lyr = (FLayerGenericRaster) lyr;
                RasterAdapter adap = _lyr.getSource();

                if (adap instanceof RasterFileAdapter) {
                    RasterFileAdapter rfadap = (RasterFileAdapter) adap;

                    try {
                        rfadap.stop();
                        /*try {
							rfadap.getDriver().close();
						} catch (IOException e) {
							e.printStackTrace();
						}*/
                    }
                    catch (FMapDriverException e) {
                        logger.error("While stopping layer: " + lyr.getName());
                        logger.error("Error message: " + e.getMessage());
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Set viewport image size
     *
     * @param size
     *            the new image size of the viewport
     */
    public void setViewPortImageSize(Dimension size) {
        viewPort.setImageSize(size);
    }

    /**
     * Adds a listener to the viewport
     *
     * @param l
     *            listener to be added
     */
    public void addViewPortLisneter(ViewPortListener l) {
        viewPort.addViewPortListener(l);
    }

    /**
     * Removes a listener from the viewport
     *
     * @param l
     *            listener to be removed
     */
    public void removeViewPortLisneter(ViewPortListener l) {
        viewPort.removeViewPortListener(l);
    }

    /**
     * Gets the number of layers
     *
     * @return layer count
     */
    public int getLayerCount() {
        return layers.size();
    }

    /**
     * Gets the currently selected layers
     *
     * @param only_vect
     *            whether or not only vector layers have to be included
     * @return array of layers currently selected (so far, only one layer can be
     *         selected)
     */
    public FLayer[] getSelectedLayers(boolean only_vect) {
        FLayer[] infos = getInfoableLayers(only_vect);

        if (infos.length == 0) {
            return null;
        }

        return infos;
    }

    /**
     * Gets the extent of the selected layers
     *
     * @return the extent of the selected layers
     */
    public Rectangle2D getSelectedLayerExtent() {
        FLayer[] ll = getSelectedLayers(false);

        if (ll == null) {
            return null;
        }

        try {
            Rectangle2D resp = ll[0].getFullExtent();

            for (int i = 1; i < ll.length; i++) {
                Rectangle2D.union(resp, ll[i].getFullExtent(), resp);
            }

            return resp;
        }
        catch (FMapDriverException e) {
            logger.error("While getting selected layer's extent: " +
                e.getMessage());

            return null;
        }
    }

    /**
     * Gets the extent of all the layers
     *
     * @return the extent of all the layers
     */
    public Rectangle2D getAllLayersExtent() {
        int size = layers.size();

        if (size == 0) {
            return null;
        }

        Rectangle2D resp = null;

        for (int i = 0; i < size; i++) {
            FLayer lyr = ((FLayer) layers.get(i));

            if (resp == null) {
                try {
                    resp = lyr.getFullExtent();
                }
                catch (FMapDriverException e) {
                    logger.error("While getting selected layer's extent: " +
                        e.getMessage());
                }
            }
            else {
                try {
                    resp.add(lyr.getFullExtent());
                }
                catch (FMapDriverException e) {
                    logger.error("While getting selected layer's extent: " +
                        e.getMessage());
                }
            }
        }

        return resp;
    }

    /**
     * Gets the extent of the selected features of a vector layer
     *
     * @param lyr
     *            the vector layer whose selected features are of interest
     * @return the extent of the selected features of the vector layer
     */
    private Rectangle2D getSelectionBox(FLyrVect lyr) {
        Rectangle2D bb = null;
        VectorialDriver vd = lyr.getSource().getDriver();

        if (vd instanceof BoundedShapes) {
            FBitSet fb = null;

            try {
                fb = lyr.getRecordset().getSelectionSupport().getSelection();
            }
            catch (FMapDriverException fde) {
                logger.error("While getting selection: " + fde.getMessage());

                return null;
            }

            if (fb.isEmpty()) {
                logger.info("No selection in this layer, nothing done.");

                return null;
            }

            int shpcount;

            try {
                shpcount = vd.getShapeCount();
            }
            catch (IOException e1) {
                logger.error("While getting shape count: " + e1.getMessage());

                return null;
            }

            int shptype = -9995;

            try {
                shptype = lyr.getShapeType();
            }
            catch (FMapDriverException e1) {
                logger.error("While getting shape type");
            }

            // --- special case: one point is selected -------------------------
            if ((fb.cardinality() == 1) && (shptype == FShape.POINT)) {
                int onlysetbit = fb.nextSetBit(0);

                try {
                    IGeometry ig = vd.getShape(onlysetbit);
                    double x = ig.getBounds2D().getCenterX();
                    double y = ig.getBounds2D().getCenterY();
                    double w = viewPort.getAdjustedExtent().getWidth();
                    double h = viewPort.getAdjustedExtent().getHeight();

                    return new Rectangle2D.Double(x - (0.5 * w), y - (0.5 * h),
                        w, h);
                }
                catch (IOException e) {
                    logger.error("While getting shape: " + onlysetbit);
                }

                return viewPort.getAdjustedExtent();
            }

            // -----------------------------------------------------------------
            if (fb.cardinality() == shpcount) {
                Rectangle2D r = null;

                try {
                    r = lyr.getFullExtent();
                }
                catch (FMapDriverException e) {
                    logger.error("While getting layer extent: " +
                        e.getMessage());
                }

                if (r == null) {
                    return null;
                }

                viewPort.setExtent(r, true);

                return null;
            }

            BoundedShapes bs = (BoundedShapes) vd;
            Rectangle2D other = null;
            int setbit = fb.nextSetBit(0);

            try {
                bb = bs.getShapeBounds(setbit);
            }
            catch (IOException e) {
                logger.error("In first shapebounds: " + e.getMessage() +
                    ". Nothing done.");

                return null;
            }

            while (setbit >= 0) {
                setbit = fb.nextSetBit(setbit + 1);

                if (setbit >= 0) {
                    try {
                        other = bs.getShapeBounds(setbit);
                    }
                    catch (IOException e) {
                        logger.error("While getting bbox: " + e.getMessage());
                    }

                    if (other != null) {
                        bb = bb.createUnion(other);
                    }
                }
            }
        }

        return bb;
    }

    /**
     * Zooms to current selection (in all layers)
     */
    public void zoomToSelection() {
        FLayer[] ll = getSelectedLayers(true);

        if (ll == null) {
            return;
        }

        if (ll.length == 0) {
            return;
        }

        Rectangle2D r = null;
        Rectangle2D aux = null;

        for (int c = 0; c < ll.length; c++) {
            aux = getSelectionBox((FLyrVect) ll[c]);

            if (aux != null) {
                if (r == null) {
                    r = aux;
                }
                else {
                    r = r.createUnion(aux);
                }
            }
        }

        if (r != null) {
            viewPort.setExtent(r, true);
        }
    }

    /**
     * Zooms to current selection in one layer
     *
     * @param lyr
     *            the layer of interest
     */
    public void zoomToSelection(FLyrVect lyr) {
        Rectangle2D r = null;
        Rectangle2D aux = null;

        aux = getSelectionBox(lyr);

        if (aux != null) {
            if (r == null) {
                r = aux;
            }
            else {
                r = r.createUnion(aux);
            }
        }

        if (r != null) {
            viewPort.setExtent(r, true);
        }
        else {
            logger.warn("No selection found.");
        }
    }

    /**
     * Gets the layers that the user has selected to be queried for info
     *
     * @param only_vect
     *            whether or not only vector layers have to be included
     * @return the layers that the user has selected to be queried for info
     */
    public FLayer[] getInfoableLayers(boolean only_vect) {
        ArrayList resplist = new ArrayList();
        int size = layers.size();
        FLayer item;

        for (int i = 0; i < size; i++) {
            item = (FLayer) layers.get(i);

            if (only_vect) {
                if ((item instanceof FLyrVect) && (item.isInfoable())) {
                    resplist.add(item);
                }
            }
            else {
                if (item.isInfoable()) {
                    resplist.add(item);
                }
            }
        }

        return (FLayer[]) resplist.toArray(new FLayer[0]);
    }

    /**
     * Gets the complete XML description of this mapcontext. It will include the
     * XML description of the viewport and all the layers
     *
     * @param name
     *            the name of the mapcontext XML element.
     * @param projPath
     *            the project path
     * @return the complete XML description of this mapcontext
     */
    public StringBuffer toXML(String name, String projPath) {
        StringBuffer buff = new StringBuffer();
        buff.append("<" + name + ">\n");

        buff.append(Utils.toXML("viewPort", viewPort));

        buff.append("<layers>\n");

        Iterator it = layers.iterator();

        while (it.hasNext()) {
            FLayer lyr = (FLayer) it.next();
            buff.append(lyr.toXML("layer", projPath));
        }

        buff.append("</layers>\n");

        buff.append("</" + name + ">\n");

        return buff;
    }

    /**
     * Clears selection in all layers
     *
     * @return return true if at least one feature was selected. It used to
     *         decide whether or not the view has to be repainted
     */
    public boolean clearSelection() {
        boolean repaint = false;

        FLayer item = null;

        for (int i = 0; i < layers.size(); i++) {
            item = (FLayer) layers.get(i);

            if (item instanceof FLyrVect) {
                FLyrVect lyrv = (FLyrVect) item;

                try {
                    repaint = repaint |
                        lyrv.getRecordset().clearSelection(false);
                }
                catch (FMapDriverException e) {
                    logger.debug("While clearing selection of " +
                        lyrv.getName() + ": " + e.getMessage());
                }
            }
        }

        return repaint;
    }

	public synchronized FAcetateLayer getAcetateLayer() {
		if(acetateLyr == null)
			 new FAcetateLayer(true);
		return acetateLyr;
	}
	
	public void setFAcetateLayer(FAcetateLayer layer){
		acetateLyr = layer;
	}
}
