/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Checkbox;
import java.awt.Color;
import java.awt.Container;
import java.awt.Image;
import java.awt.Label;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.edition.StopEditing;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerGenericRaster;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;


/**
 * Represents one row in the LayersTable (TOC). Contains several icons and labels
 * to describe the state of the layer.
 *
 * Components are:
 *
 * - Visibility checkbox - user tells whether the layer has to be drawn
 * - Scale visibility icon - indicates scale limits status:
 *
 * 			- scale not defined in layer: no icon
 * 			- current scale outside layer scale limits: gray icon
 * 			- current scale within layer scale limits: color icon
 *
 * - Infoability checkbox - user tells whether layer must be queried by info tool
 * - symbology icon - a sample of the layer's current symbology
 * - layer name - editable from layer settings (one of the dialog buttons)
 *
 *
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTable
 * @see es.prodevelop.gvsig.mobile.fmap.layer.FLayer
 *
 * @author jldominguez
 *
 */
public class LayersTableItem extends Container implements ItemListener, MouseListener {

	private static Logger logger = Logger.getLogger(LayersTableItem.class);

	private boolean selected = true;
	private LayersTable parent;

	private FLayer layer = null;
	private int index = 0;
	private int width = 0;

	//private Checkbox infoable_check = new Checkbox();
	private Checkbox visible_check = new Checkbox();
	///private boolean isEditing = false;
	//private Checkbox edition_check = new Checkbox();
	private LegendIcon legendIcon;
	private Label nameLabel = new Label();
	private ScaleVisibilityIcon scaleVisiState = new ScaleVisibilityIcon();

	private static Image  visiIcon, scaleVisiIcon;//, editionIcon, infoIcon;
	private static String NAME_CAPTION = Messages.getText("LayersTableItem.0"); //$NON-NLS-1$
	private static String TYPE_CAPTION = Messages.getText("LayersTableItem.1"); //$NON-NLS-1$
	static {
		//infoIcon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "toc_info_icon.png"); //$NON-NLS-1$ //$NON-NLS-2$
		visiIcon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "toc_dlg_heading_eye.png"); //$NON-NLS-1$ //$NON-NLS-2$
		scaleVisiIcon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "toc_dlg_heading_eye_scale.png"); //$NON-NLS-1$ //$NON-NLS-2$
		//editionIcon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "editcut.png");
	}

	/**
	 * Used to reset the position of the row to the new index.
	 * @param i
	 */
	public void updateBounds(int i) {
		index = i;
		setBounds(0, i * ROW_STEP, width, ROW_STEP);
	}

	/**
	 * Constructor.
	 *
	 * @param maxw Max width of the row in pixels
	 * @param i Index. It determines the vertical position
	 * @param _parent the TOC it is contained in
	 */
	public LayersTableItem(int maxw, int i, LayersTable _parent) {

		index = i;
		width = maxw;
		parent = _parent;
		setSelected(false);

		//ImageControl info_icon = new ImageControl(infoIcon);
		ImageControl visi_icon = new ImageControl(visiIcon);
		ImageControl scale_icon = new ImageControl(scaleVisiIcon);
		//ImageControl edition_icon = new ImageControl(editionIcon);

		Label type_lbl = new Label(TYPE_CAPTION);
		type_lbl.setAlignment(1);
		type_lbl.setFont(DialogConstants.TABLE_SMALL_ITALIC_FONT);

		Label name_lbl = new Label(NAME_CAPTION);
		name_lbl.setAlignment(1);
		name_lbl.setFont(DialogConstants.TABLE_SMALL_ITALIC_FONT);

		visi_icon.setBounds(
				LEFT_MARGIN, 0, VISIBLE_CHECK_WIDTH, ROW_STEP);
		scale_icon.setBounds(
				LEFT_MARGIN + VISIBLE_CHECK_WIDTH, 0, SCALE_CHECK_WIDTH, ROW_STEP);
		/*info_icon.setBounds(
				LEFT_MARGIN + VISIBLE_CHECK_WIDTH + SCALE_CHECK_WIDTH, 0, INFOABLE_CHECK_WIDTH, ROW_STEP);*/
		type_lbl.setBounds(
				LEFT_MARGIN + VISIBLE_CHECK_WIDTH + SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH,
				0,
				LEGEND_WIDTH, ROW_STEP);
		name_lbl.setBounds(
				LEFT_MARGIN + VISIBLE_CHECK_WIDTH + SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH + LEGEND_WIDTH, 0,
				maxw - (LEFT_MARGIN + VISIBLE_CHECK_WIDTH + SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH + LEGEND_WIDTH + RIGHT_MARGIN),
				ROW_STEP);
		/*edition_icon.setBounds(LEFT_MARGIN + VISIBLE_CHECK_WIDTH
				+ SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH + LEGEND_WIDTH, 0,
				EDITABLE_CHECK_WIDTH, ROW_STEP);*/
		name_lbl
				.setBounds(LEFT_MARGIN + VISIBLE_CHECK_WIDTH
						+ SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH
						+ LEGEND_WIDTH, 0,
						maxw
								- (LEFT_MARGIN + VISIBLE_CHECK_WIDTH
										+ SCALE_CHECK_WIDTH
										+ INFOABLE_CHECK_WIDTH + LEGEND_WIDTH
										+ INFOABLE_CHECK_WIDTH + RIGHT_MARGIN),
						ROW_STEP);

		setLayout(null);

		add(visi_icon);
		add(scale_icon);
	//	add(edition_icon);
		//add(info_icon);
		add(type_lbl);
		add(name_lbl);
	}

	/**
	 * Gets associated layer
	 * @return the associated layer
	 */
	public FLayer getLayer() {
		return layer;
	}

	/**
	 * Updates the symbology sample icon to represent a new symbol.
	 *
	 * @param sym the new symbol to be described by the symbology icon
	 */
	public void updateSymbol(FSymbol sym) {

		legendIcon.setSymbol(sym);
		legendIcon.repaint();

	}

	/**
	 * Constructor.
	 *
	 * @param lyr the layer associated with  this LayersTableItem
	 * @param maxw the max width of the row (in pixels)
	 * @param _parent the layerstable that contains this row
	 */
	public LayersTableItem(FLayer lyr, int maxw, LayersTable _parent) {

		layer = lyr;
		width = maxw;
		parent = _parent;
		setSelected(false);

		if (lyr instanceof FLyrVect) {

			FSymbol sym = ((FLyrVect) lyr).getLegend().getDefaultSymbol();
			int shptype = FShape.NULL;
			try {
				shptype = ((FLyrVect) lyr).getShapeType();
			} catch (FMapDriverException e) {
				logger.error("Al obtener shape type de capa: " + lyr.getName()); //$NON-NLS-1$
			}
			legendIcon = new LegendIcon(shptype, sym);

		} else {
			if (lyr instanceof FLayerGenericRaster) {
				legendIcon = new LegendIcon(LegendIcon.LAYER_RASTER_TYPE);
			} else {
				legendIcon = new LegendIcon();
			}
		}

		visible_check.setBounds(LEFT_MARGIN, 0, VISIBLE_CHECK_WIDTH, ROW_STEP);
		scaleVisiState.setBounds(LEFT_MARGIN + VISIBLE_CHECK_WIDTH, 0,
				SCALE_CHECK_WIDTH, ROW_STEP);
		/*infoable_check.setBounds(LEFT_MARGIN + VISIBLE_CHECK_WIDTH
				+ SCALE_CHECK_WIDTH, 0, INFOABLE_CHECK_WIDTH, ROW_STEP);*/

		legendIcon.setBounds(LEFT_MARGIN + VISIBLE_CHECK_WIDTH
				+ SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH
				+ LegendIcon.LEGEND_ICON_MARGIN_LEFT,
				LegendIcon.LEGEND_ICON_MARGIN_UP, LegendIcon.RASTER_ICON_W + 1,
				LegendIcon.RASTER_ICON_H + 1);

		nameLabel.setBounds(LEFT_MARGIN + VISIBLE_CHECK_WIDTH
				+ SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH + LEGEND_WIDTH
				+ EDITABLE_CHECK_WIDTH, 0, maxw
				- (LEFT_MARGIN + VISIBLE_CHECK_WIDTH + SCALE_CHECK_WIDTH
						+ INFOABLE_CHECK_WIDTH + LEGEND_WIDTH
						+ EDITABLE_CHECK_WIDTH + RIGHT_MARGIN), ROW_STEP);

		/*edition_check.setBounds(LEFT_MARGIN + VISIBLE_CHECK_WIDTH
				+ SCALE_CHECK_WIDTH + INFOABLE_CHECK_WIDTH + LEGEND_WIDTH, 0,
				EDITABLE_CHECK_WIDTH, ROW_STEP);*/

		nameLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		nameLabel.setText(layer.getName());
		nameLabel.addMouseListener(this);

		if ((layer.getMinScale() == -1) && (layer.getMaxScale() == -1)) {
			scaleVisiState.setState(ScaleVisibilityIcon.STATE_NOT_DEFINED);
		} else {
			if (layer.isWithinScale(parent.getScale())) {
				scaleVisiState.setState(ScaleVisibilityIcon.STATE_VISIBLE);
			} else {
				scaleVisiState.setState(ScaleVisibilityIcon.STATE_NOT_VISIBLE);
			}
		}

		setLayout(null);
		add(visible_check);
		add(scaleVisiState);
		//add(infoable_check);
		//add(edition_check);
		add(legendIcon);
		add(nameLabel);

		/*if (layer instanceof FLyrVect) {
			edition_check.setState(lyr.isEditing());
			edition_check.addItemListener(this);
		} else {
			edition_check.setEnabled(false);
		}*/

		visible_check.setState(lyr.isVisible());
		visible_check.addItemListener(this);
		//infoable_check.setState(lyr.isInfoable());
		//infoable_check.addItemListener(this);
	}

	/**
	 * Updates both position and wisth of this row
	 * @param i new index. Determines vertical position
	 * @param w width in pixels
	 */
	public void setPosition(int i, int w) {
		index = i;
		width = w;
		setBounds(0, i * ROW_STEP, w, ROW_STEP);
	}

	/**
	 * Vertical distance between the top of one row and the top of the neighbouring row.
	 */
	public static int ROW_STEP = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static int LEFT_MARGIN = 3;
	public static int RIGHT_MARGIN = 2;

	public static int VISIBLE_CHECK_WIDTH = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static int INFOABLE_CHECK_WIDTH = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static int SCALE_CHECK_WIDTH = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static int LEGEND_WIDTH = 26 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static int EDITABLE_CHECK_WIDTH = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	// public static int NAME_LABEL_WIDTH = 100;

	public static int LEGEND_TYPE_POINT = 0;
	public static int LEGEND_TYPE_LINE = 1;
	public static int LEGEND_TYPE_POLYGON = 2;
	public static int LEGEND_TYPE_RASTER = 10;

	public void itemStateChanged(ItemEvent arg0) {

		Object src = arg0.getSource();

		if (src == visible_check) {
			layer.setVisible(visible_check.getState());
		}

		/*if (src == infoable_check) {
			layer.setInfoable(infoable_check.getState());
		}*/

		/*if (src == edition_check) {
			if (edition_check.getState()) {
				if (!parent.isSomeoneEditing(index)){
					edition_check.setEnabled(true);
					parent.dissableEditionCheckBoxes(index);
					setEditing(true);
				}
			} else {
				parent.enableEditionCheckBoxes();
				setEditing(false);
			}

		}*/

	}

	/**
	 * Gets the checkbox associated with the infoability of the layer. If active,
	 * the layer will be queried when the info tool is used.
	 *
	 * @return the checkbox associated with the infoability
	 */
	/*public Checkbox getInfoableCheck() {
		return infoable_check;
	}*/

	/**
	 * @return the label that keeps the name of the layer.
	 */
	public Label getNameLabel() {
		return nameLabel;
	}

	/**
	 * gets the icon that represents the scale visibility.
	 *
	 * @return the icon that represents the scale visibility
	 */
	public ScaleVisibilityIcon getScaleVisiLabel() {
		return scaleVisiState;
	}

	/**
	 * Gets the chechkbox associated with the visibility of the layer.
	 */
	public Checkbox getVisibleCheck() {
		return visible_check;
	}

	/**
	 * Whether this row is selected.
	 *
	 * @return Whether this row is selected
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * Sets selected/unselected.
	 * @param s whether it has to be selected or not.
	 */
	public void setSelected(boolean s) {

		if (s) {
			if (parent != null) parent.unselectAll();
			selected = true;
			nameLabel.setBackground(SELECTED_BG_COLOR);
		} else {
			selected = false;
			nameLabel.setBackground(NOT_SELECTED_BG_COLOR);
		}
	}

	private final static Color SELECTED_BG_COLOR = Color.LIGHT_GRAY;
	private final static Color NOT_SELECTED_BG_COLOR = Color.WHITE;
//	private final static Color IS_EDITING = Color.ORANGE;

	public void mousePressed(MouseEvent arg0) {

		Object src = arg0.getSource();
		if (src == nameLabel) {
			setSelected(!selected);
		}
	}

	/*private void setEditing(boolean edition) {
		try {
			isEditing = edition;
			if (edition) {
				layer.setEditing(true);
//				nameLabel.setBackground(IS_EDITING);
			} else {
//				layer.setEditing(false);
				StopEditing stop = new StopEditing(((LayersTable)this.getParent()).getMapControl());
				stop.stop();
				nameLabel.setBackground(SELECTED_BG_COLOR);
			}
		} catch (Exception e) {
			logger.error("Poniendo/Terminando en edicin la capa.");
			e.printStackTrace();
		}

	}*/

	public void mouseClicked(MouseEvent arg0) { }
	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) { }

	/*public boolean isEditing() {
		return isEditing;
	}*/

	public int getIndex() {
		return index;
	}

	/*public void setEnableDissabledEditionCheck(boolean check){
		edition_check.setEnabled(check);
	}*/

}
