/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.edition;

import java.awt.Cursor;
import java.awt.Frame;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.io.File;

import org.apache.log4j.Logger;
import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGStorage;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerFileVectorial;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;

public class DistantEditionPointToolListener implements PointToolListener {

	private static Logger logger = Logger.getLogger(DistantEditionPointToolListener.class);

	private MapControl mapCtrl;
	private Frame parentFrame;
	private String fname = "";
	
	// ----------------------------------------
	private static Double ADD_OBJECT_CACHED_X = new Double(0);
	private static Double ADD_OBJECT_CACHED_Y = new Double(0);
	
	public static double getCachedX() { return ADD_OBJECT_CACHED_X.doubleValue(); }
	public static double getCachedY() { return ADD_OBJECT_CACHED_Y.doubleValue(); }
	// ----------------------------------------
	
	public DistantEditionPointToolListener(Frame infoFrame, String fileName) {
		parentFrame = infoFrame;
		fname = fileName;
	}
	
	public void point(PointEvent event) {
		
		if (mapCtrl.isProcessing()) {
			return;
		}
		
		if (event.getEvent().getID() != MouseEvent.MOUSE_PRESSED) {
			return;
		}
		
		int cnt = mapCtrl.getLayerCount();
		FLayerFileVectorial lyr = null;
		for (int i=0; i<cnt; i++) {
			if (mapCtrl.getLayer(i) instanceof FLayerFileVectorial) {
				FLayerFileVectorial lyr_item = (FLayerFileVectorial) mapCtrl.getLayer(i);
				
				//if (lyr_item.getDriver() instanceof PointCsvDriver) {
					File f = new File(lyr_item.getFilePath());
					String item = f.getName();
					if (item.compareToIgnoreCase(fname) == 0) {
						logger.debug("Layer found: " + lyr_item.toString());
						lyr = lyr_item;
					}
				//}
			}
		}
		
		if (lyr == null) {
			logger.debug("No good lyr for file: " + fname);
			MessageDialog._showModalErrorMessageDialog(
					"Add point to pseudo SHP", "Layer not found with file: " + fname,
					parentFrame
					);
			return;
		}
		
		int x = event.getPoint().x;
		int y = event.getPoint().y;
		logger.debug("Click: " + x + ", " + y);
		Point2D wp = mapCtrl.getViewPort().toMapPoint(x, y);
		logger.debug("World: " + wp.getX() + ", " + wp.getY());
		
		// ---------------------------------------
		// --- SEND REQUEST ----------------------
		// ---------------------------------------
		

		ADD_OBJECT_CACHED_X = new Double(wp.getX());
		ADD_OBJECT_CACHED_Y = new Double(wp.getY());
		
		logger.debug("AddPointToPseudoToolListener : ADD_OBJECT_CACHED_X = " + ADD_OBJECT_CACHED_X.doubleValue());
		logger.debug("AddPointToPseudoToolListener : ADD_OBJECT_CACHED_Y = " + ADD_OBJECT_CACHED_Y.doubleValue());
		

		
		ServiceComGvSIGStorage service = (ServiceComGvSIGStorage)DistantServiceManager.getManager().getDistantServiceByURI(
				ServiceComGvSIGStorage.DISTANT_URI);
		
		if(service != null){
			service.setGeoFileName(fname);
			try {
				DistantServiceManager.getManager().runDistantService(service.getDistantServiceUri());
			} catch (CommException e) {
				e.printStackTrace();
			}
		}

		// ---------------------------------------
		// --- SEND REQUEST ----------------------
		// ---------------------------------------
		/*
		String[] row = new String[5];
		row[0] = "WILL BE REPLACED..."; 
		row[1] = "LIERN";
		row[2] = "VALENCIA";
		row[3] = "SPAIN";
		row[4] = "" + (System.currentTimeMillis() % 1000);
		Controller.addPointToLayer(fname,
				wp.getX(),
				wp.getY(),
				row, 0);
				*/
	}

	public Cursor getCursor() {
		return Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	}

	public boolean cancelDrawing() {
		return false;
	}

	public void setMapControl(MapControl mc) {
		mapCtrl = mc;
	}

}
