/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.edition;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.gvsig.mobile.location.GPSManager;
import org.openmis.tellus.exception.SIGException;
import org.openmis.tellus.gvsig.manager.GvSIGTellusGUIManager;
import org.openmis.tellus.gvsig.manager.GvSIGTellusGeoManager;
import org.openmis.tellus.gvsig.manager.GvSIGTellusProjectManager;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.project.DistantProjectManager;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.gvsig.mobile.common.GvsigMobileException;
import es.prodevelop.gvsig.mobile.edition.StopEditing;
import es.prodevelop.gvsig.mobile.fmap.core.DefaultFeature;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.IFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.IFeatureIterator;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;
import es.prodevelop.gvsig.mobile.fmap.edition.EditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.edition.EditionEvent;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.layer.VectorialDefaultAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.VectorialFileAdapter;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.Graph2DUtilities;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.gpe.fmap.VectorialGpeAdapter;
import es.prodevelop.gvsig.mobile.gpe.fmap.VectorialGpeDriver;
import es.prodevelop.gvsig.mobile.gpe.layer.FlyrGpeVect;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class GvSIGTellusEditionPointManager {
	
	private static GvSIGTellusEditionPointManager manager;
	
	public GvSIGTellusEditionPointManager(){
		
	}
	
	public static GvSIGTellusEditionPointManager getManager(){
		if(manager == null){
			manager = new GvSIGTellusEditionPointManager(); 
		}
		return manager;
	}
	
	public VectorialEditableAdapter getVectorialEditableAdapter(FLyrVect lyr) throws GvsigMobileException{
		if (lyr != null) {
			//--------------------------------------------------MG
			try {
				if(!lyr.isEditing())
					lyr.setEditing(true);
			} catch (Exception e) {
				throw new GvsigMobileException("layer not editable", e);
			}
			//--------------------------------------------------
			
			if(!lyr.isAvailable())
				throw new GvsigMobileException("couche no disponible");
			VectorialEditableAdapter vea = (VectorialEditableAdapter) lyr.getSource();
			
			return vea;
		}
		
		return null;
	}
	
	public Point2D addGPSPointToLayer(
			String gf_name,
			String id_val,
			String id_fld_name,
			String type,
			String type_fld_name) throws SIGException {
		
		if (!GPSManager.getGPSManager().isConnected()) {
			throw new SIGException("GPS is not active");
		}
		
		return addPointToLayer(
				gf_name,
				GPSManager.getGPSManager().lastGPSFix.getLongitude(),
				GPSManager.getGPSManager().lastGPSFix.getLatitude(),
				id_val,
				id_fld_name,
				type,
				type_fld_name,
				false);
	}
	
	
	public Point2D addPointToLayer(
			String geofname,
			double x,
			double y,
			String id,
			String id_fld_name,
			String type,
			String type_fld_name,
			boolean wantGlobalAdd) throws SIGException {
		
		try{
			DistantEditableLayer layer = null;
			Point2D g = null;
			try{
				layer = (DistantEditableLayer)DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(geofname);
			}catch(Exception ex){
				throw new SIGException("the fileName don't define an DistantEditableLayer : " + geofname);
			}
			
			FLayer l =  GvSIGTellusGUIManager.getManager().getMapControl().getLayerWithGeoFileName(layer.getGeoFileName());	
			if (l == null) {
				throw new SIGException("No vector layer has this geo file: " + geofname);
			}
			
			if (l instanceof FLyrVect){
				
				FLyrVect lyr = (FLyrVect)l;
				
				VectorialEditableAdapter vea = getVectorialEditableAdapter(lyr);
				
				int numAttr = vea.getRecordset().getFieldCount();
				Value[] values = new Value[numAttr];
				
				//--------------------------------------------------MG
				int idFieldIndex = vea.getRecordset().getFieldIndexByName(id_fld_name);
				int typeFieldIndex = vea.getRecordset().getFieldIndexByName(type_fld_name);
				//--------------------------------------------------
				
				for (int i = 0; i < numAttr; i++) {
					values[i] = ValueFactory.createNullValue();
					//--------------------------MG
					if(i == idFieldIndex)
						values[i] = ValueFactory.createValue(id);
					if(i == typeFieldIndex)
						values[i] = ValueFactory.createValue(type);
					//--------------------------
				}
				
				
				
				g = addPoint(
						  new Point2D.Double(x, y)
						, lyr
						, GvSIGTellusGUIManager.getManager().getMapControl()
						, values
						, false
						, wantGlobalAdd);
			}else{
				throw new SIGException("Not a FLyrVect layer impossible to edit : " + geofname);
			}
	
			return g;
			
		}catch(Exception e){
			throw new SIGException(e);
		}
	}
	
	
	public  void removeAllPointsFromLayer(
			String geofname,
			String id_fld_name) throws Exception {
		
			FLyrVect lyr = GvSIGTellusGeoManager.getManager().getFLyrVect(geofname);		
			SelectableDataSource sds = null;
			long row_cnt = 0;

			// =========================================
			try {
				sds = lyr.getSource().getRecordset();
				row_cnt = sds.getRowCount();
			} catch (Exception ex) {
				throw new Exception("While getting fields info: " + ex.getMessage());
			}
			// =========================================
			lyr.setVisible(false);
			for (long i=0; i<row_cnt; i++) {
				
				removePoint(
						  lyr
						, i 
						, GvSIGTellusGUIManager.getManager().getMapControl(), false);
				
				//Thread.sleep(50);
			}
			lyr.setVisible(true);
			
			GvSIGTellusGUIManager.getManager().getMapControl().setProcessing(false);
			GvSIGTellusGUIManager.getManager().getMapControl().refresh(true);
	}
	
	
	public void removePoint(FLyrVect lyr, String id_fld_name, String id_val, MapControl mapControl) throws GvsigMobileException {
		SelectableDataSource sds = null;
		int id_ind = 0;
		long row_cnt = 0;
		

		// =========================================
		try {			
			sds = lyr.getRecordset();
			id_ind = sds.getFieldIndexByName(id_fld_name);
			row_cnt = sds.getRowCount();
		} catch (Exception ex) {
			throw new GvsigMobileException("While getting fields info: " + ex.getMessage());
		}
		// =========================================

		
		Value id_item = null;
		boolean found = false;
		long found_index = 0;
		for (long i=0; i<row_cnt; i++) {
			
			try {
				id_item = sds.getFieldValue(i, id_ind);
			} catch (DriverException e) {
				throw new GvsigMobileException("Impossible to getFieldValue " + id_ind);
			}
			
			//System.out.println(id_item.toString());
			if (id_item.toString().trim().compareTo(id_val) == 0) {
				found = true;
				found_index = i;
				break;
			}
		}
		
		if (! found) {		
			throw new GvsigMobileException("ID not found in layer: " + id_val);
		}
		
		this.removePoint(lyr, found_index, mapControl, true);			

	}

	
	public void removePoint(FLyrVect lyr, long index, MapControl mapControl, boolean refresh)throws GvsigMobileException{
		try{
			
			if (lyr != null) {
				VectorialEditableAdapter vea = getVectorialEditableAdapter(lyr);
				EditableAdapter adp = (EditableAdapter)vea;
				adp.doRemoveRow((int)index, EditionEvent.GRAPHIC);
				//int realindex = vea.getCalculatedIndex((int)index);
				//System.out.println("index : "+index+"    real index : "+realindex);
				//vea.removeRow((int)index, "point", EditionEvent.GRAPHIC);
				
				//System.out.println(vea.getRecordset().getRowCount());
				//System.out.println(vea.getRowCount());
				//lyr.setRecordset(vea.getRecordset());
			}
		
			if(refresh){
				mapControl.setProcessing(false);
				mapControl.refresh(true);
			}
			
			
		}catch (Exception e) {
			throw new GvsigMobileException("Error in EditionPointManager", e);
		}
	}

	public Point2D addPoint(Point2D pointMap, FLyrVect lyr, MapControl mapControl, Value[] values, boolean isConfirmation, boolean wantGlobalAdd) throws GvsigMobileException{	
		try{
			
			if (lyr != null) {
				VectorialEditableAdapter vea = getVectorialEditableAdapter(lyr);

				// Deberiamos comprobar que lo que escribimos es correcto:
				// Lo hacemos en el VectorialAdapter, justo antes de
				// anadir, borrar o modificar una feature
				Graphics2D gs=null;
				ViewPort vp =null;
				IGeometry geom = null;
				try {

					String newFID;

					newFID = vea.getNewFID();
					
					geom = ShapeFactory.createPoint2D(pointMap.getX(), pointMap.getY());
					
					DefaultFeature df = new DefaultFeature(geom.cloneGeometry(), values, newFID);
					int index = vea.addRow(df, "point", EditionEvent.GRAPHIC);

					vp = mapControl.getViewPort();
					BufferedImage selectionImage = Utils.createBufferedImage(vp.getImageWidth(), vp.getImageHeight());
					gs = selectionImage.createGraphics();

					Rectangle2D bb = geom.getBounds2D();

					if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {

						AffineTransform _at = vp.getAffineTransform();
						geom.transform(_at);
						Shape shp = geom.getInternalShape();

						Graphics2D mapGraph = (Graphics2D)mapControl.getCurrentImage().getGraphics();
						if(!wantGlobalAdd == false)
							mapControl.setProcessing(true);
						FSymbol sym = lyr.getLegend().getDefaultSymbol();
						mapGraph.setColor(sym.getBorderColor());
						mapGraph.setStroke(new BasicStroke(sym.getStrokeWidth()));
						Graph2DUtilities.drawPoint(mapGraph, (FPoint2D) shp);
						
						if(wantGlobalAdd == false){
							mapControl.setProcessing(false);
							mapControl.refresh(true);
						}

					}


				} catch (IOException e) {
					throw new GvsigMobileException("While adding a row: " + e.getMessage());
				}

				if(wantGlobalAdd == false){
					SaveAllEdition(isConfirmation, mapControl);
				}
				
				return pointMap;
			}
			
			return null;
			
		}catch (Exception e) {
			throw new GvsigMobileException("Error in EditionPointManager", e);
		}
	}
	
	public void SaveAllEdition(boolean isConfirmation, MapControl mapControl){
		//------------------MG
		StopEditing stop = new StopEditing(mapControl);
		stop.setConfirmation(isConfirmation);
		stop.stop();
		
		//---------------------
	}
	

}
