/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.fmap;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;

import org.apache.log4j.Logger;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.symbo.Dynamisme;
import org.openmis.tellus.layers.symbo.IconDynamique;
import org.openmis.tellus.layers.symbo.IconSymbology;
import org.openmis.tellus.project.DistantProjectManager;

import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.DiskShpDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.MemoryShpDriver;
import es.prodevelop.gvsig.mobile.fmap.layer.AlphanumericData;
import es.prodevelop.gvsig.mobile.fmap.layer.ClassifiableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerFileVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrDefault;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.InfoByPoint;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.layer.SingleLayer;
import es.prodevelop.gvsig.mobile.fmap.legend.Legend;
import es.prodevelop.gvsig.mobile.fmap.legend.VectorialLegend;
import es.prodevelop.gvsig.mobile.fmap.proj.ICoordTrans;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.cancel.Cancellable;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.Graph2DUtilities;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

public class FLayerFileVectSymb extends FLayerFileVectorial  {
	
	private static Logger logger = Logger.getLogger(FLayerFileVectSymb.class.getName());
	
	public FLayerFileVectSymb(){
		super();
	}
	
	public static FLayerFileVectorial createLayer(File f, String lyr_name, MapContext mc){
		MemoryShpDriver drv;
		logger.debug(Utils.time() + "MainWindow instancia el driver MEMORY SHP.");
		drv = new MemoryShpDriver();
		
		try {
			logger.debug(Utils.time() + "MainWindow llama open(f) de driver MEMORY SHP.");
			drv.open(f);
			logger.debug(Utils.time() + "MainWindow llama initialize() de driver MEMORY SHP.");
			drv.initialize();
		} catch (IOException e) {
			logger.error("Error while opening file: " + e.getMessage());
			
			logger.debug(Utils.time() + "No hay memoria suficuente: probamos DISK SHP.");
			drv = null; System.gc();
			drv = new DiskShpDriver();

			try {
				logger.debug(Utils.time() + "MainWindow llama open(f) de driver DISK SHP.");
				drv.open(f);
				logger.debug(Utils.time() + "MainWindow llama initialize() de driver DISK SHP.");
				drv.initialize();
			} catch (IOException e1) {
				logger.error("Error while opening file: " + e1.getMessage());
				drv = null; System.gc();
				return null;
			}
		}

		logger.debug(Utils.time() + "MainWindow instancia FLayerFileVectorial.");
		FLayerFileVectSymb shplyr = new FLayerFileVectSymb();
		shplyr.setMapContext(mc);
		
		shplyr.setVisible(true);
		shplyr.setAvailable(true);
		
		/*if (in_mem) {
			shplyr.setName(lyr_name + " [m]");
		} else {
			shplyr.setName(lyr_name + " [d]");
		}*/
		shplyr.setName(lyr_name);
		

		shplyr.setDriver(drv);
		shplyr.setProjection(mc.getProjection());
		
		logger.debug(Utils.time() + "Vamos a llamar a wakeup.");
		shplyr.wakeUp(true);

		// System.out.println("Layer shp instanciada (no probada)");
		logger.debug(Utils.time() + "MainWindow llama addLayer de mapcontext.");

		return shplyr;
	}
	

	protected void drawPoints(Graphics2D g, ViewPort vp) throws FMapDriverException{
		AffineTransform _at = vp.getAffineTransform();
		int count = getSource().getShapeCount();

		for (int i=0; i<count; i++) {
			try{
				if (getSelectionSupport().isSelected(i)) {
					g.setColor(FSymbol.SELECTION_SYMBOL.getBorderColor());
				}
				
	
				Rectangle2D bb = getSource().getShapeBounds(i);
				
				if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
				IGeometry ig = getSource().getShape(i);
				
					ig.transform(_at);
					Shape shp =  ig.getInternalShape();
					//Graph2DUtilities.drawShape(g, shp, sym);
					//Graph2DUtilities.drawPoint(g, (FPoint2D) shp);
					Graph2DUtilities.drawPoint(g, (FPoint2D) shp);
					//Rectangle2D r2 = (Rectangle2D)ig.getBounds();
					if (ig==null)
						logger.error("No hay bbox");
					//RotationUtils.drawRotatedText(g,"Hola",Color.red,(int)r2.getMinX(),(int)r2.getMinY(),0.5);
					/*RotationUtils.drawRotatedText(g,
							"Hola",
							Color.red,
							(int)((FPoint2D) shp).getX(),
							(int)((FPoint2D) shp).getY(),
							angle);
					angle-=0.1;*/

					
					
					
					//------------- ADDED FOR OPENMIS SYMBOL IMAGE VIEW ------------------------------
					/*DistantLayer layer = null;
					if(getName().endsWith(DistantEditableLayer.EXT_CSV)){
						String name = getName().substring(0, getName().indexOf(DistantEditableLayer.EXT_CSV));
						name = name+DistantEditableLayer.EXT_SHP;
						layer = DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(name);
					}else{
						layer = DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(getName());
					}*/
					DistantLayer layer = DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(getName());
					if(layer != null){
						if(layer.getSymbology() != null && layer.getSymbology().getIconSymbology() != null){
							Image img = null;
							IconSymbology symbology = layer.getSymbology().getIconSymbology();
							if(symbology.getDefaultIconName() != null){
								img = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, symbology.getDefaultIconName());
							}
							
							if(symbology.getDynamisme() != null){
								
								Dynamisme dyna = symbology.getDynamisme();
								SelectableDataSource sds = getSource().getRecordset();
								int idFieldIndex = sds.getFieldIndexByName(dyna.getFieldReference());
								
								Value valueFile = sds.getRow(i)[idFieldIndex];

								Enumeration enume = dyna.getTableIconDynamic().keys();
								while(enume.hasMoreElements()){
									String value = (String)enume.nextElement();
									if(value.equals(valueFile.toString().trim())){
										IconDynamique iconDynamique = (IconDynamique)dyna.getTableIconDynamic().get(value);
										img = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, iconDynamique.getIconName());
									}
								}
							}
	
		
							if(img != null){
								int w = img.getWidth(null);
								int h = img.getHeight(null);
								
								g.drawImage(img,(int)((FPoint2D) shp).getX() - (w/2), (int)((FPoint2D) shp).getY() - (h/2), null);
							}
						}
					}else{
						throw new Exception("IMPORTANT ERROR : The DistantLayer was not FOUND !!");
					}
					//---------------------------------------------------------------------------------
					
					
				}
				if (getSelectionSupport().isSelected(i)) {
					g.setColor(getLegend().getDefaultSymbol().getBorderColor());
				}
			} catch (Exception e){
				logger.error("drawPoints: ", e);
			}
		}
	}

}
