/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.gui;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.List;

import org.gvsig.mobile.location.tracklog.TracklogManager;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.symbo.DistantLayerSymbology;
import org.openmis.tellus.layers.symbo.Dynamisme;
import org.openmis.tellus.layers.symbo.IconDynamique;
import org.openmis.tellus.layers.symbo.IconSymbology;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.layer.FAcetateLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.gui.bar.StatusBar;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.ExportDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.LayersManagerDialog;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.gui.tool.TBActionButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBSelectableButton;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBar;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBarGroupButton;
import es.prodevelop.gvsig.mobile.gui.tool.behavior.PointBehavior;

public abstract class AbstOpenMISMainWindow extends MainWindow {
	private static final long serialVersionUID = -3690316956585873472L;
	
	private boolean onceInitDistantTools = false;
	
	private boolean isGPSStatusBar = false;
	private FAcetateLayer faceitate = null;
	
	public AbstOpenMISMainWindow(boolean isGPSStatusBar, FAcetateLayer faceitate){
		super();
		this.isGPSStatusBar = isGPSStatusBar;
		this.faceitate = faceitate;
	}
	
	public AbstOpenMISMainWindow(int _width, int _height, boolean isGPSStatusBar, FAcetateLayer faceitate) {
		super(_width, _height);
		this.isGPSStatusBar = isGPSStatusBar;
		this.faceitate = faceitate;
	}
	
	public void initialize() {	
		initialize(getWidth(), getHeight());
		super.initialize();
	}
	
	private void initialize(int width, int height) {
		
		//Set screen size, based on device size or arguments given by user
		setSize(width, height); //Uses MainWindow properties
		setResizable(true);
		addComponentListener(this);
		
		int _w = getWidth();
		this.setMapControl(new OWMapControl(this, _w));
		
		this.initToolsBars();
		//--- INIT ALL TOOLS DECLARED
		this.initToolsCanvas();
		
		/*
		 * To have the FAcetateLayerOpenMIS when the tracking is activate.
		 * Offer 2 possible ways for the trailer : green for trakcing activate, red for no-tracking.
		 */
		if(faceitate != null){
			getMapControl().getMapContext().setFAcetateLayer(faceitate);
		}else{
			getMapControl().getMapContext().setFAcetateLayer(new FAcetateLayer(false));
		}
	}
	
	
	public void mousePressed(MouseEvent arg0) {
		
		Object src = arg0.getSource();
		
		this.onMousePressed(arg0, src);
		super.mousePressed(arg0);
	}
	
	
	public void initToolsCanvas() {
		super.initToolsCanvas();
		
		//-------- ADDED TO VIEW THE GPS COORDINATE BAR ALL THE TIME
		if(isGPSStatusBar)
			statusBar.addBar(StatusBar.GPS_BAR);
	}

	
	public void initializeDistantTools() throws Exception {	
		
		if(onceInitDistantTools == false){			
			DistantProject project  = DistantProjectManager.getManager().getCurrentDistantProject();
			
			if(project != null){
				List layers = project.getEditableLayers();
				for(int i=0;i<layers.size();i++){
					DistantEditableLayer layer = (DistantEditableLayer)layers.get(i);
					
					DistantLayerSymbology symbol = layer.getSymbology();
					if(symbol != null){
						IconSymbology iconSymbol = symbol.getIconSymbology();
						
						if(iconSymbol != null && iconSymbol.getDynamisme() != null){
							Dynamisme dyna = iconSymbol.getDynamisme();
							FLyrVect lyr = null;
							//---------------- GET THE RIGHT LAYER WITH NAME -------------
							for (int j=0; j<getMapControl().getLayerCount(); j++) {
								if (getMapControl().getLayer(j) instanceof FLyrVect) {
									FLyrVect lyr_item = (FLyrVect) getMapControl().getLayer(j);

									if (lyr_item.getName().compareToIgnoreCase(layer.getGeoFileName()) == 0) {
										lyr = lyr_item;
									}
								}
							}
							//------------------------------------------------------------
							if(lyr != null){
								this.initDynamismeDistantTool(dyna, lyr);			
							}
						}
					}					
				}
			}
			onceInitDistantTools = true;
		}
	}
	
	public abstract void initToolsBars();
	public abstract void onMousePressed(MouseEvent evt, Object src);
	public abstract void initDynamismeDistantTool(Dynamisme dyna, FLyrVect lyr);
	
}
