/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.gui;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;

public class DistantQueryAttributesDialog extends ExpansibleDialog implements MouseListener {
	
private static Logger logger = Logger.getLogger(DistantQueryAttributesDialog.class);
	
	public static final int ROW_HEIGHT = 20;
	public static final int LABELS_WIDTH = 80;
	private Frame parent = null;
	
	private TocButton okButton;

	private int fld_values_col_w = 20;
	private int fld_names_col_w = 20;
	private String[] ff;
	private Label[] vv;
	
	public DistantQueryAttributesDialog(Frame p, String[] _ff, Label[] _vv) {
		
		super(p);
		parent = p;
		
		ff = _ff;
		vv = _vv;
		
		setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		setModal(true);
		init();
		
		setVisible(true);
	}
	
	public DistantQueryAttributesDialog(Frame p, Map map) {
		super(p);
		parent = p;
		
		String _ff[] = new String[map.keySet().size()];
		Label _vv[] = new Label[map.keySet().size()];
		
		Iterator it = map.keySet().iterator();
		int i = 0;
		while(it.hasNext()){
			String title = (String)it.next();
			_ff[i] = title;
			_vv[i] = (Label)map.get(title);
			i++;
		}
		
		ff = _ff;
		vv = _vv;
		
		setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		setModal(true);
		init();
		
		setVisible(true);
		
	}

	public void init() {
		
		logger.debug("CommQueryAttributesDialog - init - 1");

		
		Label title = new Label("OpenMIS Info");
		title.setFont(DialogConstants.DIALOG_BOLD_FONT);
		title.setSize(LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		
		okButton =
			new TocButton(DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, "toc_ok_dis_small.png", 1, true, false);
		okButton.addMouseListener(this);

		ScrollPane scr = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		scr.add(createPseudoTable(ff, vv));
		scr.setSize(LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		
		logger.debug("CommQueryAttributesDialog - init - 2");
		
		
		// --------------
		ExpansibleRow row00 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row00.addExtensible(title); add(row00);
		// --------------------
		 ExpansibleRow row0 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		 row0.addExtensible(scr);
		 addExpansible(row0);
		// --------------------
		ExpansibleRow row01 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row01.add(okButton);
		add(row01);
		
		// --------------
		scr.repaint();
	}
	
	public void mouseClicked(MouseEvent e) {
		
		Object src = e.getSource();
		if (src == okButton) {
			setVisible(false);
		}
		
		if (src instanceof Label) {
			
			logger.debug("mouseClicked - ON LABEL");
			
			
			Label lbl = (Label) src;
			try {
				logger.debug("mouseClicked - ON LABEL: URL = " + lbl.getText());
				URL url = new URL(lbl.getText());
				url.openStream();
			} catch (Exception ex) {
				logger.error("While accessing URL: " + ex.getMessage());
			}
			
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	
	private int getMaxStringWidth(Label[] ss, FontMetrics fm) {
		
		int resp = 0;
		
		for (int i=0; i<ss.length; i++) {
			if (ss[i] != null) {
				int aux = fm.stringWidth(ss[i].getText());
				if (aux > resp) resp = aux; 
			}
		}
		
		return resp;
	}
	
	private int getMaxStringWidth(String[] ss, FontMetrics fm) {
		
		int resp = 0;
		
		for (int i=0; i<ss.length; i++) {
			if (ss[i] != null) {
				int aux = fm.stringWidth(ss[i]);
				if (aux > resp) resp = aux; 
			}
		}
		
		return resp;
	}
	
	
	
	private Panel createPseudoTable(String[] field_names, Label[] field_values) {
		
		Panel resp = new Panel();
		resp.setLayout(null);

		logger.debug("createPseudoTable - 0");
		
		Graphics _g = parent.getGraphics();
		
		logger.debug("createPseudoTable - 1");
		
		FontMetrics _fm = _g.getFontMetrics();
		
		logger.debug("createPseudoTable - 2");
		
		fld_names_col_w = 20 + getMaxStringWidth(field_names, _fm);
		
		logger.debug("createPseudoTable - 3");

		fld_values_col_w = 35 + getMaxStringWidth(field_values,
							parent.getGraphics().getFontMetrics());
		
		logger.debug("createPseudoTable - 4");

		int cnt = field_names.length;

		resp.setBounds(0, 0, fld_names_col_w + fld_values_col_w, cnt * ROW_HEIGHT);

		for (int i=0; i<cnt; i++) {
			
			logger.debug("createPseudoTable - 5, i = " + i);
			
			Label v = null;
			Label f = null;
			

			f = getNewLabel(field_names[i], i);
			if (field_values[i] != null) {
				v = field_values[i];
				v.setBounds(fld_names_col_w, i * ROW_HEIGHT, fld_values_col_w, ROW_HEIGHT);
			}

			if (v == null) {
				
				if (f == null) {
					// separator
					
				} else {
					// title
					// f.setBounds(0, i * ROW_HEIGHT, fld_names_col_w + fld_values_col_w, ROW_HEIGHT);
					// f.setAlignment(Label.CENTER);
					f.setFont(DialogConstants.DIALOG_BOLD_FONT);
					resp.add(f);
				}
			} else {
				// pair
				resp.add(f);
				resp.add(v);
			}
		}
		
		return resp;
	}
	

	private boolean isURLTag(String str) {
		
		logger.debug("isURLTag = " + str);
		if (str == null) return false;
		return (str.compareToIgnoreCase("link") == 0);
	}
	
	private Label getNewCommLabel(String s, int i, boolean isval) {
		if (s == null) return null;

		if (isval) {
			logger.debug("xxxxxxxxxxxxxxxxx s = " + s);
			Label respv = new Label(s);
			respv.setBounds(fld_names_col_w, i * ROW_HEIGHT, fld_values_col_w, ROW_HEIGHT);
			respv.setForeground(Color.RED);
			respv.addMouseListener(this);
			return respv;
		} else {
			Label respf = new Label("LINK");
			respf.setBounds(0, i * ROW_HEIGHT, fld_names_col_w, ROW_HEIGHT);
			return respf;
		}
	}
	

	private Label getNewLabel(String s, int index) {
		if (s == null) return null;
		Label resp = new Label(" " + s);
		resp.setBounds(0, index * ROW_HEIGHT, fld_names_col_w, ROW_HEIGHT);
		return resp;
	}
}
