/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.gui;

import java.io.File;

import org.dinopolis.gpstool.gpsinput.GPSException;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.location.GPSManager;
import org.gvsig.mobile.location.tracklog.RawDataManager;
import org.gvsig.mobile.location.tracklog.TracklogManager;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.synchro.common.data.EventFileModification;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.gpe.layer.FlyrGpeVect;
import es.prodevelop.gvsig.mobile.gpe.writer.ExportGpeWriter;
import es.prodevelop.gvsig.mobile.gui.dialog.AcceptCancelDialog;
import es.prodevelop.gvsig.mobile.gui.tool.TBActionButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBSelectableButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBSwitcherButton;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBar;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBarGroupButton;
import es.prodevelop.gvsig.mobile.gui.tool.behavior.PointBehavior;
import es.prodevelop.gvsig.mobile.gui.tool.behavior.RectangleBehavior;
import es.prodevelop.gvsig.mobile.gui.tool.listener.CenterOnPointListener;
import es.prodevelop.gvsig.mobile.gui.tool.listener.PanListener;
import es.prodevelop.gvsig.mobile.gui.tool.listener.ZoomInRectListener;

public class NavigationInitToolBar implements InitToolBar{

	private static NavigationInitToolBar instance = null;
	
	
	private AbstOpenMISMainWindow window;
	
	public TBActionButton standardZoomInToolButton, standardZoomOutToolButton;
	public TBSelectableButton panButton;
	public TBSelectableButton rectZoomInButton;
	
	public NavigationInitToolBar(AbstOpenMISMainWindow window){
		this.window = window;
	}
	
	public static NavigationInitToolBar getInstance(){
		return instance;
	}
	public static void registerInitToolBar(NavigationInitToolBar newinstance){
		instance = newinstance;
	}
	
	
	public AbstOpenMISMainWindow getMainWindow() {
		return window;
	}
	
	public void initToolsCanvas(int[] positions) {

		ToolBarGroupButton navGroupButton =	new ToolBarGroupButton("group_navigation_floating_tab.png", "group_navigation_selected.png", window, positions[0]);
		ToolBar navBar = new ToolBar(window, navGroupButton, positions[0]);
		// Addition of the ToolBar to the toolbars array of MainWindow
		window.addToolBar(navBar);

		 /*
		 * All listeners and behaviors handlers used by the navigation toolbar
		 * tools selectable buttons are defined
		 */

		ZoomInRectListener zirl = new ZoomInRectListener();
		zirl.setMapControl(window.getMapControl());
		
		RectangleBehavior rb = new RectangleBehavior(zirl); //For zoom rectangle
		PanListener pal = new PanListener(window.getMapControl());
		PointBehavior pb4 = new PointBehavior(pal); //For pan
		CenterOnPointListener copl = new CenterOnPointListener();
		copl.setMapControl(window.getMapControl());

		/*
		 * Creation of the buttons. The constructor assigns the buttons to
		 * the ToolBar
		 */
		
		// Zoom rectangle
		rectZoomInButton = new TBSelectableButton(navBar,	"zoom_window_black.png", "zoom_window_orange.png", GvsigMobileGUIConstants.TB_NAV_ZOOM_RECT, rb, window); 
		
		// Zoom in (fixed with no user action)
		standardZoomInToolButton = new TBActionButton(navBar, "zoom_in_black.png", "zoom_in_orange.png", GvsigMobileGUIConstants.TB_NAV_ZOOM_IN, window); 
		// Zoom out (fixed with no user action)
		standardZoomOutToolButton = new TBActionButton(navBar, "zoom_out_black.png","zoom_out_orange.png", GvsigMobileGUIConstants.TB_NAV_ZOOM_OUT, window); 
		
		panButton = new TBSelectableButton(navBar, "pan_black.png", "pan_orange.png", GvsigMobileGUIConstants.TB_NAV_PAN, pb4, window);
		
		navBar.reorder(window.getWidth());
		window.getMapControl().addGroupButton(navGroupButton, true);
		window.getMapControl().setToolBar(navBar, window.getWidth());
	}
	
	public void doZoomIn(){
		window.getMapControl().setProcessing(true);

		boolean resp = doStandardZoomIn();
		if (!resp) {
			window.getMapControl().setProcessing(false);
			window.getMapControl().refresh(false);
		}
	}
	public void doZoomOut(){
		window.getMapControl().setProcessing(true);

		boolean resp = doStandardZoomOut();
		if (!resp) {
			window.getMapControl().setProcessing(false);
			window.getMapControl().refresh(false);
		}
	}
	
	private boolean doStandardZoomIn() {
		return
		window.getMapControl().getViewPort().scaleExtent(ZoomInRectListener.ZOOM_OUT_FACTOR, true);
	}
	
	private boolean doStandardZoomOut() {
		return
		window.getMapControl().getViewPort().scaleExtent(1.0 / ZoomInRectListener.ZOOM_OUT_FACTOR, true);
	}
	
}
