/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.gui;

import java.util.ArrayList;

import org.gvsig.mobile.location.tracklog.TracklogManager;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.gvsig.fmap.FAcetateLayerOpenMIS;
import org.openmis.tellus.gvsig.services.ServiceComGvSIGEventFileModif;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.synchro.common.data.EventFileModification;

import es.prodevelop.gvsig.mobile.fmap.layer.FAcetateLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerFileVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;

public class OWMapControl extends MapControl {

	private static final long serialVersionUID = 7809292496763773714L;

	public OWMapControl(MainWindow mw, int w) {
		super(mw,w);
	}
	
	
	public FLayer getLayerWithGeoFileName(String geofname) {
		
		int lyrcon = project.getMapContext().getLayerCount();
		FLayer lyr;
		//int geofname_l = geofname.length();

		for (int i=0; i<lyrcon; i++) {
			
			lyr = project.getMapContext().getLayer(i);
			
			if(lyr.getName().equals(geofname)){
				return lyr;
			}
			
			/*if (lyr instanceof FLayerFileVectorial) {
				
				FLayerFileVectorial lyrv = (FLayerFileVectorial) lyr;
				String filepath = lyrv.getFilePath();
				int last = filepath.lastIndexOf(geofname);
				if (filepath.length() == (geofname_l + last)) {
					return lyr;
				}
			}else{
				return lyr.get;
			}*/
		}
		return null;
	}
	
	public void removeAllLayers(){
		project.setModified(true);
		
		FLayer layers[] = getToc().getLayers();
		int count = layers.length;
		while (count > 0){
			try{
				getToc().removeItem(count);
			}catch(Exception ex){
				ex.printStackTrace();
			}
			count--;
		}
		

		ArrayList list = project.getMapContext().getLayers();
		while(list != null && list.size() > 0){
			FLayer layer = (FLayer)list.get(0);
			try{
				project.getMapContext().removeLayer(layer);
			}catch(Exception ex){
				ex.printStackTrace();
			}
			list = project.getMapContext().getLayers();
		}
		

		redraw(true);
	}
	
	public void zoomToSelection(FLyrVect lyr) {
		project.getMapContext().zoomToSelection(lyr);
	}
	

	public void switchTracklog() {
		
		if (TracklogManager.isTracking()) {
			TracklogManager.stopTracking();
			if(project.getMapContext().getAcetateLayer() instanceof FAcetateLayerOpenMIS){
				((FAcetateLayerOpenMIS)project.getMapContext().getAcetateLayer()).setActiveTracking(false);
			}	
			
			
		} else {
			TracklogManager.startTracking();
			if(project.getMapContext().getAcetateLayer() instanceof FAcetateLayerOpenMIS){
				((FAcetateLayerOpenMIS)project.getMapContext().getAcetateLayer()).setActiveTracking(true);
			}
		}
	}
	
	public void processEventFileModification(DistantLayer layer, int eventFileModif) throws Exception{
		this.processEvtFileM(eventFileModif, layer.getGeoFileName());
		this.processEvtFileM(eventFileModif, layer.getDBFFileName());
		this.processEvtFileM(eventFileModif, layer.getSHXFileName());
	}
	
	public void processEventFileModification(String geoFileName, int eventFileModif, boolean isShpDependance)throws Exception{
		if(isShpDependance){
			
			if(geoFileName.indexOf(".shp") == -1){
				this.processEvtFileM(eventFileModif, geoFileName+".shp");
				this.processEvtFileM(eventFileModif, geoFileName+".dbf");
				this.processEvtFileM(eventFileModif, geoFileName+".shx");
			}else{
				this.processEvtFileM(eventFileModif, geoFileName);	
				String name = geoFileName.substring(0, geoFileName.indexOf(".shp"));	
				this.processEvtFileM(eventFileModif, name+".dbf");
				this.processEvtFileM(eventFileModif, name+".shx");
			}
				
		}else{
			this.processEvtFileM(eventFileModif, geoFileName);
		}
	}
	
	private void processEvtFileM(int eventFileModif, String fileName)throws Exception{
		ServiceComGvSIGEventFileModif service = (ServiceComGvSIGEventFileModif)DistantServiceManager.getManager().getDistantServiceByURI(
				ServiceComGvSIGEventFileModif.DISTANT_URI);
		
		service.setActionType(eventFileModif);
		service.setFileName(fileName);
		service.setServerPath(null);
		DistantServiceManager.getManager().runDistantService(service.getDistantServiceUri());
	}


}
