/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ItemEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPasswordField;

import org.openmis.tellus.exception.SIGException;
import org.openmis.tellus.gvsig.manager.GvSIGTellusGUIManager;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ImageControl;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.gui.tool.DialogButton;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class WaiterDialog extends ExpansibleWindow implements MouseListener, java.awt.event.ItemListener{

	private static final long serialVersionUID = -3550588055811245887L;
	
	private Label infoLabel;
	private Label waitLabel;
	private Label titleLabel;
	
	private Label annulerLabel;
	private Panel dummyPanel;
	private ImageControl icon;
	
	private Frame frame = null;
	private TocButton cancelButton;
	
	
	public WaiterDialog(Frame mf) {
		super(mf);
		frame = mf;
		
		situate(mf);
		
		init();
		setVisible(true);
		
	}
	
	private void situate(Frame mf){
		int x = mf.getX() + ((mf.getWidth() - 200) / 2);
		int y = mf.getY() + ((mf.getHeight() - 150) / 2);
		setBounds(x, y, 200, 150);
	}
	
	private void init() {
		
		Image logo_img = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "icon_wait.png");
    	ImageControl icon = new ImageControl(logo_img);
		
		cancelButton = new TocButton(DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		cancelButton.addMouseListener(this);
		
		titleLabel = new Label("Synchronization"); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);
		
		
		infoLabel = new Label("Synchronisation bittorrent is running."); //$NON-NLS-1$
		infoLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);    
		infoLabel.setSize(DialogConstants.LABELS_WIDTH+110, DialogConstants.TABLE_LABEL_HEIGHT);
		
		waitLabel = new Label("Please wait..."); //$NON-NLS-1$
		waitLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);       
		waitLabel.setSize(DialogConstants.LABELS_WIDTH+100, DialogConstants.TABLE_LABEL_HEIGHT);

		annulerLabel = new Label("Cancel"); //$NON-NLS-1$
		annulerLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);       
		annulerLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		
		
		// -------------------
		dummyPanel = new Panel();
		dummyPanel.setSize(65,50);
		// --------------------
		ExpansibleRow row01 = new ExpansibleRow(50);
		row01.add(dummyPanel);row01.add(icon);
		add(row01);
		
		// --------------------
		ExpansibleRow row0 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row0.add(infoLabel);
		add(row0);
		
		dummyPanel = new Panel();
		dummyPanel.setSize(40,DialogConstants.CONTROL_HEIGHT);
		ExpansibleRow row1 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row1.add(dummyPanel);row1.add(waitLabel);
		add(row1);

		// --------------
		
		ExpansibleRow row6 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row6.add(cancelButton);row6.add(annulerLabel);
		add(row6);
		
		addEmptyRowIfPhoneME(this);
	}
	
	
	private static WaiterDialog dialog;
	public static WaiterDialog _showModalWaiterDialog(Frame parentFrame) {
		
		if(dialog == null){
			dialog = new WaiterDialog(parentFrame);
		}
		dialog.situate(parentFrame);
		dialog.setVisible(true);
		return dialog;
	}
	
	
	public void mousePressed(MouseEvent e) {
		
		Object src = e.getSource();
		if (src == cancelButton) {
			hide();
		}
	}

	public void mouseEntered(MouseEvent e) { }
	public void mouseExited(MouseEvent e) { }
	public void mouseClicked(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { }

	public void itemStateChanged(ItemEvent e) {}
	
}
