/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.manager;

import java.awt.event.MouseEvent;

import org.openmis.tellus.gvsig.gui.AbstOpenMISMainWindow;
import org.openmis.tellus.gvsig.gui.DefaultOpenMISMainWindow;
import org.openmis.tellus.gvsig.gui.OWMapControl;
import org.openmis.tellus.gvsig.gui.WaiterDialog;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.LogManager;

import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class GvSIGTellusGUIManager {

	private static GvSIGTellusGUIManager manager;
	
	private AbstOpenMISMainWindow window = null;
	
	public GvSIGTellusGUIManager(AbstOpenMISMainWindow window){
		this.window = window;
	}
	
	public static GvSIGTellusGUIManager getManager(){
		if(manager == null){
			try{
				
				PropertiesManager.getManager().addPropertiesFile(System.getProperty("user.dir")+"/conf/gvsig.properties");
				
				String path  = PropertiesManager.getManager().getProperty("path.gvsig.window.init");
				Object toload = Class.forName(path).newInstance();
				if(toload  != null && toload instanceof  AbstOpenMISMainWindow){
					manager = new GvSIGTellusGUIManager((AbstOpenMISMainWindow)toload);
				}else{
					manager = new GvSIGTellusGUIManager(new DefaultOpenMISMainWindow());
				}
			
			}catch (Exception e) {
				LogManager.traceError(0, e);
			}
		}
		return manager;
	}
	
	public AbstOpenMISMainWindow getOpenMISMainWindow(){
		return window;
	}
	
	public OWMapControl getMapControl(){
		return (OWMapControl)window.getMapControl();
	}
	
	public void showErrorMessage(String _title, String _msg, MapControl mc) {
		
		String title = (_title != null) ? _title : "NULL";
		String msg = (_msg != null) ? _msg : "NULL";
		
		
		MessageDialog._showModalErrorMessageDialog(
				title,
				msg,
				mc.getParentFrame()
				);
	}
	
	public WaiterDialog showWaiterDialog(MapControl mc) {			
		return WaiterDialog._showModalWaiterDialog(
				mc.getParentFrame()
				);
	}
	
}
