/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.manager;

import java.awt.Shape;

import org.openmis.tellus.exception.SIGException;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.project.DistantProjectManager;

import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.IFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.FBitSet;
import es.prodevelop.gvsig.mobile.gui.dialog.SearchDialog;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class GvSIGTellusGeoManager {
	
	private static GvSIGTellusGeoManager manager;
	
	public GvSIGTellusGeoManager(){
		
	}
	
	public static GvSIGTellusGeoManager getManager(){
		if(manager == null){
			manager = new GvSIGTellusGeoManager();
		}
		return manager;
	}
	
	public  FLyrVect getFLyrVect(String geofname) throws Exception{
		DistantEditableLayer layer = null;
		try{
			layer = (DistantEditableLayer)DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(geofname);
		}catch(Exception ex){
			throw new Exception("the fileName don't define an DistantEditableLayer : " + geofname);
		}
		

		FLayer l = GvSIGTellusGUIManager.getManager().getMapControl().getLayerWithGeoFileName(layer.getGeoFileName());	
		if (l == null) {
			throw new Exception("No vector layer has this geo file: " + geofname);
		}
		
		if (l instanceof FLyrVect) {
			return (FLyrVect)l;
			
		} else {
			throw new Exception("Referenced layer is not a vector layer: " + layer.getGeoFileName());
		}
	}
	
	public double[] getCoordinatesForId(
			String geo_file,
			String id_val) throws SIGException {
		
		DistantInfoableLayer layer = null;
		try{
			layer = (DistantInfoableLayer)DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(geo_file);
		}catch(Exception ex){
			throw new SIGException("the fileName don't define a DistantInfoableLayer : " + geo_file);
		}
		

		FLayer l = GvSIGTellusGUIManager.getManager().getMapControl().getLayerWithGeoFileName(geo_file);
		if (l == null) {
			throw new SIGException("No vector layer has this geo file: " + geo_file);
		}

		try{
			return getCoordinateForLayer(l, layer, id_val);
		}catch(Exception ex){
			/*if(layer instanceof DistantEditableLayer){
				
				l = GvsigMobileApp.getApp().getMapControl().getLayerWithGeoFileName(((DistantEditableLayer)layer).getEditableGeoFileName());
				if (l == null) {
					logger.warn("getCoordinatesForId - No vector layer has this geo file: " + geo_file);
					throw new Exception("No vector layer has this geo file: " + geo_file);
				}
				
				return getCoordinateForLayer(l, layer, id_val);
			}*/
		}
		return null;
		
	}
	
	private double[] getCoordinateForLayer(FLayer l, DistantInfoableLayer layer, String id_val) throws SIGException{
		try{
			if (l instanceof FLyrVect) {			
				FLyrVect lyr = (FLyrVect) l;
				SelectableDataSource sds = null;
				int id_ind = 0;
				long row_cnt = 0;
				ReadableVectorial rv = null;
				
				// =========================================
				if(layer.getDistantInteraction() == null)
					throw new SIGException("the DistantLayer "+layer.getGeoFileName()+" must define the distantInteraction not NULL");
				try {
					rv = lyr.getSource();
					sds = rv.getRecordset();
					id_ind = sds.getFieldIndexByName(layer.getDistantInteraction().getGeoFieldLinker());
					row_cnt = sds.getRowCount();
				} catch (Exception ex) {
					throw new SIGException("While getting fields info: " + ex.getMessage());
				}
				// =========================================
				
				Value id_item = null;
				boolean found = false;
				long found_index = 0;
				for (long i=0; i<row_cnt; i++) {
					id_item = sds.getFieldValue(i, id_ind);
					if (id_item.toString().trim().compareTo(id_val.trim()) == 0) {
						found = true;
						found_index = i;
						break;
					}
				}
				
				if (! found) {
					throw new SIGException("ID not found in layer: " + id_val);
				}
				
				IFeature ifeat = rv.getFeature((int) found_index);
				if (ifeat == null) {
					throw new SIGException("Found a NULL feature");
				}
				
				IGeometry ig = ifeat.getGeometry();
				if (ig == null) {
					throw new SIGException("Found a NULL geometry");
				}
				
				Shape shp = ig.getInternalShape();
				if (shp == null) {
					throw new SIGException("Found a NULL shape");
				}
				
				double[] resp = new double[2];
				
				if (shp instanceof FPoint2D) {
					FPoint2D shpp = (FPoint2D) shp;
					resp[0] = shpp.getX(); 
					resp[1] = shpp.getY(); 
				} else {
					resp[0] = ig.getBounds2D().getCenterX();
					resp[1] = ig.getBounds2D().getCenterY();
				}
	
				return resp;
		
			} else {
				throw new SIGException("Referenced layer is not a vector layer: " + l.getName());
			}
		}catch(Exception e){
			throw new SIGException(e);
		}
	}
	
	
	public void executeZoomToElement(FLyrVect lyr, String fld_name, String val) throws SIGException{
		
		try {
			FBitSet fbs = new FBitSet();
			ReadableVectorial rv = lyr.getSource();
			SelectableDataSource sds = rv.getRecordset();
			Value userv, v;
			
			int cnt = (int) sds.getRowCount();
			if (cnt < 1) 
				throw new SIGException("Count row in Layer is Negative!");
			
			int att_ind = getAttributeIndex(
					sds.getFieldNames(),
					fld_name);

			if (att_ind == -1) {
				throw new SIGException("Attribute not found: " + fld_name);
			}
			

				
			

			boolean is_numeric = SearchDialog.isNumericType(sds.getFieldType(att_ind));

			if (is_numeric) {
				userv = ValueFactory.createValue(Double.parseDouble(val));
			} else {
				userv = ValueFactory.createValue(val);
			}
			
			for (int i=0; i<cnt; i++) {
				v = sds.getFieldValue(i, att_ind);
				//if (((BooleanValue) v.equals(userv)).getValue())
				if (v.toString().trim().compareTo(userv.toString().trim()) == 0)
					fbs.set(i);
			}
			
			if (!fbs.isEmpty()) {
				if(! GvSIGTellusGUIManager.getManager().getMapControl().isProcessing())
					GvSIGTellusGUIManager.getManager().getMapControl().setProcessing(true);
				sds.setSelection(fbs);
				GvSIGTellusGUIManager.getManager().getMapControl().zoomToSelection(lyr);	
			}else{
				throw new SIGException("Zoom impossible");
			}
			
		} catch (Exception ex) {
			throw new SIGException("While zooming to item: " + ex.getMessage());
		}
	}

	
	private int getAttributeIndex(String[] ff, String f) {
		for (int i=0; i<ff.length; i++) {
			if (ff[i].compareToIgnoreCase(f) == 0) return i; 
		}
		return -1;
	}
}
