/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.manager;

import java.util.List;

import org.gvsig.mobile.location.GPSManager;
import org.gvsig.mobile.location.tracklog.RawDataManager;
import org.gvsig.mobile.location.tracklog.TracklogManager;
import org.gvsig.mobile.location.tracklog.WaypointManager;
import org.openmis.tellus.gvsig.fmap.FLayerFileVectSymb;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.symbo.DistantLayerSymbology;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.project.impl.DefaultDistantProject;
import org.openmobileis.common.util.exception.DatabaseException;

import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerGenericRaster;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.legend.SingleSymbolLegend;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public class GvSIGTellusProjectManager {

	private static GvSIGTellusProjectManager manager;
	
	public GvSIGTellusProjectManager(){
		
	}
	
	public static GvSIGTellusProjectManager getManager(){
		if(manager == null){
			manager = new GvSIGTellusProjectManager();
		}
		return manager;
	}
	
	
	public void loadFile(String projectName, DistantLayer layer) throws Exception{
		//--- Get the project or create it ---------
		DistantProject project = DistantProjectManager.getManager().getDistantProject(projectName);
		if(project  == null){
			if(projectName != null){
				project = new DefaultDistantProject(projectName);	
			}
		}
		
		if(project != null){
			project.addDistantLayer(layer);
				
		}		
		DistantProjectManager.getManager().storeDistantProject(project);
	}
	
	public void loadProject(String usedProjectName) throws Exception {
		String error = loadDistantProject(usedProjectName);
		if(error != null){
			throw new Exception(error);
		}else{
			GvSIGTellusGUIManager.getManager().getOpenMISMainWindow().initializeDistantTools();
		}
	}
	
	
	public String loadDistantProject(String usedProjectName){
		try {
			DistantProject currentProject = DistantProjectManager.getManager().getDistantProject(usedProjectName);
			if(currentProject == null){
				return "The distantProject is not define : "+usedProjectName+". Please use the LoadFile distant service before";
			}else{
				DistantProjectManager.getManager().setProjectAsCurrent(currentProject.getUniqueProjectName());
				//remove layers present before
				GvSIGTellusGUIManager.getManager().getMapControl().removeAllLayers();
				//Thread.sleep(100);
				//getMapControl().setProjection(new IP)
		
				//load the layers
				List list = currentProject.getClassifiedDistantLayersList();
				for(int i=0;i<list.size();i++){
					DistantLayer layer = (DistantLayer)list.get(i);
					if(layer != null){
						String error = loadDistantLayer(layer);
						if(error != null)
							return error;
					}
				}
			}
			
		} catch (Exception e) {
			return "FATAL error DistantProjectManager : "+e.getMessage();
		}
		return null;
	}
	
	
	private String loadDistantLayer(DistantLayer layer){
		try{
			
			if (layer.getExtension().equalsIgnoreCase(DistantLayer.EXT_SHP)){
				FLayer fl = FLayerFileVectSymb.createLayer(layer.getFile(), layer.getGeoFileName()
						, GvSIGTellusGUIManager.getManager().getMapControl().getProject().getMapContext());
				if(fl != null){
					//Thread.sleep(50);
					GvSIGTellusGUIManager.getManager().getMapControl().addLayer(fl,true);		
					manageSymbology(layer, fl);
					GvSIGTellusGUIManager.getManager().getMapControl().draw(true);
					
				}else{
					return "The layer is Null, Verify the compatibility of the file";
				}
		
			
				/*if(layer instanceof DistantEditableLayer && ((DistantEditableLayer)layer).getDistantInteraction() != null){	
					fl = createCSVLayer((DistantEditableLayer)layer, getMapControl().getProject().getMapContext(), getMapControl().getProjection());
					if(fl != null){
						getMapControl().addLayer(fl,true);
						manageSymbology(layer, fl);
						getMapControl().draw(true);
					}else{
						return "The layer is CVS created is Null";
					}
				}*/
			}else if (layer.getExtension().equalsIgnoreCase(DistantLayer.EXT_ECW)) {
				FLayerGenericRaster rasterLayer = FLayerGenericRaster.createEcwLayer(layer.getFile(), layer.getGeoFileName()
						, GvSIGTellusGUIManager.getManager().getMapControl().getProject().getMapContext());
				if (rasterLayer != null) {
					rasterLayer.setVisible(true);
					rasterLayer.setName(layer.getGeoFileName());
					
					//Thread.sleep(50);
					GvSIGTellusGUIManager.getManager().getMapControl().addLayer(rasterLayer,true);
					GvSIGTellusGUIManager.getManager().getMapControl().draw(true);
					
				}else{
					return "The layer is Null, Verify the compatibility of the file";
				}
			}
			
			
		}catch(Exception ex){
			return ex.getMessage();
		}
		return null;
	}
	
	private void manageSymbology(DistantLayer layer, FLayer fl){
		if(fl instanceof FLyrVect && layer.getSymbology() != null){
			FLyrVect layerTemp = (FLyrVect)fl;
			FSymbol symbol = ((SingleSymbolLegend) layerTemp.getLegend()).getDefaultSymbol();
			DistantLayerSymbology distantSymbo = layer.getSymbology();
			
			if(distantSymbo.getColor() != null)
				symbol.setBorderColor(distantSymbo.getColor());
			if(distantSymbo.getColorFilled() != null){
				symbol.setFillColor(distantSymbo.getColorFilled());
				symbol.setFilled(distantSymbo.isFilled());
			}

			symbol.setStrokeWidth(distantSymbo.getSize());
			
			((SingleSymbolLegend) layerTemp.getLegend()).setDefaultSymbol(symbol);
		}
	}
	
	
	public String unloadDistantProject(String usedProjectName){
		try {
			DistantProject currentProject = DistantProjectManager.getManager().getDistantProject(usedProjectName);
			if(currentProject == null){
				return "The distantProject is not define : "+usedProjectName+". Please use the LoadFile distant service before";
			}else{

				TracklogManager.closeTracklog();
				WaypointManager.closeWaypoints();
				RawDataManager.closeRawData();
				GPSManager.getGPSManager().disconnectFromGPS();
				GvSIGTellusGUIManager.getManager().getMapControl().closeDataSources();
				
				//unload the layers
				List list = currentProject.getClassifiedDistantLayersList();
				for(int i=0;i<list.size();i++){
					DistantLayer layer = (DistantLayer)list.get(i);
					if(layer != null){
						FLayer lyr = GvSIGTellusGUIManager.getManager().getMapControl().getLayerWithGeoFileName(layer.getGeoFileName());
						GvSIGTellusGUIManager.getManager().getMapControl().removeLayer(lyr);
						/*if(layer instanceof DistantEditableLayer){
							DistantEditableLayer editLayer = (DistantEditableLayer)layer;
							
							lyr = getMapControl().getLayerWithGeoFileName(editLayer.getEditableGeoFileName());
							getMapControl().removeLayer(lyr);
						}*/
					}
				}
				
				DistantProjectManager.getManager().setProjectAsCurrent(null);
			}
			
		} catch (DatabaseException e) {
			return e.getMessage();
		}
		return null;
	}
	
}
