/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.services;

import java.awt.Label;
import java.awt.Shape;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.openmis.com.inter.agents.sax.data.CommParameter;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmis.tellus.gvsig.gui.DistantQueryAttributesDialog;
import org.openmis.tellus.gvsig.gui.UrlLabel;
import org.openmis.tellus.gvsig.manager.GvSIGTellusGUIManager;
import org.openmis.tellus.gvsig.manager.GvSIGTellusGeoManager;
import org.openmis.tellus.gvsig.manager.GvSIGTellusProjectManager;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantProjectManager;

import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.app.GvsigMobileApp;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.IFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;

public class ServiceComGvSIGInformation implements DistantService {

	private static Logger logger = Logger.getLogger(ServiceComGvSIGInformation.class);
	public static final String DISTANT_URI = "getInformationFromID";
	
	private String ID = null;
	private String geoFileName = null;
	
	public ServiceComGvSIGInformation(){
	}
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}
	
	public void setGeoFileName(String geoFileName) {
		this.geoFileName = geoFileName;
	}
	
	public void setID(String id) {
		ID = id;
	}

	public void requestSend(DistantSendMessageRequest req) {

		req.addParameter("ID", ID, DistantMessage.TYPE_STRING);
		req.addParameter("geoFileName", geoFileName, DistantMessage.TYPE_STRING);

	}

	public void responseSend(DistantSendMessageResponse res) {
		if(res.isError()){
			GvSIGTellusGUIManager.getManager().showErrorMessage("Error OpenMIS info", res.getErrorMessage(), GvSIGTellusGUIManager.getManager().getMapControl());
		}else{
			logger.debug("performAction - getObjectInfoFromIDResponse - 0");
			
			Map map = new HashMap();
			map.put("OpenMIS Info", null);
			
			List keys = res.getParamKeys();
			for (int j=0; j<keys.size(); j++) {
				String key = (String) keys.get(j);
				
				CommParameter cp = res.getParameter(key);
				if (key.compareToIgnoreCase("link") == 0) {
					
					URL u = null;
					
					try {
						u = new URL(cp.getDataValue());
						
						map.put("LINK", new UrlLabel(cp.getMetaInfo(), u));
						
					} catch (MalformedURLException e) {
						logger.debug("Bad URL: " + cp.getDataValue());
					}
				} else {
					map.put(cp.getMetaInfo(), new Label(cp.getDataValue()));
				}
			}
			
			new DistantQueryAttributesDialog(GvSIGTellusGUIManager.getManager().getMapControl().getParentFrame(), map);
		}
	}
	
	
	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		String geoFileName = req.getParameter("geoFileName").getDataValue();
		String ID = req.getParameter("ID").getDataValue();
		
		try {
			double coords[] = GvSIGTellusGeoManager.getManager().getCoordinatesForId(geoFileName, ID);
			if(coords != null){
				res.addParameter("coordX", "" + coords[0], DistantMessage.TYPE_DOUBLE);
				res.addParameter("coordY", "" + coords[1], DistantMessage.TYPE_DOUBLE);
			}else{
				res.setErrorMessage("Return of coordinates NULL");
			}
		} catch (Exception e) {
			res.setErrorMessage(e.getMessage());
		}
	}
	

}
