/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.services;

import java.awt.Color;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.impl.DefaultDistantEditableLayer;
import org.openmis.tellus.layers.impl.DefaultDistantInfoableLayer;
import org.openmis.tellus.layers.impl.DefaultDistantInteraction;
import org.openmis.tellus.layers.impl.DefaultDistantLayer;
import org.openmis.tellus.layers.symbo.DefaultDistantLayerSymbology;
import org.openmis.tellus.layers.symbo.Dynamisme;
import org.openmis.tellus.layers.symbo.IconDynamique;
import org.openmis.tellus.layers.symbo.IconSymbology;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.project.impl.DefaultDistantProject;


public class ServiceComGvSIGLoadFile implements DistantService {

	public static String DISTANT_URI = "loadGeoFile";
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}

	public void requestSend(DistantSendMessageRequest req) {
		// TODO Auto-generated method stub

	}

	public void responseSend(DistantSendMessageResponse res) {
		// TODO Auto-generated method stub

	}

	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		String projectName = req.getParameter("projectName").getDataValue();
	
		try {
			//--- Get the project or create it ---------
			DistantProject project = DistantProjectManager.getManager().getDistantProject(projectName);
			if(project  == null){
				if(projectName != null){
					project = new DefaultDistantProject(projectName);	
				}
			}
			
			if(project != null){
				String pathGeoFile = req.getParameter("pathGeoFile").getDataValue();
				String type = req.getParameter("type").getDataValue();
				int position = Integer.parseInt(req.getParameter("position").getDataValue());
								
				File layerFile = new File(pathGeoFile);
				DistantLayer layer = null;
				
				if(DistantLayer.TYPE.equals(type)){
					layer = new DefaultDistantLayer(layerFile);
					
				}else if(DistantInfoableLayer.TYPE.equals(type)){
					boolean interaction = false;
					if("true".equals(req.getParameter("interaction").getDataValue())){
						interaction = true;
					}

					if(interaction){
						String fieldName = req.getParameter("fieldName").getDataValue();
						layer = new DefaultDistantInfoableLayer(layerFile, new DefaultDistantInteraction(fieldName));
					}else{
						layer = new DefaultDistantInfoableLayer(layerFile);
					}
					
				}else if(DistantEditableLayer.TYPE.equals(type)){
					boolean interaction = false;
					if("true".equals(req.getParameter("interaction").getDataValue())){
						interaction = true;
					}

					if(interaction){
						String fieldName = req.getParameter("fieldName").getDataValue();
						layer = new DefaultDistantEditableLayer(layerFile, new DefaultDistantInteraction(fieldName));
					}else{
						layer = new DefaultDistantEditableLayer(layerFile);
					}
				}				
				
				if(layer != null){
					
					boolean symbology = false;
					if("true".equals(req.getParameter("symbology").getDataValue())){
						symbology = true;
					}
					
					if(symbology){
						layer.setSymbology(new DefaultDistantLayerSymbology());
						if(req.getParameter("symboColorRGB") != null){
							int colorRGB = Integer.parseInt(req.getParameter("symboColorRGB").getDataValue());
							layer.getSymbology().setColor(new Color(colorRGB));
						}
						if(req.getParameter("symboColorFillRGB") != null){
							int colorRGB = Integer.parseInt(req.getParameter("symboColorFillRGB").getDataValue());
							layer.getSymbology().setColorFilled(new Color(colorRGB));
							
							if(req.getParameter("symboIsColorFilled") != null){		
								boolean isFilled = false;
								if("true".equals(req.getParameter("symboIsColorFilled").getDataValue())){
									isFilled = true;
								}
								
								layer.getSymbology().setFilled(isFilled);
							}
						}
						if(req.getParameter("symboSize") != null){
							int size = Integer.parseInt(req.getParameter("symboSize").getDataValue());
							layer.getSymbology().setSize(size);
						}
						
						
						boolean symbologyIcon = false;
						if("true".equals(req.getParameter("symbologyIcon").getDataValue())){
							symbologyIcon = true;
						}
						
						if(symbologyIcon){
							layer.getSymbology().setIconSymbology(new IconSymbology());
							if(req.getParameter("defaultIconName") != null){
								String defaultIconName = req.getParameter("defaultIconName").getDataValue();
								layer.getSymbology().getIconSymbology().setDefaultIconName(defaultIconName);
							}
							
							boolean symbologyDynamisme = false;
							if("true".equals(req.getParameter("symbologyDynamisme").getDataValue())){
								symbologyDynamisme = true;
							}
							
							
							if(symbologyDynamisme){
								Dynamisme dyna = new Dynamisme(req.getParameter("dynaFieldRef").getDataValue());
								int sizeIcon = Integer.parseInt(req.getParameter("dynaIconSize").getDataValue());
								for( int i=0;i<sizeIcon;i++){
									String value = req.getParameter("dynaIconValue"+i).getDataValue();
									String iconName = req.getParameter("dynaIconName"+i).getDataValue();
									
									boolean isDynaButEdit = false;
									if("true".equals(req.getParameter("dynaButEdit"+i).getDataValue())){
										isDynaButEdit = true;
									}
									
									IconDynamique iconDynamique = new IconDynamique(value, iconName);
									iconDynamique.setButtonEdit(isDynaButEdit);
									
									if(isDynaButEdit){
										boolean isSelectable = false;
										if("true".equals(req.getParameter("dynaButSelect"+i).getDataValue())){
											isSelectable = true;
										}
										
										boolean isActionable = false;
										if("true".equals(req.getParameter("dynaButAction"+i).getDataValue())){
											isActionable = true;
										}
										
										String iconButName = req.getParameter("dynaButIconName"+i).getDataValue();
										String iconButSelectName = req.getParameter("dynaButSelectIconName"+i).getDataValue();
										
										iconDynamique.setSelectableButton(isSelectable);
										iconDynamique.setActionableButton(isActionable);
										iconDynamique.setIconButtonName(iconButName);
										iconDynamique.setIconButtonSelectName(iconButSelectName);
									}
									
									dyna.addIconDynamic(value, iconDynamique);
								}
								
								layer.getSymbology().getIconSymbology().setDynamisme(dyna);
							}
							
						}
					}
					
					
					layer.setPosition(position);
					project.addDistantLayer(layer);
				}
				
				DistantProjectManager.getManager().storeDistantProject(project);
			}
			
		} catch (Throwable e) {
			res.setErrorMessage("FATAL error DistantFileLoader : "+e.getMessage());
		}

	}

	
}
