/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.services;

import java.io.File;

import org.gvsig.mobile.location.GPSManager;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.impl.DefaultDistantEditableLayer;
import org.openmis.tellus.layers.impl.DefaultDistantInfoableLayer;
import org.openmis.tellus.layers.impl.DefaultDistantInteraction;
import org.openmis.tellus.layers.impl.DefaultDistantLayer;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.project.impl.DefaultDistantGPSManager;
import org.openmis.tellus.project.impl.DefaultDistantProject;

public class ServiceComGvSIGLoadGPSManager implements DistantService {

	public static String DISTANT_URI = "loadGPSManager";
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}

	public void requestSend(DistantSendMessageRequest req) {
		// TODO Auto-generated method stub

	}

	public void responseSend(DistantSendMessageResponse res) {
		// TODO Auto-generated method stub

	}

	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		String projectName = req.getParameter("projectName").getDataValue();
		try {
			//--- Get the project or create it ---------
			DistantProject project = DistantProjectManager.getManager().getDistantProject(projectName);
			if(project  == null){
				if(projectName != null){
					project = new DefaultDistantProject(projectName);	
				}
			}
			
			if(project != null){	
				DistantGPSManager gps = new DefaultDistantGPSManager();
				
				boolean isviewgpsposition = false;
				if("true".equals(req.getParameter("isviewgpsposition").getDataValue())){
					isviewgpsposition = true;
				}
				
				gps.setViewGPSPosition(isviewgpsposition);
				
				if(req.getParameter("simulationfile") != null){
					String simuFilePath = req.getParameter("simulationfile").getDataValue();
					gps.setSimulationFile(new File(simuFilePath));
				}
				
				if(req.getParameter("trackerfile") != null){
					String trackerFilePath = req.getParameter("trackerfile").getDataValue();
					gps.setTrackerFile(new File(trackerFilePath));
					if(req.getParameter("autogenerationshp") != null){
						String autoGeneSHP = req.getParameter("autogenerationshp").getDataValue();
						gps.setAutoGenerationFile(new File(autoGeneSHP));
					}
				}
				
				project.setGPSManager(gps);
				DistantProjectManager.getManager().storeDistantProject(project);
			}
			
		} catch (Throwable e) {
			res.setErrorMessage("FATAL error DistantGPSManagerLoader : "+e.getMessage());
		}
	}

}
