/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.services;

import java.awt.Rectangle;
import java.awt.Shape;

import org.apache.log4j.Logger;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmis.tellus.gvsig.manager.GvSIGTellusGUIManager;
import org.openmis.tellus.gvsig.manager.GvSIGTellusGeoManager;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.project.DistantProjectManager;

import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.gvsig.mobile.app.GvsigMobileApp;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.FBitSet;
import es.prodevelop.gvsig.mobile.gui.dialog.SearchDialog;

public class ServiceComGvSIGSituation implements DistantService {

	public static final String DISTANT_URI = "situateGeometricID";
	private static Logger logger = Logger.getLogger(ServiceComGvSIGSituation.class);
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}

	public void requestSend(DistantSendMessageRequest req) {
		// TODO Auto-generated method stub

	}

	public void responseSend(DistantSendMessageResponse res) {
		// TODO Auto-generated method stub

	}

	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		try{
			// -------------------------------------------------
			try{
				ServiceComGvSIGForeground.foreground();
			}catch(Throwable ex){
				res.setErrorMessage("not found processes.dll");
			}
			// -------------------------------------------------
			String geofname = req.getParameter("geoFileName").getDataValue();
			
			DistantInfoableLayer layer = null;
			try{
				layer = (DistantInfoableLayer)DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(geofname);
			}catch(Exception ex){
				logger.warn("the fileName don't define a DistantInfoableLayer " + geofname+" : "+ex.getMessage());
				throw new Exception("the fileName don't define a DistantInfoableLayer : " + geofname);
			}
			
			
			FLayer l = GvSIGTellusGUIManager.getManager().getMapControl().getLayerWithGeoFileName(layer.getGeoFileName());
			if (l == null) {
				logger.warn("No vector layer has this geo file: " + geofname);
				res.setErrorMessage("No vector layer has this geo file: " + geofname);
			}
			
			if (l instanceof FLyrVect) {
				//String fldName = req.getParameter("fieldName").getDataValue();
				String id = req.getParameter("ID").getDataValue();
				
				if(layer.getDistantInteraction() == null)
					throw new Exception("the DistantLayer "+layer.getGeoFileName()+" must define the distantInteraction not NULL");
				
				try{			
					GvSIGTellusGeoManager.getManager().executeZoomToElement((FLyrVect) l, layer.getDistantInteraction().getGeoFieldLinker(), id);
				}catch(Exception ex){
					/*if(layer instanceof DistantEditableLayer){
						l = GvsigMobileApp.getApp().getMapControl().getLayerWithGeoFileName(((DistantEditableLayer)layer).getEditableGeoFileName());
						if (l == null) {
							logger.warn("No vector layer has this geo file: " + geofname);
							res.setErrorMessage("No vector layer has this geo file: " + geofname);
						}
						executeZoomToElement((FLyrVect) l, layer.getDistantInteraction().getGeoFieldLinker(), id);
					}*/
				}
				
			} else {
				logger.debug("Referenced layer is not a vector layer: " + geofname);
				res.setErrorMessage("Referenced layer is not a vector layer: " + geofname);
			}	
		}catch(Exception ex){
			res.setErrorMessage(ex.getMessage());
		}
	}
	
	
}
