/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.gvsig.services;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import org.openmis.tellus.gvsig.manager.GvSIGTellusGUIManager;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.gvsig.edition.GvSIGTellusEditionPointManager;
import org.openmis.tellus.gvsig.edition.DistantEditionPointToolListener;
import org.apache.log4j.Logger;
import org.gvsig.mobile.location.GPSManager;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmobileis.common.util.exception.DatabaseException;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.gvsig.mobile.app.GvsigMobileApp;
import es.prodevelop.gvsig.mobile.fmap.core.DefaultFeature;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.Graph2DUtilities;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.edition.EditionEvent;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;




public class ServiceComGvSIGStorage implements DistantService {

	public static final String DISTANT_URI = "storeGeometricFromID";
	private String geoFileName = null;
	
	public String getDistantServiceUri() {
		return "storeGeometricFromID";
	}
	
	public void setGeoFileName(String geoFileName){
		this.geoFileName = geoFileName;
	}

	public void requestSend(DistantSendMessageRequest req) {
		req.addParameter("geoFileName", geoFileName, DistantMessage.TYPE_STRING);
	}

	public void responseSend(DistantSendMessageResponse res) {
		// TODO Auto-generated method stub

	}

	public void runCall(DistantCallMessageRequest req,DistantCallMessageResponse res) {
		try{
			if(DistantProjectManager.getManager().getCurrentDistantProject() == null){
				res.setErrorMessage("No project load on gvSIG");
			}else{

				String id = req.getParameter("ID").getDataValue();
				String IDfieldName = req.getParameter("IDfieldName").getDataValue();
				String type = req.getParameter("Type").getDataValue();
				String TypefieldName = req.getParameter("TypefieldName").getDataValue();
				
				
				String geoFileName = req.getParameter("geoFileName").getDataValue();
				String gps = req.getParameter("gps").getDataValue();
				
				boolean isGPS = ((gps.compareTo("1") == 0) 
						|| (gps.compareToIgnoreCase("true") == 0));
		
		
				Point2D point = null;
					
					if (isGPS) {
						
						point = GvSIGTellusEditionPointManager.getManager().addGPSPointToLayer(
								geoFileName,
								id,
								IDfieldName,
								type,
								TypefieldName
								);
						
					} else {
						point = GvSIGTellusEditionPointManager.getManager().addPointToLayer(
								geoFileName,
								DistantEditionPointToolListener.getCachedX(),
								DistantEditionPointToolListener.getCachedY(),
								id,
								IDfieldName,
								type,
								TypefieldName,
								false
								);
					}
					// -------------------------------------------------
					try{
						ServiceComGvSIGForeground.foreground();
					}catch(Throwable ex){
						//res.setErrorMessage("not found processes.dll");
					}
					
					if(point != null){
						//FPoint2D pt2d = (FPoint2D)point.getInternalShape();
						res.addParameter("coordx", Double.toString(point.getX()), DistantMessage.TYPE_DOUBLE);
						res.addParameter("coordy", Double.toString(point.getY()), DistantMessage.TYPE_DOUBLE);
						//System.out.println(pt2d.getY());
					}
			}
			// -------------------------------------------------
		} catch (Exception ex) {
			res.setErrorMessage(ex.getMessage());
		}
	}
		
	

}
