/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.service.distant;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.layers.symbo.Dynamisme;
import org.openmis.tellus.layers.symbo.IconDynamique;
import org.openmis.tellus.service.base.DistantTemplateService;

public final class OpenMISLoadFileDistantService implements DistantService {

	public static String DISTANT_URI = "loadGeoFile";
	private DistantLayer layer = null;
	private String projectName = null;
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}
	
	public void setDistantLayer(DistantLayer layer, String projectName){
		this.layer = layer;
		this.projectName = projectName;
	}


	public void requestSend(DistantSendMessageRequest req) {
		if(projectName != null && layer != null && layer.getFile() !=  null){
			req.addParameter("type", DistantLayer.TYPE, DistantMessage.TYPE_STRING);
			req.addParameter("projectName", projectName, DistantMessage.TYPE_STRING);
			req.addParameter("geoFileName", layer.getGeoFileName(), DistantMessage.TYPE_STRING);
			req.addParameter("pathGeoFile", layer.getPathGeoFile(), DistantMessage.TYPE_STRING);
			req.addParameter("position", Integer.toString(layer.getPosition()), DistantMessage.TYPE_INTEGER);
			
			if(layer.getSymbology() != null){
				req.addParameter("symbology", "true", DistantMessage.TYPE_BOOLEAN);
				if(layer.getSymbology().getColor() != null)
					req.addParameter("symboColorRGB", Integer.toString(layer.getSymbology().getColor().getRGB()), DistantMessage.TYPE_INTEGER);
				if(layer.getSymbology().getColorFilled() != null){
					req.addParameter("symboColorFillRGB", Integer.toString(layer.getSymbology().getColorFilled().getRGB()), DistantMessage.TYPE_INTEGER);
					req.addParameter("symboIsColorFilled", Boolean.toString(layer.getSymbology().isFilled()), DistantMessage.TYPE_BOOLEAN);
				}
				req.addParameter("symboSize", Integer.toString(layer.getSymbology().getSize()), DistantMessage.TYPE_INTEGER);
				
				if(layer.getSymbology().getIconSymbology() != null){
					req.addParameter("symbologyIcon", "true", DistantMessage.TYPE_BOOLEAN);
					if(layer.getSymbology().getIconSymbology().getDefaultIconName() != null)
						req.addParameter("defaultIconName", layer.getSymbology().getIconSymbology().getDefaultIconName(), DistantMessage.TYPE_STRING);
					
					if(layer.getSymbology().getIconSymbology().getDynamisme() != null){
						req.addParameter("symbologyDynamisme", "true", DistantMessage.TYPE_BOOLEAN);
						Dynamisme dyna = layer.getSymbology().getIconSymbology().getDynamisme();
						req.addParameter("dynaFieldRef", dyna.getFieldReference(), DistantMessage.TYPE_STRING);
						
						Enumeration enume = dyna.getTableIconDynamic().keys();
						req.addParameter("dynaIconSize", Integer.toString(dyna.getTableIconDynamic().size()), DistantMessage.TYPE_INTEGER);
						
						int compt = 0;
						while(enume.hasMoreElements()){
							String value = (String)enume.nextElement();
							IconDynamique iconDynamique = (IconDynamique)dyna.getTableIconDynamic().get(value);
							if(value != null && iconDynamique != null){
								req.addParameter("dynaIconValue"+compt, value, DistantMessage.TYPE_STRING);
								req.addParameter("dynaIconName"+compt, iconDynamique.getIconName(), DistantMessage.TYPE_STRING);
								
								req.addParameter("dynaButEdit"+compt, Boolean.toString(iconDynamique.isButtonEdit()), DistantMessage.TYPE_BOOLEAN);
								if(iconDynamique.isButtonEdit()){
									req.addParameter("dynaButSelect"+compt, Boolean.toString(iconDynamique.isSelectableButton()), DistantMessage.TYPE_BOOLEAN);
									req.addParameter("dynaButAction"+compt, Boolean.toString(iconDynamique.isActionableButton()), DistantMessage.TYPE_BOOLEAN);
									req.addParameter("dynaButIconName"+compt, iconDynamique.getIconButtonName(), DistantMessage.TYPE_STRING);
									req.addParameter("dynaButSelectIconName"+compt, iconDynamique.getIconButtonSelectName(), DistantMessage.TYPE_STRING);
								}
								
								compt++;
							}	
						}
						
					}else{
						req.addParameter("symbologyDynamisme", "false", DistantMessage.TYPE_BOOLEAN);
					}
					
				}else{
					req.addParameter("symbologyIcon", "false", DistantMessage.TYPE_BOOLEAN);
				}

				
			}else{
				req.addParameter("symbology", "false", DistantMessage.TYPE_BOOLEAN);
			}
			
			if(layer instanceof DistantInfoableLayer){
				req.addParameter("type", DistantInfoableLayer.TYPE, DistantMessage.TYPE_STRING);
				if( ((DistantInfoableLayer)layer).getDistantInteraction() != null){
					req.addParameter("interaction", "true", DistantMessage.TYPE_BOOLEAN);
					req.addParameter("fieldName", ((DistantInfoableLayer)layer).getDistantInteraction().getGeoFieldLinker(), DistantMessage.TYPE_STRING);
					req.addParameter("classLink", ((DistantInfoableLayer)layer).getDistantInteraction().getLinkedClasse().toString(), DistantMessage.TYPE_CLASS);
				}else{
					req.addParameter("interaction", "false", DistantMessage.TYPE_BOOLEAN);
				}
				
			}
			if(layer instanceof DistantEditableLayer){
				req.addParameter("type", DistantEditableLayer.TYPE, DistantMessage.TYPE_STRING);
				if( ((DistantEditableLayer)layer).getDistantInteraction() != null){
					req.addParameter("interaction", "true", DistantMessage.TYPE_BOOLEAN);
					req.addParameter("fieldName", ((DistantEditableLayer)layer).getDistantInteraction().getGeoFieldLinker(), DistantMessage.TYPE_STRING);
					req.addParameter("classLink", ((DistantEditableLayer)layer).getDistantInteraction().getLinkedClasse().toString(), DistantMessage.TYPE_CLASS);
				}else{
					req.addParameter("interaction", "false", DistantMessage.TYPE_BOOLEAN);
				}
			}
			
		}
	}


	public void responseSend(DistantSendMessageResponse res) {
		// TODO Auto-generated method stub

	}


	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		// TODO Auto-generated method stub

	}

}
