/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.service.distant;

import java.util.Enumeration;

import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantService;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantGPSManager;
import org.openmis.tellus.project.DistantProject;
import org.openmobileis.common.util.log.LogManager;

public final class OpenMISLoadProjectDistantService implements DistantService {
	public static String DISTANT_URI = "loadProject";
	
	private DistantProject project = null;
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}
	
	public void setCurrentDistantProject(DistantProject project){
		this.project = project;
	}

	public void requestSend(DistantSendMessageRequest req) {	
		if(project != null){
			loadLayers();
			loadGPSManager();
			req.addParameter("usedProjectName", project.getUniqueProjectName(), DistantMessage.TYPE_STRING);
		}
	}

	public void responseSend(DistantSendMessageResponse res) {
		
	}

	public void runCall(DistantCallMessageRequest req,DistantCallMessageResponse res) {
		try{
			if(req.getParameter("askforproject") != null){
				sendLater later = new sendLater();
				later.start();	//to send the project before because is using the transaction HTTP and can't execute two transactions in the same time
			}
		}catch(Exception ex){
			LogManager.traceError(0,ex);
			res.setErrorMessage(ex.getMessage());
		}
	}
	
	
	private void loadLayers(){
		try{
			OpenMISLoadFileDistantService load = (OpenMISLoadFileDistantService)DistantServiceManager.getManager().getDistantServiceByURI(OpenMISLoadFileDistantService.DISTANT_URI);
			
			Enumeration enume = project.getDistantLayersTable().elements();
			while(enume.hasMoreElements()){
				DistantLayer layer = (DistantLayer)enume.nextElement();
				load.setDistantLayer(layer, project.getUniqueProjectName());
				DistantServiceManager.getManager().runDistantService(load.getDistantServiceUri());
			}
		}catch(Exception ex){
			LogManager.traceError(0, "Please verify the DistantService used is load in your appplication (SIGOpenMISInit.class implementation)");
		}
	}
	
	private void loadGPSManager(){
		try{
			OpenMISLoadGPSManagerDistantService load = (OpenMISLoadGPSManagerDistantService)DistantServiceManager.getManager().getDistantServiceByURI(OpenMISLoadGPSManagerDistantService.DISTANT_URI);
			
			if(project.getGPSManager() != null){
				DistantGPSManager gps = project.getGPSManager();
				load.setDistantGPSManager(gps, project.getUniqueProjectName());
				DistantServiceManager.getManager().runDistantService(load.getDistantServiceUri());
			}
		}catch(Exception ex){
			LogManager.traceError(0, "Please verify the DistantService used is load in your appplication (SIGOpenMISInit.class implementation)");
		}
	}
	
	private class sendLater extends Thread{
		
		public sendLater(){
			
		}
		
		public void run() {
			try {
				DistantServiceManager.getManager().runDistantService(DISTANT_URI);
			} catch (CommException e) {
				LogManager.traceError(0, e);
			}
		}
	}

}
