/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.service.distant.template;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.sax.data.CommMessage;
import org.openmis.com.inter.agents.sax.data.CommMessageSOAP;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.service.base.DistantTemplateService;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.common.ServiceManager;

public abstract class OpenMISDeleteGeoDistantTemplateService extends DistantTemplateService {

	public static final String DISTANT_URI = "removeGeometricFromID";
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}

	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/com/delete/object";
	}
	
	public void requestSend(DistantSendMessageRequest req) {
		try{
			if(getDistantEditableLayer() != null){
				String geoFileName = getDistantEditableLayer().getGeoFileName();

				if(getDistantEditableLayer().getDistantInteraction()  == null)
					throw new CommException("The layer "+geoFileName+" must implements have a DistantInteraction not NULL");
				
				String fieldName = getDistantEditableLayer().getDistantInteraction().getGeoFieldLinker();
				
				
				if(getObjectID() != null && fieldName != null && geoFileName !=null){
					req.addParameter("ID",getObjectID(),DistantMessage.TYPE_STRING);
					req.addParameter("fieldName",fieldName,DistantMessage.TYPE_STRING);
					req.addParameter("geoFileName",geoFileName,DistantMessage.TYPE_STRING);	
				}
			}

		}catch(Exception e){
			LogManager.traceError(0, e);
		}
	}
	
	public void responseSend(DistantSendMessageResponse res) {
		this.response(res, getObjectID());	//A CHANGER
	}
	
	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		//nothing to do
	}

	
	public abstract String getObjectID();
	public abstract void response(DistantSendMessageResponse res, String idObject);
	public abstract DistantEditableLayer getDistantEditableLayer();


}
