/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.service.distant.template;


import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.service.base.DistantTemplateService;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.common.ServiceManager;

public abstract class OpenMISInformationDistantTemplateService extends DistantTemplateService {

	public static final String DISTANT_URI = "getInformationFromID";
	
	public OpenMISInformationDistantTemplateService(){
		super();
	}
	
	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+"/com/get/infos";
	}

	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}
	
	
	public void requestSend(DistantSendMessageRequest req) {
		try{
			if(getDistantInfoableLayer() != null){
				String geoFileName = getDistantInfoableLayer().getGeoFileName();
				
				if(getDistantInfoableLayer().getDistantInteraction()  == null)
					throw new CommException("The layer "+geoFileName+" must implements have a DistantInteraction not NULL");
				
				String fieldName = getDistantInfoableLayer().getDistantInteraction().getGeoFieldLinker();
				
				if(getObjectID() != null && fieldName != null && geoFileName !=null){
					req.addParameter("ID",getObjectID(),DistantMessage.TYPE_STRING);
					req.addParameter("fieldName",fieldName,DistantMessage.TYPE_STRING);
					req.addParameter("geoFileName",geoFileName,DistantMessage.TYPE_STRING);	
				}
			}

		}catch(Exception e){
			LogManager.traceError(0,e);
		}
	}
	
	public void responseSend(DistantSendMessageResponse res) {
		// nothing to do
	}
	
	
	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		try{
			String geoFileName = req.getParameter("geoFileName").getDataValue();
			String ID = req.getParameter("ID").getDataValue();
			
			DistantLayer layer = DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(geoFileName);
			if(layer != null && layer instanceof DistantInfoableLayer){
				runCall(ID, (DistantInfoableLayer)layer, res);
			}else{
				res.setErrorMessage("Unknow corresponding between SIG file and SI datas");
			}
		}catch(Exception e){
			LogManager.traceError(0,e);
			res.setErrorMessage(e.getMessage());
		}
	}
	
	public abstract String getObjectID();
	public abstract DistantInfoableLayer getDistantInfoableLayer();
	public abstract void runCall(String idObject, DistantInfoableLayer layer, DistantCallMessageResponse res);
}
