/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.service.distant.template;

import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmis.com.inter.agents.exception.CommException;
import org.openmis.com.inter.agents.sax.data.CommMessage;
import org.openmis.com.inter.agents.sax.data.CommMessageSOAP;
import org.openmis.com.inter.agents.sax.data.CommParameter;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantCallMessageResponse;
import org.openmis.com.inter.agents.services.datas.DistantMessage;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageRequest;
import org.openmis.com.inter.agents.services.datas.DistantSendMessageResponse;
import org.openmis.com.inter.agents.services.distant.DistantServiceManager;
import org.openmis.tellus.layers.DistantEditableLayer;
import org.openmis.tellus.layers.DistantInfoableLayer;
import org.openmis.tellus.layers.DistantLayer;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.service.base.DistantTemplateService;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.embedded.util.SystemAPI;
import org.openmobileis.services.common.ServiceManager;

import es.prodevelop.gvsig.mobile.common.ProcessUtils;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public abstract class OpenMISStoreGeoDistantTemplateService extends DistantTemplateService {
	
	public static final String DISTANT_URI = "storeGeometricFromID";
	public static final String SERVICE_URI = "/com/store/object";
	
	public String getDistantServiceUri() {
		return DISTANT_URI;
	}

	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI()+SERVICE_URI;
	}

	
	public void requestSend(DistantSendMessageRequest req) {
		try{
			if(getDistantEditableLayer() != null){
				String geoFileName = getDistantEditableLayer().getGeoFileName();

				if(getDistantEditableLayer().getDistantInteraction()  == null)
					throw new CommException("The layer "+geoFileName+" must implements have a DistantInteraction not NULL");
				
				String IDfieldName = getDistantEditableLayer().getDistantInteraction().getGeoFieldLinker();			
				
				if(getObjectID() != null && IDfieldName != null && geoFileName !=null){
					req.addParameter("ID",getObjectID(),DistantMessage.TYPE_STRING);
					req.addParameter("IDfieldName",IDfieldName,DistantMessage.TYPE_STRING);
					
					req.addParameter("TypefieldName","Type",DistantMessage.TYPE_STRING);
					req.addParameter("Type","1",DistantMessage.TYPE_STRING);
					
					req.addParameter("geoFileName",geoFileName,DistantMessage.TYPE_STRING);
					
					if(positionToGPS()){
						req.addParameter("gps","1",DistantMessage.TYPE_BOOLEAN);	//ajoute intervention a la position GPS actuel
					}else{
						req.addParameter("gps","0",DistantMessage.TYPE_BOOLEAN);	//ajoute intervention a la position du click dans l'outil gvSIG
					}
				}
			}

		}catch(Exception e){
			LogManager.traceError(0, e);
		}
	}
	
	
	public void responseSend(DistantSendMessageResponse res) {
		double coordx = -1;
		double coordy = -1;
		
		try{
			coordx = Double.parseDouble(res.getParameter("coordx").getDataValue());
			coordy = Double.parseDouble(res.getParameter("coordy").getDataValue());
		}catch(Exception e){
		}
		
		this.response(res, coordx, coordy);
	}
	
	public void runCall(DistantCallMessageRequest req, DistantCallMessageResponse res) {
		try{
			String geoFileName = req.getParameter("geoFileName").getDataValue();
			
			DistantLayer layer = DistantProjectManager.getManager().getCurrentDistantProject().getDistantLayer(geoFileName);
			if(layer != null && layer instanceof DistantEditableLayer){
				runCall((DistantEditableLayer)layer, res);
			}else{
				res.setErrorMessage("Unknow corresponding between SIG file and SI datas");
			}
		}catch(Exception e){
			LogManager.traceError(0, e);
			res.setErrorMessage(e.getMessage());
		}
	}

	public abstract void response(DistantSendMessageResponse res, double coordx, double coordy);
	public abstract String getObjectID();
	public abstract boolean positionToGPS();
	public abstract DistantEditableLayer getDistantEditableLayer();
	public abstract void runCall(DistantEditableLayer layer, DistantCallMessageResponse res);
}
