/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.common.data;

import java.io.Serializable;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Interface to implements to communicate with the listener ProjectModificationServiceListener use to facilitate the integration of
 * modifications in projects xml. The ProjectModificationServiceListener is waiting EventFileModification object type.
 *
 * case : <BR>
 *  - ADD : to be use when a new layer is added to the project<BR>
 *  - REPLACE : to be use when a layer is modified in the project<BR>
 *  - DELETE :  to be use when a layer is deleted in the project.<BR>
 *  <BR>
 *  Side (Do not forget!!!): <BR>
 *  - SIDE_CLIENT : call the service with SIDE_CLIENT signicate the modification on the layer is due to the Client (syn_Number == current_Number)<BR>
 *  - SIDE_SERVER : call the service with SIDE_SERVER significate the modification on the layer is due to the Server (sync_Number > current_Number)<BR>
 * @author	Manuel Gomez-Rosado
 * @version 1.0.
 * @see ProjectModificationServiceListener, DefaultEventFileModification
 */
public interface EventFileModification extends Serializable{
	
	static final long serialVersionUID = 5521257935120563452L;
	
	public final static int SIDE_CLIENT = 0;
	public final static int SIDE_SERVER = 1;
	
	public final static int ACTION_ADD = 0;
	public final static int ACTION_REPLACE = 1;
	public final static int ACTION_DELETE = 2;
	
	public int getSideModification();
	public int getActionType();
	public String getProjectName();
	public String getFileName();
	//public String getServerPath();
}
