/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.common.file;

import java.io.IOException;

import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.collection.Array;
import org.xmlpull.v1.XmlPullParserException;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class ProjectFileManager {
	
	private static ProjectFileManager manager;
	private ProjectFileFactory factory;
	
	public ProjectFileManager(ProjectFileFactory factory) {
		this.factory = factory;
	}
	
	public static ProjectFileManager getManager () {
		return manager;
	}
   
	  public static void removeManager()	{
		  manager = null;
	  }
	
	  public ProjectFileFactory getFactory() {
		  return this.factory;
	  }

	public static void registerManager(ProjectFileManager newManager) {
		manager = newManager;
		ApplicationContextManager.getManager().addManager(manager);
	}
	
	
	public Array getAllLayers() throws IOException, XmlPullParserException{
		return this.getFactory().getAllLayers();
	}
	
	public DefaultSynchroLayer getLayerFromID(String id) throws IOException, XmlPullParserException{
		return this.getFactory().getLayerFromID(id);
	}
	public DefaultSynchroLayer getLayerFromFileName(String filename) throws IOException, XmlPullParserException{
		return this.getFactory().getLayerFromFileName(filename);
	}
	
	public void storeNewLayer(DefaultSynchroLayer layer) throws IOException, XmlPullParserException {
		this.getFactory().storeNewLayer(layer);
	}
	
	public void storeNewXML(Array layers) throws IOException, XmlPullParserException {
		this.getFactory().storeNewXML(layers);
	}

	
	public boolean deleteLayerFromID(String id) throws IOException, XmlPullParserException{
		return this.getFactory().deleteLayerFromID(id);
	}

}
