/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.common.file;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.utils.FileLayerUtilities;
import org.openmis.tellus.synchro.common.xml.ParseXMLProject;
import org.openmis.tellus.synchro.common.xml.XMLGenerator;
import org.openmis.tellus.synchro.common.xml.XMLProjectFileGenerator;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.xmlpull.v1.XmlPullParserException;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class XMLProjectFileFactory implements ProjectFileFactory{

	private File xmlFile;
	
	public XMLProjectFileFactory(File xmlFile) {
		super();
		this.xmlFile = xmlFile;
	}
	
	public Array getAllLayers() throws IOException, XmlPullParserException {
		ParseXMLProject parser = new ParseXMLProject(new FileReader(this.xmlFile));
		
	    SynchroAtomicObject[] ret =  parser.parse();
	    Array array = new Array();
	    for(int i=0;i<ret.length;i++){
	    	array.add(ret[i]);
	    }
		return array;
	}
	
	
	public DefaultSynchroLayer getLayerFromID(String id) throws IOException, XmlPullParserException{
		Array array = getAllLayers();
		for(int i=0;i<array.size();i++){
			DefaultSynchroLayer layer = (DefaultSynchroLayer)array.get(i);
			if(layer.getUID().equals(id)){
				return layer;
			}
		}
		return null;
	}
	
	public DefaultSynchroLayer getLayerFromFileName(String fileName) throws IOException, XmlPullParserException{
		Array array = getAllLayers();
		for(int i=0;i<array.size();i++){
			DefaultSynchroLayer layer = (DefaultSynchroLayer)array.get(i);
			if(layer.getFileName().equals(fileName)){
				return layer;
			}
		}
		return null;
	}
		
	
	public void storeNewLayer(DefaultSynchroLayer newLayer) throws IOException, XmlPullParserException {
		Array array = getAllLayers();
			
		boolean replace = false;
		for(int i=0;i<array.size();i++){
			DefaultSynchroLayer OldLayer = (DefaultSynchroLayer)array.get(i);
			
			//----------------- to test if it's a layer replace or an new layer ------------------------
			if(OldLayer.getUID().equals(newLayer.getUID()) || newLayer.getFileName().equals(OldLayer.getFileName())){	//test the ID eguality and the fileName eguality
				//it's a REPLACE type
				
				//----------------- reconstitution of the layer to keep intregration of datas -------
				String fileName = newLayer.getFileName();
				String projectName = newLayer.getProjectName(); 
				long modif = newLayer.getModifSyncNumber();
				
				//OldLayer because the UniqueID is not modificable and must stay in case of replace
				OldLayer.setFileName(fileName);
				OldLayer.setProjectName(projectName);
				OldLayer.setModifSyncNumber(modif);
				//------------------------------------------------------------------------------------
				
				array.replace(i, OldLayer);	//replace oldLayer by the new one modificate
				replace = true;
				break;	//break the for because modification done
			}
		}
		//if it's a ADD type
		if(! replace){
			array.add(newLayer);	//add the new layer in the xml project file	
		}
		
		XMLGenerator generation = new XMLProjectFileGenerator();
		generation.generateNewXMLProjectFile(array);		
		FileLayerUtilities.saveInFile(this.xmlFile.getAbsolutePath(), generation.getInputStreamXML());		
	}
	
	public void storeNewXML(Array layers) throws IOException, XmlPullParserException {
		XMLGenerator generation = new XMLProjectFileGenerator();
		generation.generateNewXMLProjectFile(layers);
		
		FileLayerUtilities.saveInFile(this.xmlFile.getAbsolutePath(), generation.getInputStreamXML());
		
	}
	
	
	public boolean deleteLayerFromID(String id) throws IOException, XmlPullParserException{
		Array array = getAllLayers();
		boolean test = false;
		for(int i=0;i<array.size();i++){
			DefaultSynchroLayer layer = (DefaultSynchroLayer)array.get(i);
			if(layer.getUID().equals(id)){
				array.remove(i);	//delete the layer
				test = true;
			}
		}
		if(test){
			XMLGenerator generation = new XMLProjectFileGenerator();
			generation.generateNewXMLProjectFile(array);
			
			FileLayerUtilities.saveInFile(this.xmlFile.getAbsolutePath(), generation.getInputStreamXML());
		}
		return test;
	}

}
