/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.common.service;


import java.io.File;

import org.openmis.tellus.project.DistantProject;
import org.openmis.tellus.project.DistantProjectManager;
import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.data.EventFileModification;
import org.openmis.tellus.synchro.common.file.ProjectFileManager;
import org.openmis.tellus.synchro.common.file.XMLProjectFileFactory;
import org.openmis.tellus.synchro.terminal.listeners.SynchroLayerListenerProcessing;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.xmlpull.v1.XmlPullParserException;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class ServiceEventFileModification {


	public ServiceEventFileModification(EventFileModification eventFile) throws Exception{
		try{
			
			if(eventFile != null){
				
				String projectName = eventFile.getProjectName();
				String projectsDirectory = null;
				//depending the side modification to select the right folder for project xml
				if(eventFile.getSideModification() == EventFileModification.SIDE_CLIENT){
					projectsDirectory = PropertiesManager.getManager().getProperty("xml.terminal.folder.projects");
					
				}else if(eventFile.getSideModification() == EventFileModification.SIDE_SERVER){
					projectsDirectory = PropertiesManager.getManager().getProperty("xml.server.folder.projects");
				}
				
				File xmlProjectFile = new File(projectsDirectory+File.separator+projectName+".xml");
				if(!xmlProjectFile.exists())
					xmlProjectFile.createNewFile();
				
				//register the manager to write in xmlProjectFile
				ProjectFileManager.registerManager(new ProjectFileManager(new XMLProjectFileFactory(xmlProjectFile)));	
				
				if(eventFile.getActionType() == EventFileModification.ACTION_ADD || eventFile.getActionType() == EventFileModification.ACTION_REPLACE){
					DefaultSynchroLayer layer = this.constructSynchroLayer(eventFile);
					ProjectFileManager.getManager().storeNewLayer(layer);
					
				}else if(eventFile.getActionType() == EventFileModification.ACTION_DELETE){
					DefaultSynchroLayer layer = ProjectFileManager.getManager().getLayerFromFileName(eventFile.getFileName());
					if(layer != null){
						ProjectFileManager.getManager().deleteLayerFromID(layer.getUID());
					}			
				}else{		
					throw new Exception("Object empty");
				}
				
			}else{
				throw new Exception("Object sended is NULL");
			}
			
		}catch(ClassNotFoundException ex){
			LogManager.traceError(0, "Error during execution of service ProjectModificationServiceListener, run() : unserializeData ERROR : "+ex);
			throw new Exception("Object not corresponding a EventFileModification");
		}catch(XmlPullParserException ex){
			LogManager.traceError(0, "Error during execution of service ProjectModificationServiceListener, run() : XMLParser ERROR : "+ex);
		}catch(Exception ex){
			LogManager.traceError(0, "Error during execution of service ProjectModificationServiceListener, run() : Exception ERROR : "+ex);
		}

	}
	
	
	private long getCurrentSyncNumber(EventFileModification event) throws DatabaseException{
		
		if(event.getSideModification() == EventFileModification.SIDE_SERVER){
			return SyncNumberManager.getManager().getCurrentSynchroNumber()+1;	//+1 because modification on server and modifsync_number must be > currentsync_number
			
		}else if (event.getSideModification() == EventFileModification.SIDE_CLIENT){
			String service = SynchroLayerListenerProcessing.getShortSyncName()+event.getProjectName();
			return NumSyncManagerDB.getManager().getSyncNumberForService(service);
		}else {
			throw new DatabaseException("SIDE TO GET SYNC_NUMBER NOT FOUND !!");
		}
	}
	
	private DefaultSynchroLayer constructSynchroLayer(EventFileModification event) throws Exception{
		DefaultSynchroLayer layer = new DefaultSynchroLayer();
		layer.setFileName(event.getFileName());
		layer.setProjectName(event.getProjectName());
		/*if(event.getActionType() == EventFileModification.ACTION_REPLACE && event.getServerPath() == null){
			DefaultSynchroLayer temp = ProjectFileManager.getManager().getLayerFromFileName(event.getFileName());
			if(temp != null){
				layer.setServerPath(temp.getServerPath());
			}else{
				layer.setServerPath("");	//TODO have a server path for layers not defined
			}
		}else{
			layer.setServerPath(event.getServerPath());
		}*/
		long currentSyncNumber = this.getCurrentSyncNumber(event);
		layer.setCreationDate(currentSyncNumber);
		layer.setModifSyncNumber(currentSyncNumber);
		
		return layer;
	}
}
