/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.common.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.log.LogManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class FileLayerUtilities {

	public synchronized static void saveInFile(String completePathFile, InputStream in) throws IOException{
		BufferedInputStream bis = new BufferedInputStream(in);
		File file = new File(completePathFile);
		if(!file.exists())
			file.createNewFile();
		
		FileOutputStream stream = new FileOutputStream(completePathFile);
		BufferedOutputStream bos = null;
		try {
			int read_bytes = 0;
			byte[] buffer = new byte[4096];
			bos = new BufferedOutputStream(stream, buffer.length);
			while ( (read_bytes = bis.read(buffer, 0, buffer.length)) != -1) {
				bos.write(buffer,0, read_bytes);        	
			}
		}catch(IOException ex){
			LogManager.traceError(0, "Error during execution of service FileLayerUtilities, saveInFile() : "+ex);
			throw new IOException("ERROR IN SAVING FILE !! pathFile : "+file.getAbsolutePath());	
		} finally {
			bos.flush();
			bos.close();
			stream.close();
		}

	}
	
	public synchronized static boolean deleteFile(String completePathFile) {
		File filetoDelete = new File (completePathFile);
		if(filetoDelete.exists()){
			return filetoDelete.delete();
		}
		return false;

	}
	
	public synchronized static byte[] readFile(String completePathFile) throws IOException{
		FileInputStream fileStream = new FileInputStream(completePathFile);
		try {
			File file = new File(completePathFile);
	        if(file.exists()){
				//read channel XML
				byte[] buffer = new byte[2048];
				ByteArrayOutputStream stream = new ByteArrayOutputStream(buffer.length);
				int bytes_read;
				try {
					while(( bytes_read = fileStream.read(buffer)) != -1)  // Copy bytes to zipfile
						stream.write(buffer, 0, bytes_read);
				} finally {
					stream.flush();
					stream.close();
				}
				return stream.toByteArray();
	        }else{
	        	throw new IOException("FILE TO READ NOT FOUND !! pathFile : "+file.getAbsolutePath());
	        }
		}catch(IOException ex){
			LogManager.traceError(0, "Error during execution of service FileLayerUtilities, readFile() : "+ex);
			throw new IOException("Error in reading file");
		
		} finally {
			fileStream.close();
		}
		
	}
	
   public synchronized static  byte[] serializeObject(Object obj) throws IOException
   {
       ByteArrayOutputStream bstr;
       bstr = new ByteArrayOutputStream();
       try
       {
           ObjectOutputStream ostr = new ObjectOutputStream(bstr);
           ostr.writeObject(obj);
       }
       finally
       {
           bstr.flush();
           bstr.close();
       }
       return bstr.toByteArray();
   }
   
	public synchronized static Object unserializeData(String data) throws IOException,ClassNotFoundException {
		byte[] b = GeneralCoder.decodeBase64(data.getBytes());
		ByteArrayInputStream bstr = new ByteArrayInputStream(b);
		try {
			ObjectInputStream ostr = new ObjectInputStream(bstr);
			return ostr.readObject();
		} finally {
			bstr.close();
		}
	}

}
