/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.common.xml;

import java.io.IOException;
import java.io.Reader;

import org.kxml2.io.KXmlParser;
import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.xmlpull.v1.XmlPullParserException;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class ParseXMLProject {
	
	private final static String TAG_XML = "xml";
	private final static String TAG_LAYER = "layer";
	
	private Reader XMLProjectReader;
	private Array array;
	

	
	public ParseXMLProject(Reader XMLProjectReader) {
		 this.XMLProjectReader = XMLProjectReader;	   
		 this.array = new Array();
	}
	
	public SynchroAtomicObject[] parse() throws XmlPullParserException, IOException {
	    try {
	      KXmlParser parser = new KXmlParser();
	      parser.setInput(this.XMLProjectReader);
	  
	      int eventType = parser.getEventType();
	      do {
	        if (eventType == KXmlParser.START_DOCUMENT) {
	        } else if (eventType == KXmlParser.END_DOCUMENT) {
	        } else if (eventType == KXmlParser.START_TAG) {
	          this.processStartElement(parser);
	        } else if (eventType == KXmlParser.END_TAG) {
	          this.processEndElement(parser);
	        } else if (eventType == KXmlParser.TEXT) {
	          this.processText(parser);
	        }
	        eventType = parser.next();
	      } while (eventType != KXmlParser.END_DOCUMENT);
	    } finally   {
	      this.XMLProjectReader.close();
	    }
	    
	    SynchroAtomicObject[] ret = new SynchroAtomicObject[array.size()];
	    array.toArray(ret);
	    return ret;
    }
	
	private void processStartElement(KXmlParser parser) {
	    String name = parser.getName();

	    if (name.equals(ParseXMLProject.TAG_XML)) {
	      // Do nothing
	    } else if (name.equals(ParseXMLProject.TAG_LAYER)) {
	      this.openLayerTag(parser);
	   }
	}
	
	private void openLayerTag(KXmlParser parser) {
	    int count = parser.getAttributeCount();
	    
	    DefaultSynchroLayer synchroLayer = null;
	    
	    for (int i = 0; i < count; i++) {
	    	try{	
		      if (parser.getAttributeName(i).equals("ID")) {
		    	  synchroLayer = new DefaultSynchroLayer(parser.getAttributeValue(i));
		    	 
		      }else if (parser.getAttributeName(i).equals("creationDate")) {	  
				 synchroLayer.setCreationDate(Long.parseLong(parser.getAttributeValue(i)));

			  } else if (parser.getAttributeName(i).equals("syncNumberModif")) {
				  synchroLayer.setModifSyncNumber(Long.parseLong(parser.getAttributeValue(i))); 
				  
			  } else if (parser.getAttributeName(i).equals("filename")) {
				  synchroLayer.setFileName(parser.getAttributeValue(i));
				  
			  } else if (parser.getAttributeName(i).equals("projectName")) {
				  synchroLayer.setProjectName(parser.getAttributeValue(i));
				  
		      }else {
		        LogManager.traceWarning(LogServices.SYNCHROSERVICE, "Corrupted module file, unrecognized attribute " + parser.getAttributeName(i) + " for modlayer");
		      }
		    }catch(NumberFormatException ex){
	  		  LogManager.traceError(0,"Error during execution of service ParseXMLProject, openLayerTag() adding syncNumber Layer Error : "+ex);
	  	  	}	 
	    }
	    try{
	    	if(synchroLayer != null){
		    	synchroLayer.setModificationType(SynchroAtomicObject.ADD);
		    	array.add(synchroLayer);
		    }	
	    }catch(Exception ex){
	    	LogManager.traceEmergency(0, "Error during execution of service ParseXMLProject, openLayerTag() adding synchroLayer in array : "+ex);
	    }
	    
	}
    
	private void processText(KXmlParser parser) throws XmlPullParserException {
		    // Do nothing
    }
	
	private void processEndElement(KXmlParser parser) throws XmlPullParserException {
	    // Do nothing
		
	}
}
