/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.server.listeners;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.data.SendSynchroLayer;
import org.openmis.tellus.synchro.common.data.SynchroProject;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.RequestCommand;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.util.MemoryFile;
import org.openmobileis.synchro.security.auth.Credential;


/**
 * @author Manuel Gomez-Rosado
 *
 */
public abstract class SynchroLayerBaseAnalyserTarget extends DefaultOpenMSPSynchroTargetListener {
	
	public SynchroLayerBaseAnalyserTarget() {
		super();
	}


	public String getTargetName() {
		return "XMLAnalyserTarget";
	}


	protected ContainerMessage processMapCommand(Credential cred, ContainerMessage mapContainer) throws OpenMSPException {
		return null;
	}

	
	protected void notifySyncAction(long sessionId, Credential cred, Command syncCommand) throws OpenMSPException {


	}


	protected void processResultCommand(Credential cred, Result resultCommande) throws OpenMSPException {
		

	}


	protected void processStatusCommand(Credential cred, Status statusCommande) throws OpenMSPException {


	}


	protected Status processAddCommand(Credential cred, ContainerMessage addContainer) throws OpenMSPException {
		
		RequestCommand addCommand = (RequestCommand)addContainer.getElement();
		
		try{		
			Status status = new Status(addCommand.getCmdId(), Status.STATUS_FAILED);	
			if(addCommand.getMetaInformation().equals("LAYERSADD")){
				
	    	    while(addContainer.hasMoreMessage()){
	    	    	DataItem item = (DataItem) addContainer.nextMessage().getElement();
	    	    	SendSynchroLayer sendLayer = ((SendSynchroLayer)unserializeData(item.getData()));
		    		this.addLayer(sendLayer.getSynchroLayer(), sendLayer.getBytes());
			
	    	    	status = new Status(addCommand.getCmdId(), Status.STATUS_OK);
	    	    }  
	    	}
			
			return status; 
		}catch(IOException ex)  {
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processAddCommand() : Type IOException "+ex);
	       return new Status(addCommand.getCmdId(), Status.STATUS_FAILED);
	       
		}catch(ClassNotFoundException ex)  {
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processAddCommand() : Type ClassNotFoundException "+ex);
	       return new Status(addCommand.getCmdId(), Status.STATUS_FAILED);
	       
		}catch(Exception ex)  {
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processAddCommand() : Type Exception "+ex);
		    return new Status(addCommand.getCmdId(), Status.STATUS_FAILED);
		}
	}


	protected Status processDeleteCommand(Credential cred, ContainerMessage deleteContainer) throws OpenMSPException {
		
		RequestCommand deleteCommand = (RequestCommand)deleteContainer.getElement();
		try{		
			Status status = new Status(deleteCommand.getCmdId(), Status.STATUS_FAILED);	
			if(deleteCommand.getMetaInformation().equals("LAYERSDELETE")){
				
	    	    while(deleteContainer.hasMoreMessage()){
	    	    	DataItem item = (DataItem) deleteContainer.nextMessage().getElement();	    	    	
	    	    	DefaultSynchroLayer layer = (DefaultSynchroLayer)unserializeData(item.getData());
	    	    	this.removeLayer(layer);
	    	    	status = new Status(deleteCommand.getCmdId(), Status.STATUS_OK);
	    	    }  
	    	}
			
			return status; 
		}catch(IOException ex)  {
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processDeleteCommand() : Type IOException "+ex);
			return new Status(deleteCommand.getCmdId(), Status.STATUS_FAILED);
			
		}catch(ClassNotFoundException ex)  {
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processDeleteCommand() : Type ClassNotFoundException "+ex);
			return new Status(deleteCommand.getCmdId(), Status.STATUS_FAILED);
			
		}catch(Exception ex)  {
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processDeleteCommand() : Type Exception "+ex);
			return new Status(deleteCommand.getCmdId(), Status.STATUS_FAILED);
		}
		
	}


	protected Status processReplaceCommand(Credential cred,ContainerMessage replaceContainer) throws OpenMSPException {
		
		RequestCommand replaceCommand = (RequestCommand)replaceContainer.getElement();
		try{
			Status status = new Status(replaceCommand.getCmdId(), Status.STATUS_FAILED);
			if(replaceCommand.getMetaInformation().equals("XMLADD")){
				 while(replaceContainer.hasMoreMessage()){
		    	    	DataItem item = (DataItem) replaceContainer.nextMessage().getElement();	    	    	
		    	    	Array layers = (Array)unserializeData(item.getData());
		    	    	if(layers != null){
		    			    SynchroProject project = (SynchroProject)unserializeData(item.getMetaInformation());
		    	    		this.replaceXMLbyNew(layers, this.getFileFromProjectName(project.getUniqueProjectName()));
		    	    		status = new Status(replaceCommand.getCmdId(), Status.STATUS_OK);
		    	    	}	
		    	    }
			}
			return status;
		}catch(IOException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processReplaceCommand() : Type IOException "+ex);
			return new Status(replaceCommand.getCmdId(), Status.STATUS_FAILED);
		}catch(ClassNotFoundException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processReplaceCommand() : Type ClassNotFoundException "+ex);
			return new Status(replaceCommand.getCmdId(), Status.STATUS_FAILED);
		}catch(Exception ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processReplaceCommand() : Type Exception "+ex);
			return new Status(replaceCommand.getCmdId(), Status.STATUS_FAILED);
		}
	}


	protected void beginProcessOpenMSpCommand(Credential cred) throws OpenMSPException {
		//this.addFileRepository = null;

	}


	protected ContainerMessage[] endProcessOpenMSpCommand() throws OpenMSPException {
		return null;
	}
	
	
	protected ContainerMessage[] processGetCommand(Credential cred, ContainerMessage getContainer) throws OpenMSPException {
		
		RequestCommand getCommand = (RequestCommand)getContainer.getElement();    
	    try {
	    	if(getCommand.getMetaInformation().equals("XMLREQUEST")){
	    		
	    		//------------------------ GET Informations -----------------------------------------
	    		DataItem item = (DataItem) getContainer.nextMessage().getElement();
	    	    Result result = new Result(getCommand.getCmdId(), this.getTargetName(), item.getSource());
	    	    // ------------------------ Sync Number management -----------------------------------
	    	    long newSyncnNumber = SyncNumberManager.getManager().getNextSynchroNumber().getSynchroNumber();	//Generate NEW SyncID
	    	    result.setMetaInformation(Long.toString(newSyncnNumber));	//send new sync Number   	   
    	    
	    	    //----------------------------- SEND XML PROJECT FILE -------------------------------------------    	    	    
	    	    ContainerMessage[] containerms = new ContainerMessage[2];
	    	    ContainerMessage newContainer = new ContainerMessage(result);
	    	    
	    	    SynchroProject project = (SynchroProject)unserializeData(item.getData());
    	    
	    	    File xmlFile = this.getFileFromProjectName(project.getUniqueProjectName());
	            this.addFileToSynchro(new MemoryFile(File.separator+project.getUniqueProjectName()+File.separator+xmlFile.getName(), new BufferedInputStream(new FileInputStream(xmlFile))));
	    	    Item newItem = new DataItem(Element.ITEM, "XML", xmlFile.getName(), null, null);	   
	    	    newContainer.add(newItem);    
	    	    
	    	    Status status = new Status(getCommand.getCmdId(), Status.STATUS_OK);
	    	    containerms[0] = new ContainerMessage(status);
	    	    containerms[1] = newContainer;
	    	    
	    	    return containerms; 
	    	    
	    	}else if(getCommand.getMetaInformation().equals("LAYERSREQUEST")){
	    		
        	    ContainerMessage[] containerms = new ContainerMessage[2];
        	    ContainerMessage newContainer=null;
        	    
	    	    while(getContainer.hasMoreMessage()){
	    	    	DataItem item = (DataItem) getContainer.nextMessage().getElement();
	    	    	SynchroProject project = (SynchroProject)unserializeData(item.getMetaInformation());
	    	    	DefaultSynchroLayer layer = (DefaultSynchroLayer)unserializeData(item.getData());	    	    	
	    	        
	        	    Result result = new Result(getCommand.getCmdId(), this.getTargetName(), item.getSource());
	        	    result.setMetaInformation("RESULT : Return the layers to the Terminal");
	        	    newContainer = new ContainerMessage(result);
	    	    	Status status=null;
	    	    	
	    	    	File file = this.getFileFromLayer(layer);
	    	    	if(! file.exists()){
	    	    		status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
	    	    	}else{
	    	    		status = new Status(getCommand.getCmdId(), Status.STATUS_OK);

	    	            this.addFileToSynchro(new MemoryFile(File.separator+project.getUniqueProjectName()+File.separator+file.getName(), new BufferedInputStream(new FileInputStream(file))));
	    	            Item newItem = new DataItem(Element.ITEM, "LAYER", file.getName(), null, null);
	    	    	    newContainer.add(newItem);

	    	    	}
	    	    	containerms[0] = new ContainerMessage(status);
	    	    }
	    	    containerms[1] = newContainer;
	    	    
	    	    return containerms; 
	    	}  	
	    	
	    } catch (IOException ex)  {
	    	LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processGetCommand() : Type IOException "+ex);
	        Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
	        status.setCmdRef(getCommand.getCmdId());
	        ContainerMessage statusContainer = new ContainerMessage(status);
	        ContainerMessage[] ret = new ContainerMessage[1];
	        ret[0] = statusContainer;
	        return ret;
	        
	    }catch (ServiceException ex)  {
	    	LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processGetCommand() : Type ServiceException "+ex);
	        Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
	        status.setCmdRef(getCommand.getCmdId());
	        ContainerMessage statusContainer = new ContainerMessage(status);
	        ContainerMessage[] ret = new ContainerMessage[1];
	        ret[0] = statusContainer;
	        return ret;
	        
	    }catch (ClassNotFoundException ex)  {
	    	LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processGetCommand() : Type ClassNotFoundException "+ex);
	        Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
	        status.setCmdRef(getCommand.getCmdId());
	        ContainerMessage statusContainer = new ContainerMessage(status);
	        ContainerMessage[] ret = new ContainerMessage[1];
	        ret[0] = statusContainer;
	        return ret;
	        
	    }catch (DatabaseException ex)  {
	    	LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processGetCommand() : Type DatabaseException "+ex);
	        Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
	        status.setCmdRef(getCommand.getCmdId());
	        ContainerMessage statusContainer = new ContainerMessage(status);
	        ContainerMessage[] ret = new ContainerMessage[1];
	        ret[0] = statusContainer;
	        return ret;
	        
	    }catch (Exception ex)  {
	    	LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyserTarget, processGetCommand() : Type Exception "+ex);
	        Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
	        status.setCmdRef(getCommand.getCmdId());
	        ContainerMessage statusContainer = new ContainerMessage(status);
	        ContainerMessage[] ret = new ContainerMessage[1];
	        ret[0] = statusContainer;
	        return ret;
	    }
	    return null;
	}

	private Object unserializeData(String data) throws IOException,ClassNotFoundException {
		byte[] b = GeneralCoder.decodeBase64(data.getBytes());
		ByteArrayInputStream bstr = new ByteArrayInputStream(b);
		try {
			ObjectInputStream ostr = new ObjectInputStream(bstr);
			return ostr.readObject();
		} finally {
			bstr.close();
		}
	}
	
	public abstract void addLayer(DefaultSynchroLayer layer, byte[] myBytes) throws IOException;
	public abstract void removeLayer(DefaultSynchroLayer layer) throws IOException;
	public abstract File getFileFromLayer(DefaultSynchroLayer layer) throws IOException;
	public abstract void replaceXMLbyNew(Array newLayers, File xmlFile) throws IOException;
	public abstract File getFileFromProjectName(String projectName)throws IOException;

}
