/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.server.listeners;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;

import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.utils.FileLayerUtilities;
import org.openmis.tellus.synchro.common.xml.XMLGenerator;
import org.openmis.tellus.synchro.common.xml.XMLProjectFileGenerator;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.UniqueIdGenerator;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class SynchroLayerTargetProcessing extends SynchroLayerBaseAnalyserTarget {


	public SynchroLayerTargetProcessing() {
		super();
	}


	public void addLayer(DefaultSynchroLayer layer, byte[] myBytes) throws IOException{
		try{
			String completeFilePath = null;
			if(layer != null){
				completeFilePath = getFilePath(layer);
				
				/*if(layer.getServerPath() == null || layer.getServerPath().length() == 0){
					File directory = new File(PropertiesManager.getManager().getProperty("projects.server.folder"));
					completeFilePath = directory+File.separator+projectName+File.separator+layer.getFileName();
				}else{
					completeFilePath = layer.getServerPath()+File.separator+layer.getFileName();
				}*/			
			}else{	
				File directory = new File(PropertiesManager.getManager().getProperty("tempory.server.layers.added"));
				if(!directory.exists()){
					directory.mkdir();
				}
				completeFilePath = directory+File.separator+UniqueIdGenerator.getManager().getNewStringID();
				LogManager.traceDebug(0, "SynchroLayerTargetProcessing addLayer(), layer sended was NULL, File corresponding added in the tempory folder with a tempory name : "+completeFilePath);
			}
			FileLayerUtilities.saveInFile(completeFilePath, new ByteArrayInputStream(myBytes));
			
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerTargetProcessing, addLayer() : "+ex);
			throw new IOException(ex.getMessage());
		}
 
	}
	/**
	 * @return File : return the ServerXMLProjectFile to use during the synchro
	 */
	public File getFileFromProjectName(String projectName)throws IOException{
		try{
			String projectsDirectory = PropertiesManager.getManager().getProperty("xml.server.folder.projects");
			if(projectsDirectory != null){
				File file = new File (projectsDirectory+File.separator+projectName+".xml");
				if(!file.exists())
					file.createNewFile();
				
				return file;
			}else{
				throw new IOException("Property xml.server.folder.projects not define !!!");
			}
		
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerTargetProcessing, getFileFromProjectName() : "+ex);
			throw new IOException();
		}
	}


	public void removeLayer(DefaultSynchroLayer layer) throws IOException{
		try{

			boolean result = FileLayerUtilities.deleteFile(getFilePath(layer));
			if(!result)
				LogManager.traceError(0, "Client Synchro Layers : ERROR IN DELETING FILE : NOT FIND : "+layer.getFileName());
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerTargetProcessing, removeLayer() : "+ex);
			//throw new IOException();
		}
    	
	}
	
	private String getFilePath(DefaultSynchroLayer layer){
		File directory = new File(PropertiesManager.getManager().getProperty("projects.server.folder"));
		return directory+File.separator+layer.getProjectName()+File.separator+layer.getFileName();
	}


	public File getFileFromLayer(DefaultSynchroLayer layer) throws IOException{
		File file = new File(getFilePath(layer));
		if(!file.exists()){
			LogManager.traceError(0, "Error during execution of service SynchroLayerTargetProcessing, getFileFromLayer() FILE NOT FOUND !! pathFile : "+file.getAbsolutePath());
			throw new IOException();
		}
		return file;
	}
	
	public void replaceXMLbyNew(Array newLayers, File xmlFile)throws IOException{
		try{			
			XMLGenerator generation = new XMLProjectFileGenerator();
			generation.generateNewXMLProjectFile(newLayers);		
			FileLayerUtilities.saveInFile(xmlFile.getAbsolutePath(), generation.getInputStreamXML());		
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerTargetProcessing, replaceXMLbyNew() : "+ex);
			throw new IOException();
		}
	}


}
