/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.terminal.listeners;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.data.SynchroProject;
import org.openmis.tellus.synchro.common.xml.ParseXMLProject;
import org.openmis.tellus.synchro.terminal.conflict.resolver.SynchroLayerConflicResolverServerWin;
import org.openmis.tellus.synchro.terminal.project.ProjectProfilManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.algo.replication.ReplicationReturnData;
import org.openmobileis.synchro.algo.replication.ReplicationSAOEvent;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.xmlpull.v1.XmlPullParserException;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public abstract class SynchroLayerAnalyserListener extends SynchroLayerBaseAnalyserListener {

	public SynchroLayerAnalyserListener(SynchroProject project) {
		super(project);
	}


	public SynchroAtomicObject[] getListAOTerminalForReplication() throws Exception{
		File terminalXML = null;
		try{
			terminalXML = ProjectProfilManager.getManager().getTerminalXMLProjectFile(this.getSynchroProject());
			if(!terminalXML.exists()){		
				terminalXML.createNewFile();
			}
				
			ParseXMLProject parser = new ParseXMLProject(new FileReader(terminalXML));
			
			return parser.parse();
			
		}catch(IOException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, getListAOTerminalForReplication() IOException, Directory XML file project Terminal NOT FOUND !! : "+terminalXML.getAbsolutePath()+" : "+ex);
			throw new IOException("Directory XML file project Terminal NOT FOUND");
		}
		catch(XmlPullParserException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, getListAOTerminalForReplication() XmlPullParserException ERROR IN PARSING XML PROJECT FILE TERMINAL : "+terminalXML.getAbsolutePath()+" : "+ex);
			throw new XmlPullParserException("ERROR IN PARSING XML PROJECT FILE TERMINAL");
		}
		catch(Exception ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, getListAOTerminalForReplication() Exception ERROR !! : "+ex);
			throw new Exception("Exception ERROR");
		}
	}


	public SynchroAtomicObject[] getListAOServerForReplication(String xmlFilename) throws Exception{
		File file = null;
		try{	
			file = ProjectProfilManager.getManager().getServerXMLProjectFile(xmlFilename, this.getSynchroProject());
			ParseXMLProject parser = new ParseXMLProject(new FileReader(file));
			SynchroAtomicObject[] ret = parser.parse();
			file.delete();
			
			return ret;
			
		}catch(IOException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, getListAOServerForReplication() IOException, Tempory Directory XML file project Server NOT FOUND !! : "+file.getAbsolutePath()+" : "+ex);
			throw new IOException("Tempory Directory XML file project Server NOT FOUND");
		}
		catch(XmlPullParserException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, getListAOServerForReplication() ERROR IN PARSING XML PROJECT FILE SERVER : "+file.getAbsolutePath()+" : "+ex);
			throw new XmlPullParserException("ERROR IN PARSING XML PROJECT FILE SERVER");
		}
		catch(Exception ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, getListAOServerForReplication() Exception ERROR !! : "+ex);
			throw new Exception("Exception ERROR");
		}
	}


	public SynchroConflicResolver getConfictResolver() throws Exception{
		//------------- with this confict resolver SERVER WIN
		return new SynchroLayerConflicResolverServerWin();
	}
	
	public Array generateNewArrayLayersForXML(SynchroAtomicObject ListAOServer[], ReplicationSAOEvent ServerEvents[]) throws Exception{
		try{
			LayersGeneratorForXML generator = new LayersGeneratorForXML();		
			return generator.generateNewXMLProjectFile(ListAOServer, ServerEvents);
			
		}catch(Exception ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, generateNewXMLFile() ERROR DURING GENERATION OF NEW XML PROJECT FILE: "+ex);
			throw new Exception("ERROR DURING GENERATION OF NEW XML PROJECT FILE");
		}
		
	}
	
	public void processReplicationReturnData(ReplicationReturnData returnData) throws IOException{
		try{
			//----------------- On genere les modifications pour le terminal
			ReplicationSAOEvent eventsClient[] = returnData.getClientSideModificationList();
			for(int i=0;i<eventsClient.length;i++){
				ReplicationSAOEvent event = eventsClient[i];
				DefaultSynchroLayer layer = (DefaultSynchroLayer)event.getAtomicObject();
				//------------ Soit operation de DELETE sur terminal
				if(event.getTypeModif() == ReplicationSAOEvent.DELETE){
					this.removeLayer(layer);
			    //----------- Soit operation de GET a demander sur le server
				}else{
					this.addGETLayer(layer);
				}
			}
			//----------------- On genere les modifications pour le server
			
			ReplicationSAOEvent eventsServer[] = returnData.getServerSideModificationList();
			for(int i=0;i<eventsServer.length;i++){
				ReplicationSAOEvent event = eventsServer[i];
				DefaultSynchroLayer layer = (DefaultSynchroLayer)event.getAtomicObject();
				//----------- Soit une demande de DELETE  faire sur le server
				if(event.getTypeModif() == ReplicationSAOEvent.DELETE){
					this.addDELETELayer(layer);
				}
				//----------- Soit envoi des fichiers correspondants au ADD ou MODIFY layers
				else{
					this.addADDLayer(layer, this.getBytesFromLayer(layer));
				}
			}
		}catch(IOException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerAnalyser, processReplicationReturnData() : "+ex);
		}
	}

	public abstract void removeLayer(DefaultSynchroLayer layer) throws IOException;
	public abstract byte[] getBytesFromLayer(DefaultSynchroLayer layer) throws IOException;

}
