/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.terminal.listeners;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.ObjectOutputStream;

import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.data.SendSynchroLayer;
import org.openmis.tellus.synchro.common.data.SynchroProject;
import org.openmis.tellus.synchro.terminal.project.ProjectProfilManager;
import org.openmis.tellus.synchro.terminal.replication.SynchroLayerArrayReplication;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.PersistentPropertiesManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.algo.replication.ReplicationReturnData;
import org.openmobileis.synchro.algo.replication.ReplicationSAOEvent;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.journal.JournalEntry;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.journal.SimpleLogRenderer;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.DefaultOpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.RequestCommand;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;



/**
 * @author Manuel Gomez-Rosado
 *
 */
public abstract class SynchroLayerBaseAnalyserListener extends DefaultOpenMSPSyncListener {

	private boolean syncGoOn = false, firstStep=false, secondStep=false;
	private Status resultStatus = null;
	protected String synchroPath;
	protected SynchroProject project;
	
	public SynchroLayerBaseAnalyserListener(SynchroProject project){	
		super();
		this.project = project;
		PersistentPropertiesManager.getManager().getProperty(this.getSyncName(), "synchroStatus");
		JournalManager.getManager().registerJournalLogRenderer(new SimpleLogRenderer(this.getSyncName(), project.getUniqueProjectName()));
	}
	
	public SynchroProject getSynchroProject(){
		return this.project;
	}
	
	public void startSync(Credential cred, SynchroDescriptor synchrodescriptor)throws OpenMSPException {
		super.startSync(cred, synchrodescriptor);
		synchroPath = synchrodescriptor.getSynchroConduit().getIntallPath();
		syncGoOn = true;
		firstStep=true;
	}
	
	public void sendData(Message message) throws OpenMSPException {
		if (resultStatus != null) {
			message.add(new ContainerMessage(resultStatus));
			resultStatus = null;
		}
		if (!syncGoOn) { // synchronize data only for the first call.
			return;
		}
		
		//-------------------------- Request for XML file server and increment syncNumber on server
		try {
			if(firstStep){
				
				// ----------------- Premire ETAPE : Add GET COMMAND to request the XML Projet File Server
				Command getCommand = new Command(Element.GET, this.getSyncName(),"XMLAnalyserTarget");
				getCommand.setMetaInformation("XMLREQUEST");
				long ns = NumSyncManagerDB.getManager().getSyncNumberForService(this.getSyncName());
				
				getCommand.setSourceSessionID(ns);
				DefaultOpenMSPSyncListener.addCredentialToCommand(null, getCommand);
				ContainerMessage getContainer = new ContainerMessage(getCommand);
				
				//Get the project to use during the synchronisation
				SynchroProject project = this.getSynchroProject();
				
				Item newItem = new DataItem(
						Element.ITEM
						,"synchroProject"
						,new String (GeneralCoder.encodeBase64(serializeObject(project)))
						,new String(this.getSyncName())
						,new String(this.getTargetName())
						);
				getContainer.add(newItem);
			    message.add(getContainer);// add before sync container because GET command must be proceed before the SYNC command.
			    firstStep=false;
			    
			}else{
				 secondStep=true;
			}
		      
		   //--------------------- Deuxime ETAPE : Add commands to GET,ADD,DELETE layers from server
		    if(secondStep){
				//Get the project in use during the synchronisation
				SynchroProject project = this.getSynchroProject();
		    	SessionContext context = SessionContextManager.getManager().getSessionContext();
  	
		    	//---------------- GET actions
		    	Command getCommand = new Command(Element.GET, this.getSyncName(), this.getTargetName());
			    getCommand.setMetaInformation("LAYERSREQUEST");
			    long ns = NumSyncManagerDB.getManager().getSyncNumberForService(this.getSyncName());
			    
			    getCommand.setSourceSessionID(ns);
			    DefaultOpenMSPSyncListener.addCredentialToCommand(null, getCommand);	      
			    ContainerMessage getContainer = new ContainerMessage(getCommand);
			    

			    Array gettedLayers = (Array)context.getAttribute(this.getNamedSessionGETLayer());
			    if(gettedLayers != null){
				    for(int i=0;i<gettedLayers.size();i++){
				    	Item newItem = new DataItem(
								Element.ITEM
								,new String(GeneralCoder.encodeBase64(serializeObject(project)))
								,new String(GeneralCoder.encodeBase64(serializeObject(gettedLayers.get(i))))
								,new String(this.getSyncName())
								,new String(this.getTargetName())
								);
						getContainer.add(newItem);    	
				    }
				    message.add(getContainer);
			    }
			    
		    	//---------------- SYNC command
			    Command syncCommand = new Command(Element.SYNC, this.getSyncName(), this.getTargetName());
			    syncCommand.setSourceSessionID(ns);
			    DefaultOpenMSPSyncListener.addCredentialToCommand(null, syncCommand);   
			    ContainerMessage syncContainer = new ContainerMessage(syncCommand);
		    		    
			    
			    RequestCommand newCommand = null;
				ContainerMessage newContainer = null;
				newCommand = new RequestCommand(Element.DELETE);
	            newCommand.setMetaInformation("LAYERSDELETE");
	            newContainer = new ContainerMessage(newCommand);
			    
	            Array deletedLayers = (Array)context.getAttribute(this.getNamedSessionDELETELayer());
			    if(deletedLayers != null){
				    for(int i=0;i<deletedLayers.size();i++){
				    	Item newItem = new DataItem(
								Element.ITEM
								,"DeleteLayers"
								,new String(GeneralCoder.encodeBase64(serializeObject(deletedLayers.get(i))))
								,new String(this.getSyncName())
								,new String(this.getTargetName())
								);
				    	newContainer.add(newItem);    	
				    }
				    syncContainer.add(newContainer);
			    }	    
			    
		    	//---------------- ADD/REPLACE actions
			    newCommand = new RequestCommand(Element.ADD);
	            newCommand.setMetaInformation("LAYERSADD");
	            newContainer = new ContainerMessage(newCommand);
	            
	            Array addedLayers = (Array)context.getAttribute(this.getNamedSessionADDLayer());
			    if(addedLayers != null){
				    for(int i=0;i<addedLayers.size();i++){
				    	//SendSynchroLayer sendlayer  = (SendSynchroLayer)addedLayers.get(i);
				    	Item newItem = new DataItem(
							Element.ITEM
							,"AddLayers"
							,new String(GeneralCoder.encodeBase64(serializeObject(addedLayers.get(i))))
							,new String(this.getSyncName())
							,new String(this.getTargetName())
							);
					    newContainer.add(newItem); 
		            }
				    syncContainer.add(newContainer);
			    }
			    
			    //---------------- REPLACE XML Project File server
			    newCommand = new RequestCommand(Element.REPLACE);
	            newCommand.setMetaInformation("XMLADD");		    
	            newContainer = new ContainerMessage(newCommand);
	            
	            //-----------------------
	            Array array = (Array)context.getAttribute(this.getNamedSessionNewListLayersForXML());
	            //-----------------------
	            if(array != null){
			    	Item newItem = new DataItem(
							Element.ITEM
							,new String(GeneralCoder.encodeBase64(serializeObject(project)))
							,new String(GeneralCoder.encodeBase64(serializeObject(array)))
							,new String(this.getSyncName())
							,new String(this.getTargetName())
							);
			    	newContainer.add(newItem); 
	            }

				syncContainer.add(newContainer);
			    			    
				
				//---------- end Synchronization
			    message.add(syncContainer);
				secondStep=false;
				syncGoOn = false;
				
				//------ To reinit the differents sessions
				this.reinitSessions();
				
		    }
    
		    
		} catch (IOException ex) {
			this.setSynchroStatus(Status.STATUS_DEVICE_FULL);
			LogManager.traceError(0, "Error during execution of service SynchroLayerBaseAnalyserListener, sendData() : Type IOException "+ex);
		}catch (OpenMSPException ex) {
			this.setSynchroStatus(Status.STATUS_FAILED);
			LogManager.traceError(0, "Error during execution of service SynchroLayerBaseAnalyserListener, sendData() : Type OpenMSPException "+ex);
		}catch (Exception ex) {
			this.setSynchroStatus(Status.STATUS_FAILED);
			LogManager.traceError(0, "Error during execution of service SynchroLayerBaseAnalyserListener, sendData() : Type Exception "+ex);
		}
			
	}

	
	public String getSyncName() {
		if(project != null)
			return "XMLList."+project.getUniqueProjectName();
		else
			return null;
	}
	//must be the same short name than in the getSyncName() method
	public static String getShortSyncName(){
		return "XMLList.";
	}

	public String getTargetName() {
		return "XMLAnalyserTarget";
	}
	
	public void receiveResultCommand(ContainerMessage resultContainer,ContainerMessage initialCommand) throws OpenMSPException {
		Result result = (Result) resultContainer.getElement();
		
		while (resultContainer.hasMoreMessage()) {
			
			DataItem item = (DataItem) resultContainer.nextMessage().getElement();
			try {
				//-------------------- Get List AO terminal and server and compare them
				if(item.getMetaInformation().equals("XML")){
					
					//---------------------- Sync Number Management -----------------------------------------------
					long newSyncNumber = Long.parseLong(result.getMetaInformation());
					long currentSyncNumber = NumSyncManagerDB.getManager().getSyncNumberForService(this.getSyncName());
					//---------------------------------------------------------------------------------------------
					SynchroAtomicObject AOlistServer[] = this.getListAOServerForReplication(item.getData());
					SynchroAtomicObject AOlistTerminal[] = this.getListAOTerminalForReplication();
					
					SynchroLayerArrayReplication replication = new SynchroLayerArrayReplication(this.getConfictResolver());
					ReplicationReturnData returnData = replication.replicates(AOlistTerminal, AOlistServer, currentSyncNumber);
					
					//----------------- Generate modifications
					this.processReplicationReturnData(returnData);
					SessionContext context = SessionContextManager.getManager().getSessionContext();
				    context.setAttribute(this.getNamedSessionNewSyncNumber(), Long.toString(newSyncNumber));
					
					//----------------- New XML File Generation
					Array array = this.generateNewArrayLayersForXML(AOlistServer, returnData.getServerSideModificationList());
					context.setAttribute(this.getNamedSessionNewListLayersForXML(), array);
					
				}			
				// ------------------------------------------------------------
				resultStatus = new Status(result.getCmdId(), Status.STATUS_OK);
				
			}catch(SynchroException ex){
				LogManager.traceError(0, "Error during execution of service SynchroLayerBaseAnalyserListener, receiveResultCommand() : Type SynchroException "+ex);
				resultStatus = new Status(result.getCmdId(),Status.STATUS_FAILED);
				this.setSynchroStatus(Status.STATUS_FAILED);
			}catch(Exception ex){
				LogManager.traceError(0, "Error during execution of service SynchroLayerBaseAnalyserListener, receiveResultCommand() : Type Exception "+ex);
				resultStatus = new Status(result.getCmdId(),Status.STATUS_FAILED);
				this.setSynchroStatus(Status.STATUS_FAILED);
			}
		}

	}
	
	
	public void endSync()  throws OpenMSPException {
		try{		
			if(this.getSynchroStatus() == Status.STATUS_OK){	//on ne sauvegarde les modifs que si tte la synchro est OK
				SessionContext context = SessionContextManager.getManager().getSessionContext();
				String newSyncNumberStr = (String) context.getAttribute(this.getNamedSessionNewSyncNumber());
				Array array = (Array)context.getAttribute(this.getNamedSessionNewListLayersForXML());
				
				this.replaceXMLbyNew(array, ProjectProfilManager.getManager().getTerminalXMLProjectFile(this.getSynchroProject()));
				NumSyncManagerDB.getManager().saveSyncNumberForService(Long.parseLong(newSyncNumberStr), this.getSyncName());
				
				context.removeAttribute(this.getNamedSessionNewSyncNumber());
				context.removeAttribute(this.getNamedSessionNewListLayersForXML());
			}else{
				this.setSynchroStatus(Status.STATUS_FAILED);
				SessionContext context = SessionContextManager.getManager().getSessionContext();
				if(context.getAttribute(this.getNamedSessionNewSyncNumber()) != null)
					context.removeAttribute(this.getNamedSessionNewSyncNumber());
				if(context.getAttribute(this.getNamedSessionNewListLayersForXML()) != null)
					context.removeAttribute(this.getNamedSessionNewListLayersForXML());
			}
		}catch(Exception ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerBaseAnalyser, endSync() ERROR IN SAVING NEW XML PROJECT IN FILE : "+PropertiesManager.getManager().getProperty("xml.terminal.file.project")+" : "+ex);
			this.setSynchroStatus(Status.STATUS_FAILED);
		}
		
		super.endSync();
	}

	public void receiveStatusCommand(Status statusCommande, ContainerMessage initialCommand) throws OpenMSPException {	
		if(statusCommande.getStatus() != Status.STATUS_OK){
			this.notifySynchroFailure(statusCommande);
		}
	}
	
	public void notifySynchroFailure(Status status) {
	    this.setSynchroStatus(Status.STATUS_FAILED);    
	    if(status.getStatus() == Status.STATUS_WRONG_AUTHENTIFICATION){
	    	JournalManager.getManager().saveJournalEntry(this.getSyncName(),JournalEntry.ERROR_SYNC,"Erreur d'authentification", Status.STATUS_WRONG_AUTHENTIFICATION);	
	    }else if(status.getStatus() == Status.STATUS_HOST_UNKNOW){
	    	JournalManager.getManager().saveJournalEntry(this.getSyncName(),JournalEntry.ERROR_SYNC,"Erreur, Serveur Inconnu", Status.STATUS_HOST_UNKNOW);	
	    }else if(status.getStatus() == Status.STATUS_DEVICE_FULL){
	    	JournalManager.getManager().saveJournalEntry(this.getSyncName(),JournalEntry.ERROR_SYNC,"Erreur d'criture, device full", Status.STATUS_DEVICE_FULL);	
	    }else{
	    	JournalManager.getManager().saveJournalEntry(this.getSyncName(),JournalEntry.ERROR_SYNC,"Echec", Status.STATUS_FAILED);
	    } 
	}


	protected void addGETLayer(DefaultSynchroLayer layer){
		SessionContext context = SessionContextManager.getManager().getSessionContext();
		Array array = (Array)context.getAttribute(this.getNamedSessionGETLayer());
		if(array == null)
			array = new Array();
		
		array.add(layer);
		context.setAttribute(this.getNamedSessionGETLayer(), array);
	}
	
	protected void addDELETELayer(DefaultSynchroLayer layer){
		SessionContext context = SessionContextManager.getManager().getSessionContext();
		Array array = (Array)context.getAttribute(this.getNamedSessionDELETELayer());
		if(array == null)
			array = new Array();
		
		array.add(layer);
		context.setAttribute(this.getNamedSessionDELETELayer(), array);
	}
	
	protected void addADDLayer(DefaultSynchroLayer layer, byte[] myBytes){
		SendSynchroLayer sendLayer = new SendSynchroLayer(layer, myBytes, getSynchroProject().getUniqueProjectName());	
		SessionContext context = SessionContextManager.getManager().getSessionContext();
		Array array = (Array)context.getAttribute(this.getNamedSessionADDLayer());
		if(array == null)
			array = new Array();
		
		array.add(sendLayer);
		context.setAttribute(this.getNamedSessionADDLayer(), array);
	}
	
	private void reinitSessions(){
		SessionContext context = SessionContextManager.getManager().getSessionContext();
		
		if(context.getAttribute(this.getNamedSessionDELETELayer()) != null)
			context.removeAttribute(this.getNamedSessionDELETELayer());
		
		if(context.getAttribute(this.getNamedSessionADDLayer()) != null)
			context.removeAttribute(this.getNamedSessionADDLayer());
		
		if(context.getAttribute(this.getNamedSessionGETLayer()) != null)
			context.removeAttribute(this.getNamedSessionGETLayer());
	}
	
	private String getNamedSessionNewSyncNumber(){
		return "NewSyncNumber_Synchro."+project.getUniqueProjectName();
	}
	private String getNamedSessionNewListLayersForXML(){
		return "NewListLayersForXML_Synchro."+project.getUniqueProjectName();
	}
	private String getNamedSessionGETLayer(){
		return "GETLayer_Synchro."+project.getUniqueProjectName();
	}
	private String getNamedSessionDELETELayer(){
		return "DELETELayer_Synchro."+project.getUniqueProjectName();
	}
	private String getNamedSessionADDLayer(){
		return "ADDLayer_Synchro."+project.getUniqueProjectName();
	}
		
	private byte[] serializeObject(Object obj) throws IOException {
		ByteArrayOutputStream bstr = new ByteArrayOutputStream();
		try {
			ObjectOutputStream ostr = new ObjectOutputStream(bstr);
			ostr.writeObject(obj);
		} finally {
			bstr.flush();
			bstr.close();
		}
		return bstr.toByteArray();

	}

	public abstract SynchroAtomicObject[] getListAOTerminalForReplication() throws Exception;
	public abstract SynchroAtomicObject[] getListAOServerForReplication(String xmlFilename) throws Exception;
	public abstract SynchroConflicResolver getConfictResolver() throws Exception;
	public abstract Array generateNewArrayLayersForXML(SynchroAtomicObject ListAOServer[], ReplicationSAOEvent ServerEvents[]) throws Exception;
	public abstract void replaceXMLbyNew(Array layers, File xmlFile) throws IOException;	
	public abstract void processReplicationReturnData(ReplicationReturnData returnData) throws IOException;

}
