/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.terminal.listeners;

import java.io.File;
import java.io.IOException;

import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.data.SynchroProject;
import org.openmis.tellus.synchro.common.utils.FileLayerUtilities;
import org.openmis.tellus.synchro.common.xml.XMLGenerator;
import org.openmis.tellus.synchro.common.xml.XMLProjectFileGenerator;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class SynchroLayerListenerProcessing extends SynchroLayerAnalyserListener {


	public SynchroLayerListenerProcessing(SynchroProject project) {
		super(project);
	}
	
	
	public void replaceXMLbyNew(Array newLayers, File xmlFile) throws IOException{
		try{
			XMLGenerator generation = new XMLProjectFileGenerator();
			generation.generateNewXMLProjectFile(newLayers);			
			FileLayerUtilities.saveInFile(xmlFile.getAbsolutePath(), generation.getInputStreamXML());
			
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerListenerProcessing, replaceXMLbyNew() ERROR IN SAVING NEW XML PROJECT IN FILE : "+PropertiesManager.getManager().getProperty("xml.terminal.file.project")+" : "+ex);
			throw new IOException("ERROR IN SAVING NEW XML PROJECT IN FILE");		
		}
	}
	
	public void removeLayer(DefaultSynchroLayer layer) throws IOException{
		try{
			String projectName = this.getSynchroProject().getUniqueProjectName();
			String inPathDelete = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath")+File.separator+projectName+File.separator+layer.getFileName();
			
			boolean result = FileLayerUtilities.deleteFile(inPathDelete);
			if(!result)
				LogManager.traceError(0, "Server Synchro Layers : ERROR IN DELETING FILE : NOT FIND : "+layer.getFileName());
		}catch(Throwable ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerListenerProcessing, removeLayer() ERROR IN DELETING FILE : "+layer.getFileName()+" : "+ex);
			//throw new IOException("ERROR IN DELETING FILE");	
		}
	}
	
	public byte[] getBytesFromLayer(DefaultSynchroLayer layer) throws IOException{
		try{
			String projectName = this.getSynchroProject().getUniqueProjectName();
			String inPathAdd = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath")+File.separator+projectName+File.separator+layer.getFileName();
			
			return FileLayerUtilities.readFile(inPathAdd);
		}catch(IOException ex){
			LogManager.traceError(0, "Error during execution of service SynchroLayerListenerProcessing, getStreamFromLayer() ERROR IN READING FILE : "+layer.getFileName()+" : "+ex);
			throw new IOException("ERROR IN READING FILE");	
		}
	}

}
