/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.terminal.project;

import java.io.File;

import org.openmis.tellus.synchro.common.data.SynchroGeograhicProfil;
import org.openmis.tellus.synchro.common.data.SynchroProject;
import org.openmis.tellus.synchro.terminal.listeners.SynchroLayerBaseAnalyserListener;
import org.openmis.tellus.synchro.terminal.listeners.SynchroLayerListenerProcessing;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.synchro.client.SynchroManager;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class ProjectProfilManager {


	private static ProjectProfilManager manager;
	private SynchroGeograhicProfil profilGeo = null;
	
	private  ProjectProfilManager() {
		super();
	}

	/**
	 * 
	 * @return ProjectProfilManager. The ProjectProfilManager is ONLY use on Terminal
	 */
	
	public static ProjectProfilManager getManager () {
		if (manager == null)	{
			synchronized(ProjectProfilManager.class)	{
				if (manager == null)	{
					manager = new ProjectProfilManager();
					ApplicationContextManager.getManager().addManager(manager);
				}
			}
		}
		return manager;
	}
   
	public static void removeManager()	{
		manager = null;
	}
	
	public SynchroGeograhicProfil getProfilGeographic() {
		return profilGeo;
	}
	
	/**
	 * 
	 * @param profilGeo : add the SynchroGeograhicProfil to the synchro to regsiter All the listener corresponding
	 */	

	public void addSynchroGeograhicProfil(SynchroGeograhicProfil profilGeo){
		this.profilGeo = profilGeo;
		Array listSynchroProject = profilGeo.getAllSynchroProject();
		this.removeAllOldListenersByClassType();
		if(listSynchroProject != null){
			for(int i=0;i<listSynchroProject.size();i++){
				SynchroProject project = (SynchroProject)listSynchroProject.get(i);
				String name = SynchroLayerListenerProcessing.getShortSyncName()+project.getUniqueProjectName();
				OpenMSPSyncListener listener = OpenMSPSynchroManager.getManager().getListenerByName(name);
				//Array array = OpenMSPSynchroManager.getManager().getAllListeners();
				if(listener == null)	//if not exist create it
					OpenMSPSynchroManager.getManager().addListener(new SynchroLayerListenerProcessing(project), null); 
			}
	//		SynchroManager.getManager().registerSynchroProcessor(OpenMSPSynchroManager.getManager());   
		}
	}
	
	private void removeAllOldListenersByClassType(){
		Array array = OpenMSPSynchroManager.getManager().getAllListeners();
		if(array != null){
			for(int i=0;i<array.size();i++){
				OpenMSPSyncListener listener = (OpenMSPSyncListener)array.get(i);
				if(listener instanceof SynchroLayerBaseAnalyserListener){
					OpenMSPSynchroManager.getManager().removeListenerByName(listener.getSyncName());
					JournalManager.getManager().clearJournalLogRendererForService(listener.getSyncName());
				}
			}
		}
	}

	
	/**
	 * @param project : Synchroproject of the listener
	 * @return the xmlTerminalFile From The Synchroproject.
	 */
	public File getTerminalXMLProjectFile(SynchroProject project){		
		String projectName = project.getUniqueProjectName();
		String projectsDirectory = PropertiesManager.getManager().getProperty("xml.terminal.folder.projects");
		File file = new File(projectsDirectory+File.separator+projectName+".xml");
		return file;
	}
	
	
	/**
	 * 
	 * @param xmlFilename : name of the file recuperate by the synchronisation
	 *  @param project : Synchroproject of the listener
	 * @return the xmlServerFile pass by the synchronisation to do the replication
	 */
	public File getServerXMLProjectFile(String xmlFilename, SynchroProject project){
		String projectName = project.getUniqueProjectName();
		String fileAddDirectory = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath");
		return new File(fileAddDirectory+File.separator+projectName+File.separator+xmlFilename);
	}
}
