/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.terminal.replication;



import org.openmobileis.common.util.collection.Orderable;
import org.openmobileis.common.util.collection.Sort;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.synchro.algo.replication.ReplicationReturnData;
import org.openmobileis.synchro.algo.replication.ReplicationSAOEvent;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;

/**
 * @author Manuel Gomez-Rosado
 *
 */
public class SynchroLayerArrayReplication implements Orderable {

	protected SynchroConflicResolver conflicResolver;
	
	public SynchroLayerArrayReplication(SynchroConflicResolver conflicResolver) {
		this.conflicResolver = conflicResolver;
	}
	
	
	public ReplicationReturnData replicates(SynchroAtomicObject[] listAOClient, SynchroAtomicObject[] listAOServ, long currentSyncNumber)throws SynchroException {
	    ReplicationReturnData SAOEventList = new ReplicationReturnData();
			try {
				if (listAOServ.length >1)	{
					Sort.shellsort(listAOServ, this);
				}
				if (listAOClient.length >1)	{
					Sort.shellsort(listAOClient, this);
				}
				
				int comptServ = 0;
				int comptClient = 0;
				
				while (comptServ < listAOServ.length && comptClient < listAOClient.length) {
					
					SynchroAtomicObject serverAtomicObject = listAOServ[comptServ];
					SynchroAtomicObject clientAtomicObject = listAOClient[comptClient];
					
					String serverUID = serverAtomicObject.getUID();
					String clientUID = clientAtomicObject.getUID();
				 
					//----- Si object prsent des deux cots server et client
					if (serverUID.equals(clientUID)){
				    
						//detect if there is really a modification
						//System.out.println("CurrentSyncNumber : "+currentSyncNumber);
						
						//modif on both side
						if(serverAtomicObject.getModifSyncNumber() > currentSyncNumber && clientAtomicObject.getModifSyncNumber() == currentSyncNumber){					
							serverAtomicObject.setModificationType(SynchroAtomicObject.REPLACE);
							
							ReplicationSAOEvent eventAO = conflicResolver.resolveReplaceandModified(serverAtomicObject);
							SAOEventList.addReplicationSAOEvent(eventAO);
							
						}// modif on server side
						else if(serverAtomicObject.getModifSyncNumber() > currentSyncNumber){	
							serverAtomicObject.setModificationType(SynchroAtomicObject.REPLACE);
							
							ReplicationSAOEvent eventAO = new ReplicationSAOEvent(serverUID);
							eventAO.setUpdateSide(ReplicationSAOEvent.CLIENT_SIDE);
							eventAO.setTypeModif(serverAtomicObject.getModificationType());
							eventAO.setAobject(serverAtomicObject);
							SAOEventList.addReplicationSAOEvent(eventAO);

							
						}//modif on client side
						else if(clientAtomicObject.getModifSyncNumber() == currentSyncNumber){
							clientAtomicObject.setModificationType(SynchroAtomicObject.REPLACE);
							
							ReplicationSAOEvent eventAO = new ReplicationSAOEvent(clientUID);
							eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
							eventAO.setTypeModif(clientAtomicObject.getModificationType());
							eventAO.setAobject(clientAtomicObject);
							SAOEventList.addReplicationSAOEvent(eventAO);	
						}
						
						comptServ++;
						comptClient++;
						
						//-------- si l'ID server est superieur  l'ID client
					}else if (serverUID.compareTo(clientUID) > 0) {
					    
						clientAtomicObject = (SynchroAtomicObject)listAOClient[comptClient];
						
						if(clientAtomicObject.getCreationDate() < currentSyncNumber){
							
							if(clientAtomicObject.getModifSyncNumber() == currentSyncNumber){
								//----- L'object a aussi t modifier sur le terminal donc conflit
							
								ReplicationSAOEvent eventAO = conflicResolver.resolveDeleteandModified(clientAtomicObject);
								SAOEventList.addReplicationSAOEvent(eventAO);							
								
							}else{
								// object supprim sur le serveur  rpercuter cot client
								clientAtomicObject.setModificationType(SynchroAtomicObject.DELETE);
								
								ReplicationSAOEvent eventAO = new ReplicationSAOEvent(clientAtomicObject.getUID());
								eventAO.setUpdateSide(ReplicationSAOEvent.CLIENT_SIDE);
								eventAO.setTypeModif(clientAtomicObject.getModificationType());
								eventAO.setAobject(clientAtomicObject);
								SAOEventList.addReplicationSAOEvent(eventAO);
							}
							
						}else{
							//---- Sinon il s'agit d'un ajout d'object cot client  rpercut cot server
							clientAtomicObject.setModificationType(SynchroAtomicObject.ADD);
							
							ReplicationSAOEvent eventAO = new ReplicationSAOEvent(clientAtomicObject.getUID());
							eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
							eventAO.setTypeModif(clientAtomicObject.getModificationType());
							eventAO.setAobject(clientAtomicObject);
							SAOEventList.addReplicationSAOEvent(eventAO);
						}
						
						comptClient++;
						
						//-------- si l'ID client est suprieur  l'ID server
					} else {
				    
						serverAtomicObject = (SynchroAtomicObject)listAOServ[comptServ];
						
						if(serverAtomicObject.getCreationDate() < currentSyncNumber){
							
							if(serverAtomicObject.getModifSyncNumber() > currentSyncNumber){
								//----- L'object a aussi t modifier sur le server donc conflit
								ReplicationSAOEvent eventAO = conflicResolver.resolveReplaceandDeleted(serverAtomicObject);
								SAOEventList.addReplicationSAOEvent(eventAO);
								
							}else{
								//object supprim sur le client  rpercuter cot server
								serverAtomicObject.setModificationType(SynchroAtomicObject.DELETE);
								
								ReplicationSAOEvent eventAO = new ReplicationSAOEvent(serverAtomicObject.getUID());
								eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
								eventAO.setTypeModif(serverAtomicObject.getModificationType());
								eventAO.setAobject(serverAtomicObject);
								SAOEventList.addReplicationSAOEvent(eventAO);
							}
							
						}else{
							//---- sinon il s'agit d'un ajout d'object cot server  rpercuter cot client
							serverAtomicObject.setModificationType(SynchroAtomicObject.ADD);
							
							ReplicationSAOEvent eventAO = new ReplicationSAOEvent(serverAtomicObject.getUID());
							eventAO.setUpdateSide(ReplicationSAOEvent.CLIENT_SIDE);
							eventAO.setTypeModif(serverAtomicObject.getModificationType());
							eventAO.setAobject(serverAtomicObject);
							SAOEventList.addReplicationSAOEvent(eventAO);
						}
						
						comptServ++;
					}

				}
				
				//---------------------- On check les listes savoir si elle n'ont pas t toutes parses
				
				//--------- liste d'AO client
				if (comptServ >= listAOServ.length) {
					if (comptClient < listAOClient.length) {
						for (; comptClient < listAOClient.length; comptClient++) {
							
							SynchroAtomicObject clientAtomicObject = (SynchroAtomicObject)listAOClient[comptClient];
							
							if(clientAtomicObject.getCreationDate() < currentSyncNumber){
								if(clientAtomicObject.getModifSyncNumber() == currentSyncNumber){
									//----- L'object a aussi t modifier sur le terminal donc conflit
									ReplicationSAOEvent eventAO = conflicResolver.resolveDeleteandModified(clientAtomicObject);
									SAOEventList.addReplicationSAOEvent(eventAO);
									
								}else{
									// object supprim sur le serveur  rpercuter cot client
									clientAtomicObject.setModificationType(SynchroAtomicObject.DELETE);
									
									ReplicationSAOEvent eventAO = new ReplicationSAOEvent(clientAtomicObject.getUID());
									eventAO.setUpdateSide(ReplicationSAOEvent.CLIENT_SIDE);
									eventAO.setTypeModif(clientAtomicObject.getModificationType());
									eventAO.setAobject(clientAtomicObject);
									SAOEventList.addReplicationSAOEvent(eventAO);
								}
								
							}else{
								clientAtomicObject.setModificationType(SynchroAtomicObject.ADD);
								
								ReplicationSAOEvent eventAO = new ReplicationSAOEvent(clientAtomicObject.getUID());
								eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
								eventAO.setTypeModif(clientAtomicObject.getModificationType());
								eventAO.setAobject(clientAtomicObject);
								SAOEventList.addReplicationSAOEvent(eventAO);
							}
							
							
						}
					}
					
					//--------- liste d'AO server
				} else { // firstTab not finished ==> second one is finished.
					for (; comptServ < listAOServ.length; comptServ++) {
						
						SynchroAtomicObject serverAtomicObject = (SynchroAtomicObject)listAOServ[comptServ];
						
						if(serverAtomicObject.getCreationDate() < currentSyncNumber){
							if(serverAtomicObject.getModifSyncNumber() > currentSyncNumber){
								//----- L'object a aussi t modifier sur le server donc conflit
								ReplicationSAOEvent eventAO = conflicResolver.resolveReplaceandDeleted(serverAtomicObject);
								SAOEventList.addReplicationSAOEvent(eventAO);
								
							}else{
								//object supprim sur le client  rpercuter cot server
								serverAtomicObject.setModificationType(SynchroAtomicObject.DELETE);
								
								ReplicationSAOEvent eventAO = new ReplicationSAOEvent(serverAtomicObject.getUID());
								eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
								eventAO.setTypeModif(serverAtomicObject.getModificationType());
								eventAO.setAobject(serverAtomicObject);
								SAOEventList.addReplicationSAOEvent(eventAO);
							}
							
						}else{
							serverAtomicObject.setModificationType(SynchroAtomicObject.ADD);
							
							ReplicationSAOEvent eventAO = new ReplicationSAOEvent(serverAtomicObject.getUID());
							eventAO.setUpdateSide(ReplicationSAOEvent.CLIENT_SIDE);
							eventAO.setTypeModif(serverAtomicObject.getModificationType());
							eventAO.setAobject(serverAtomicObject);
							SAOEventList.addReplicationSAOEvent(eventAO);
						}
						

					}
				}
				
				
			} catch (Throwable e) {
				throw new SynchroException(e);
			}
			return SAOEventList;
		}


	  
		// < 0 if p1 < p2, 0 if equal, > 0 if p1 > p2
		public int compareTo(Object AO1, Object AO2) {
			String AoID1 = ((SynchroAtomicObject)AO1).getUID();
			String AoID2 = ((SynchroAtomicObject)AO2).getUID();
	   
			return AoID1.compareTo(AoID2);
		}
	
}
