/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.tellus.synchro.terminal.services.tellus;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import org.openmis.bittorrent.client.ClientBittorrent;
import org.openmis.bittorrent.client.OpenMISClientBittorrent;
import org.openmis.bittorrent.client.OpenMISStorageListener;
import org.openmis.tellus.exception.SIGException;
import org.openmis.tellus.synchro.common.data.DefaultSynchroLayer;
import org.openmis.tellus.synchro.common.data.SynchroGeograhicProfil;
import org.openmis.tellus.synchro.common.data.SynchroProject;
import org.openmis.tellus.synchro.common.xml.ParseXMLProject;
import org.openmis.tellus.synchro.terminal.project.ProjectProfilManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;

/**
 * @author Manuel GOMEZ ROSADO
 *
 */
public abstract class AbsBittorrentSyncFactory extends AbsSyncFactory{
	//------------------------------------------
	private Array arrayClientBitt  = new Array();
	private int count = 0;
	
	public void startBittorrentSynchro() throws SIGException{
		try{

			SynchroGeograhicProfil profil = ProjectProfilManager.getManager().getProfilGeographic();	
			Array projects = profil.getAllSynchroProject();
			
			for(int j=0;j<projects.size();j++){
				SynchroProject project = (SynchroProject)projects.get(j);

				File terminalXML = ProjectProfilManager.getManager().getTerminalXMLProjectFile(project);
				ParseXMLProject parser = new ParseXMLProject(new FileReader(terminalXML));
				
				SynchroAtomicObject[] listAO = parser.parse();
				String directory = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath")+File.separator+project.getUniqueProjectName();
				this.reset();
				for(int i=0;i<listAO.length;i++){
					DefaultSynchroLayer layer = (DefaultSynchroLayer)listAO[i];
					synchronized (layer) {
						if(layer.getFileName().endsWith(".torrent")){
							ClientBittorrent client = new OpenMISClientBittorrent(
									directory+File.separator+layer.getFileName()
									, directory
									);
							arrayClientBitt.add(client);
							count++;
						}
						
					}
				}
			}
			

			//---------- start all client bitt -----
			for(int i=0;i<arrayClientBitt.size();i++){
				ClientBittorrent client = (ClientBittorrent)arrayClientBitt.get(i);
				synchronized (client) {
					client.buildStorage(new SIGBittorrentStorageListener(client, this));
					
					if(client.isCompleteDownload() == false){
						client.startClientTracker();
					}
				}
			}
			//--------------------------------------

		}catch(Exception ex){
			throw new SIGException(ex);
		}
	}
	
	
	public synchronized void removeClientBittorrent(ClientBittorrent client){
		count--;
		if(arrayClientBitt.contains(client)){
			System.out.println(count);
			
			if(count == 0){	//Array of ClientBittorrent is empty, so All client bittorrent has finished.
				System.out.println("TOUT FINI !!");
				try {
					openBittorrentProject();
				} catch (SIGException ex) {
					LogManager.traceError(0, ex);
				}
				this.reset();
			}
			
		}
	}
	
	private void reset(){
		
		for(int i=0;i<arrayClientBitt.size();i++){
			ClientBittorrent client = (ClientBittorrent)arrayClientBitt.get(i);
			synchronized (client) {
				try {
					if(client.getStorageListener() instanceof OpenMISStorageListener){
						OpenMISStorageListener list = (OpenMISStorageListener)client.getStorageListener();
						list.setIgnore(true);
					}
					
					client.stopClientTracker();
				} catch (IOException e) {
					LogManager.traceError(0, e);
				}
			}
		}
		
		count = 0;
		arrayClientBitt.clear();
	}
	
	public abstract void openBittorrentProject()throws SIGException;
}
